@@ -2,8 +2,7 @@
 # SQL-Ledger Accounting
 # Copyright (C) 2000
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
 #  Contributors: Jim Rawlings <jim@your-dba.com>
@@ -39,26 +38,25 @@ sub get_account {
   my $dbh = $form->dbconnect($myconfig);
 
   my $query = qq|SELECT accno, description, charttype, gifi_accno,
-                 category, link
+                 category, link, contra
                  FROM chart
 	         WHERE id = $form->{id}|;
   my $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
   my $ref = $sth->fetchrow_hashref(NAME_lc);
-  
-  foreach my $key (keys %$ref) {
-    $form->{"$key"} = $ref->{"$key"};
-  }
+  for (keys %$ref) { $form->{$_} = $ref->{$_} }
+  $sth->finish;
 
   # get default accounts
-  $query = qq|SELECT inventory_accno_id, income_accno_id, expense_accno_id
+  $query = qq|SELECT inventory_accno_id, income_accno_id, expense_accno_id,
+              fxgain_accno_id, fxloss_accno_id
               FROM defaults|;
   $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
   $ref = $sth->fetchrow_hashref(NAME_lc);
-  map { $form->{$_} = $ref->{$_} } keys %ref;
+  for (keys %$ref) { $form->{$_} = $ref->{$_} }
   $sth->finish;
 
   # check if we have any transactions
@@ -88,20 +86,19 @@ sub save_account {
 		    $form->{AP_tax},
 		    $form->{AP_paid},
 		    $form->{IC},
+		    $form->{IC_income},
 		    $form->{IC_sale},
+		    $form->{IC_expense},
 		    $form->{IC_cogs},
 		    $form->{IC_taxpart},
-		    $form->{IC_income},
-		    $form->{IC_expense},
 		    $form->{IC_taxservice},
-		    $form->{CT_tax}
 		    ) {
      $form->{link} .= "${item}:" if ($item);
   }
   chop $form->{link};
 
   # strip blanks from accno
-  map { $form->{$_} =~ s/( |')//g } qw(accno gifi_accno);
+  for (qw(accno gifi_accno)) { $form->{$_} =~ s/( |')//g }
   
   foreach my $item (qw(accno gifi_accno description)) {
     $form->{$item} =~ s/-(-+)/-/g;
@@ -111,6 +108,8 @@ sub save_account {
   my $query;
   my $sth;
   
+  $form->{contra} *= 1;
+  
   # if we have an id then replace the old record
   if ($form->{id}) {
     $query = qq|UPDATE chart SET
@@ -119,15 +118,17 @@ sub save_account {
 		charttype = '$form->{charttype}',
 		gifi_accno = '$form->{gifi_accno}',
 		category = '$form->{category}',
-		link = '$form->{link}'
+		link = '$form->{link}',
+		contra = '$form->{contra}'
 		WHERE id = $form->{id}|;
   } else {
     $query = qq|INSERT INTO chart 
-                (accno, description, charttype, gifi_accno, category, link)
+                (accno, description, charttype, gifi_accno, category, link,
+		contra)
                 VALUES ('$form->{accno}',|
 		.$dbh->quote($form->{description}).qq|,
 		'$form->{charttype}', '$form->{gifi_accno}',
-		'$form->{category}', '$form->{link}')|;
+		'$form->{category}', '$form->{link}', '$form->{contra}')|;
   }
   $dbh->do($query) || $form->dberror($query);
 
@@ -142,7 +143,7 @@ sub save_account {
     ($chart_id) = $dbh->selectrow_array($query);
   }
 
-  if ($form->{IC_taxpart} || $form->{IC_taxservice} || $form->{CT_tax}) {
+  if ($form->{IC_taxpart} || $form->{IC_taxservice} || $form->{AR_tax} || $form->{AP_tax}) {
    
     # add account if it doesn't exist in tax
     $query = qq|SELECT chart_id
@@ -244,6 +245,7 @@ sub gifi_accounts {
   while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
     push @{ $form->{ALL} }, $ref;
   }
+  $sth->finish;
 
   $dbh->disconnect;
   
@@ -341,6 +343,7 @@ sub warehouses {
   while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
     push @{ $form->{ALL} }, $ref;
   }
+  $sth->finish;
 
   $dbh->disconnect;
   
@@ -428,6 +431,7 @@ sub departments {
   while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
     push @{ $form->{ALL} }, $ref;
   }
+  $sth->finish;
 
   $dbh->disconnect;
   
@@ -446,7 +450,7 @@ sub get_department {
 	         WHERE id = $form->{id}|;
   ($form->{description}, $form->{role}) = $dbh->selectrow_array($query);
   
-  map { $form->{$_} = $ref->{$_} } keys %$ref;
+  for (keys %$ref) { $form->{$_} = $ref->{$_} }
 
   # see if it is in use
   $query = qq|SELECT * FROM dpt_trans
@@ -518,6 +522,7 @@ sub business {
   while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
     push @{ $form->{ALL} }, $ref;
   }
+  $sth->finish;
 
   $dbh->disconnect;
   
@@ -605,6 +610,7 @@ sub sic {
   while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
     push @{ $form->{ALL} }, $ref;
   }
+  $sth->finish;
 
   $dbh->disconnect;
   
@@ -625,10 +631,9 @@ sub get_sic {
   $sth->execute || $form->dberror($query);
 
   my $ref = $sth->fetchrow_hashref(NAME_lc);
-  
-  map { $form->{$_} = $ref->{$_} } keys %$ref;
-
+  for (keys %$ref) { $form->{$_} = $ref->{$_} }
   $sth->finish;
+
   $dbh->disconnect;
 
 }
@@ -702,6 +707,7 @@ sub language {
   while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
     push @{ $form->{ALL} }, $ref;
   }
+  $sth->finish;
 
   $dbh->disconnect;
   
@@ -722,9 +728,7 @@ sub get_language {
   $sth->execute || $form->dberror($query);
 
   my $ref = $sth->fetchrow_hashref(NAME_lc);
-  
-  map { $form->{$_} = $ref->{$_} } keys %$ref;
-
+  for (keys %$ref) { $form->{$_} = $ref->{$_} }
   $sth->finish;
 
   $dbh->disconnect;
@@ -779,7 +783,259 @@ sub delete_language {
 }
 
 
+sub recurring_transactions {
+  my ($self, $myconfig, $form) = @_;
 
+  my $dbh = $form->dbconnect($myconfig);
+
+  my $query = qq|SELECT curr FROM defaults|;
+  my ($defaultcurrency) = $dbh->selectrow_array($query);
+  $defaultcurrency =~ s/:.*//g;
+
+  $form->{sort} ||= "nextdate";
+  my @a = ($form->{sort});
+  my $sortorder = $form->sort_order(\@a);
+  
+  $query = qq|SELECT 'ar' AS module, 'ar' AS transaction, a.invoice,
+                 n.name AS description, a.amount,
+                 s.*, se.formname AS recurringemail,
+                 sp.formname AS recurringprint,
+		 s.nextdate - current_date AS overdue, 'customer' AS vc,
+		 ex.buy AS exchangerate, a.curr,
+		 (s.nextdate IS NULL OR s.nextdate > s.enddate) AS expired
+                 FROM recurring s
+		 JOIN ar a ON (a.id = s.id)
+		 JOIN customer n ON (n.id = a.customer_id)
+                 LEFT JOIN recurringemail se ON (se.id = s.id)
+                 LEFT JOIN recurringprint sp ON (sp.id = s.id)
+		 LEFT JOIN exchangerate ex ON
+		      (ex.curr = a.curr AND a.transdate = ex.transdate)
+
+	 UNION
+
+                 SELECT 'ap' AS module, 'ap' AS transaction, a.invoice,
+		 n.name AS description, a.amount,
+                 s.*, se.formname AS recurringemail,
+                 sp.formname AS recurringprint,
+		 s.nextdate - current_date AS overdue, 'vendor' AS vc,
+		 ex.sell AS exchangerate, a.curr,
+		 (s.nextdate IS NULL OR s.nextdate > s.enddate) AS expired
+                 FROM recurring s
+		 JOIN ap a ON (a.id = s.id)
+		 JOIN vendor n ON (n.id = a.vendor_id)
+                 LEFT JOIN recurringemail se ON (se.id = s.id)
+                 LEFT JOIN recurringprint sp ON (sp.id = s.id)
+		 LEFT JOIN exchangerate ex ON
+		      (ex.curr = a.curr AND a.transdate = ex.transdate)
+	
+	 UNION
+
+                 SELECT 'gl' AS module, 'gl' AS transaction, FALSE AS invoice,
+		 a.description,
+		 (SELECT SUM(ac.amount) FROM acc_trans ac WHERE ac.trans_id = a.id AND ac.amount > 0) AS amount,
+                 s.*, se.formname AS recurringemail,
+                 sp.formname AS recurringprint,
+		 s.nextdate - current_date AS overdue, '' AS vc,
+		 '1' AS exchangerate, '$defaultcurrency' AS curr,
+		 (s.nextdate IS NULL OR s.nextdate > s.enddate) AS expired
+                 FROM recurring s
+		 JOIN gl a ON (a.id = s.id)
+                 LEFT JOIN recurringemail se ON (se.id = s.id)
+                 LEFT JOIN recurringprint sp ON (sp.id = s.id)
+	
+	UNION
+
+                 SELECT 'oe' AS module, 'so' AS transaction, FALSE AS invoice,
+		 n.name AS description, a.amount,
+                 s.*, se.formname AS recurringemail,
+                 sp.formname AS recurringprint,
+		 s.nextdate - current_date AS overdue, 'customer' AS vc,
+		 ex.buy AS exchangerate, a.curr,
+		 (s.nextdate IS NULL OR s.nextdate > s.enddate) AS expired
+                 FROM recurring s
+		 JOIN oe a ON (a.id = s.id)
+		 JOIN customer n ON (n.id = a.customer_id)
+                 LEFT JOIN recurringemail se ON (se.id = s.id)
+                 LEFT JOIN recurringprint sp ON (sp.id = s.id)
+		 LEFT JOIN exchangerate ex ON
+		      (ex.curr = a.curr AND a.transdate = ex.transdate)
+		 WHERE a.quotation = '0'
+		 
+	UNION
+
+                 SELECT 'oe' AS module, 'po' AS transaction, FALSE AS invoice,
+		 n.name AS description, a.amount,
+                 s.*, se.formname AS recurringemail,
+                 sp.formname AS recurringprint,
+		 s.nextdate - current_date AS overdue, 'vendor' AS vc,
+		 ex.sell AS exchangerate, a.curr,
+		 (s.nextdate IS NULL OR s.nextdate > s.enddate) AS expired
+                 FROM recurring s
+		 JOIN oe a ON (a.id = s.id)
+		 JOIN vendor n ON (n.id = a.vendor_id)
+                 LEFT JOIN recurringemail se ON (se.id = s.id)
+                 LEFT JOIN recurringprint sp ON (sp.id = s.id)
+		 LEFT JOIN exchangerate ex ON
+		      (ex.curr = a.curr AND a.transdate = ex.transdate)
+		 WHERE a.quotation = '0'
+		 
+		 ORDER BY $sortorder|;
+
+  my $sth = $dbh->prepare($query);
+  $sth->execute || $form->dberror($query);
+
+  my $id;
+  my $transaction;
+  my %e = ();
+  my %p = ();
+  
+  while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
+
+    $ref->{exchangerate} ||= 1;
+    
+    if ($ref->{id} != $id) {
+
+      if (%e) {
+	$form->{transactions}{$transaction}->[$i]->{recurringemail} = "";
+	for (keys %e) { $form->{transactions}{$transaction}->[$i]->{recurringemail} .= "${_}:" }
+	chop $form->{transactions}{$transaction}->[$i]->{recurringemail};
+      }
+      if (%p) {
+	$form->{transactions}{$transaction}->[$i]->{recurringprint} = "";
+	for (keys %p) { $form->{transactions}{$transaction}->[$i]->{recurringprint} .= "${_}:" }
+	chop $form->{transactions}{$transaction}->[$i]->{recurringprint};
+      }
+	 
+      %e = ();
+      %p = ();
+      
+      push @{ $form->{transactions}{$ref->{transaction}} }, $ref;
+      
+      $id = $ref->{id};
+      $i = $#{ $form->{transactions}{$ref->{transaction}} };
+
+    }
+    
+    $transaction = $ref->{transaction};
+   
+    $e{$ref->{recurringemail}} = 1 if $ref->{recurringemail};
+    $p{$ref->{recurringprint}} = 1 if $ref->{recurringprint};
+ 
+  }
+  $sth->finish;
+
+  # this is for the last row
+  if (%e) {
+    $form->{transactions}{$transaction}->[$i]->{recurringemail} = "";
+    for (keys %e) { $form->{transactions}{$transaction}->[$i]->{recurringemail} .= "${_}:" }
+    chop $form->{transactions}{$transaction}->[$i]->{recurringemail};
+  }
+  if (%p) {
+    $form->{transactions}{$transaction}->[$i]->{recurringprint} = "";
+    for (keys %p) { $form->{transactions}{$transaction}->[$i]->{recurringprint} .= "${_}:" }
+    chop $form->{transactions}{$transaction}->[$i]->{recurringprint};
+  }
+
+
+  $dbh->disconnect;
+  
+}
+
+sub recurring_details {
+  my ($self, $myconfig, $form, $id) = @_;
+
+  my $dbh = $form->dbconnect($myconfig);
+
+  my $query = qq~SELECT s.*, ar.id AS arid, ar.invoice AS arinvoice,
+                 ap.id AS apid, ap.invoice AS apinvoice,
+		 ar.duedate - ar.transdate AS overdue,
+		 ar.datepaid - ar.transdate AS paid,
+		 oe.reqdate - oe.transdate AS req,
+		 oe.id AS oeid, oe.customer_id, oe.vendor_id
+                 FROM recurring s
+                 LEFT JOIN ar ON (ar.id = s.id)
+		 LEFT JOIN ap ON (ap.id = s.id)
+		 LEFT JOIN oe ON (oe.id = s.id)
+                 WHERE s.id = $id~;
+  my $sth = $dbh->prepare($query);
+  $sth->execute || $form->dberror($query);
+
+  my $ref = $sth->fetchrow_hashref(NAME_lc);
+  $form->{vc} = "customer" if $ref->{customer_id};
+  $form->{vc} = "vendor" if $ref->{vendor_id};
+  for (keys %$ref) { $form->{$_} = $ref->{$_} }
+  $sth->finish;
+  
+  $form->{invoice} = ($form->{arid} && $form->{arinvoice});
+  $form->{invoice} = ($form->{apid} && $form->{apinvoice}) unless $form->{invoice};
+  
+  $query = qq|SELECT * FROM recurringemail
+              WHERE id = $id|;
+  $sth = $dbh->prepare($query);
+  $sth->execute || $form->dberror($query);
+
+  $form->{recurringemail} = "";
+  while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
+    $form->{recurringemail} .= "$ref->{formname}:$ref->{format}:";
+    $form->{message} = $ref->{message};
+  }
+  $sth->finish;
+  
+  $query = qq|SELECT * FROM recurringprint
+              WHERE id = $id|;
+  $sth = $dbh->prepare($query);
+  $sth->execute || $form->dberror($query);
+
+  $form->{recurringprint} = "";
+  while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
+    $form->{recurringprint} .= "$ref->{formname}:$ref->{format}:$ref->{printer}:";
+  }
+  $sth->finish;
+  
+  chop $form->{recurringemail};
+  chop $form->{recurringprint};
+  
+  for (qw(arinvoice apinvoice)) { delete $form->{$_} }
+
+  $dbh->disconnect;
+  
+}
+
+
+sub update_recurring {
+  my ($self, $myconfig, $form, $id) = @_;
+
+  my $dbh = $form->dbconnect($myconfig);
+
+  my $query = qq|SELECT nextdate, repeat, unit
+                 FROM recurring
+		 WHERE id = $id|;
+  my ($nextdate, $repeat, $unit) = $dbh->selectrow_array($query);
+  
+  my %advance = ( 'Pg' => "(date '$nextdate' + interval '$repeat $unit')",
+                  'DB2' => qq|(date ('$nextdate') + "$repeat $unit")|,
+		 );
+  $interval{Oracle} = $interval{PgPP} = $interval{Pg};
+
+  # check if it is the last date
+  $query = qq|SELECT $advance{$myconfig->{dbdriver}} > enddate
+              FROM recurring
+	      WHERE id = $id|;
+  my ($last_repeat) = $dbh->selectrow_array($query);
+  if ($last_repeat) {
+    $advance{$myconfig->{dbdriver}} = "NULL";
+  }
+  
+  $query = qq|UPDATE recurring SET
+              nextdate = $advance{$myconfig->{dbdriver}}
+	      WHERE id = $id|;
+  $dbh->do($query) || $form->dberror($query);
+  
+  $dbh->disconnect;
+
+}
+
+ 
 sub load_template {
   my ($self, $form) = @_;
   
@@ -800,7 +1056,7 @@ sub save_template {
   open(TEMPLATE, ">$form->{file}") or $form->error("$form->{file} : $!");
   
   # strip 
-  $form->{body} =~ s/\r\n/\n/g;
+  $form->{body} =~ s/\r//g;
   print TEMPLATE $form->{body};
 
   close(TEMPLATE);
@@ -848,11 +1104,11 @@ sub save_preferences {
 sub save_defaults {
   my ($self, $myconfig, $form) = @_;
 
-  map { ($form->{$_}) = split /--/, $form->{$_} } qw(inventory_accno income_accno expense_accno fxgain_accno fxloss_accno);
+  for (qw(inventory_accno income_accno expense_accno fxgain_accno fxloss_accno)) { ($form->{$_}) = split /--/, $form->{$_} }
   
   my @a;
   $form->{curr} =~ s/ //g;
-  map { push(@a, uc pack "A3", $_) if $_ } split /:/, $form->{curr};
+  for (split /:/, $form->{curr}) { push(@a, uc pack "A3", $_) if $_ }
   $form->{curr} = join ':', @a;
     
   # connect to database
@@ -875,6 +1131,7 @@ sub save_defaults {
 	         fxloss_accno_id =
 		     (SELECT id FROM chart
 		                WHERE accno = '$form->{fxloss_accno}'),
+	         glnumber = '$form->{glnumber}',
 	         sinumber = '$form->{sinumber}',
 		 vinumber = '$form->{vinumber}',
 	         sonumber = '$form->{sonumber}',
@@ -920,16 +1177,16 @@ sub defaultaccounts {
   $sth->execute || $form->dberror($query);
   
   $form->{defaults} = $sth->fetchrow_hashref(NAME_lc);
-  $form->{defaults}{IC} = $form->{defaults}{inventory_accno_id};
+  $form->{defaults}{IC_inventory} = $form->{defaults}{inventory_accno_id};
   $form->{defaults}{IC_income} = $form->{defaults}{income_accno_id};
+  $form->{defaults}{IC_sale} = $form->{defaults}{income_accno_id};
   $form->{defaults}{IC_expense} = $form->{defaults}{expense_accno_id};
+  $form->{defaults}{IC_cogs} = $form->{defaults}{expense_accno_id};
   $form->{defaults}{FX_gain} = $form->{defaults}{fxgain_accno_id};
   $form->{defaults}{FX_loss} = $form->{defaults}{fxloss_accno_id};
   
-  
   $sth->finish;
 
-
   $query = qq|SELECT id, accno, description, link
               FROM chart
               WHERE link LIKE '%IC%'
@@ -937,6 +1194,7 @@ sub defaultaccounts {
   $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
+  my $nkey;
   while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
     foreach my $key (split(/:/, $ref->{link})) {
       if ($key =~ /IC/) {
@@ -1019,7 +1277,7 @@ sub backup {
   $t[4] = substr("0$t[4]", -2);
 
   my $boundary = time;
-  my $tmpfile = "$userspath/$boundary.$myconfig->{dbname}-$form->{dbversion}-$t[5]$t[4]$t[3].sql";
+  my $tmpfile = "$userspath/$boundary.$myconfig->{dbname}-$form->{dbversion}-$t[5]$t[4]$t[3]";
   my $out = $form->{OUT};
   $form->{OUT} = ">$tmpfile";
 
@@ -1095,11 +1353,13 @@ sub backup {
 
       if (/create sequence (\w+)/i) {
 	push @sequences, $1;
+	next;
       }
 
       if (/end function/i) {
 	push @functions, $_;
 	$function = 0;
+	$temp = 0;
 	next;
       }
 
@@ -1153,12 +1413,9 @@ sub backup {
 |;
 
  
-  my $restrict = ($myconfig->{dbdriver} eq 'DB2') ? "RESTRICT" : "";
-  
   @tables = grep !/^temp/, @tables;
   # drop tables and sequences
-  map { print OUT qq|DROP TABLE $_;\n| } @tables;
-  map { print OUT qq|DROP SEQUENCE $_ $restrict;\n| } @sequences;
+  for (@tables) { print OUT qq|DROP TABLE $_;\n| }
 
   print OUT "--\n";
   
@@ -1208,7 +1465,9 @@ $myconfig->{dboptions};
     $sth->finish;
   }
 
-
+ 
+  my $restrict = ($myconfig->{dbdriver} eq 'DB2') ? "RESTRICT" : "";
+ 
   # create sequences and triggers
   foreach $item (@sequences) {
     if ($myconfig->{dbdriver} eq 'DB2') {
@@ -1217,10 +1476,7 @@ $myconfig->{dboptions};
       $query = qq|SELECT last_value FROM $item|;
     }
     
-    $sth = $dbh->prepare($query);
-    $sth->execute || $form->dberror($query);
-    my ($id) = $sth->fetchrow_array;
-    $sth->finish;
+    my ($id) = $dbh->selectrow_array($query);
     $id++;
   
     print OUT qq|--
@@ -1236,10 +1492,10 @@ DROP SEQUENCE $item $restrict;\n|;
   print OUT "--\n";
   
   # functions
-  map { print OUT $_ } @functions;
+  for (@functions) { print OUT $_ }
 
   # triggers
-  map { print OUT $_ } @triggers;
+  for (@triggers) { print OUT $_ }
 
   # add the index files
   open(FH, "sql/$myconfig->{dbdriver}-indices.sql");
@@ -1273,12 +1529,12 @@ DROP SEQUENCE $item $restrict;\n|;
 
     $mail->{to} = qq|"$myconfig->{name}" <$myconfig->{email}>|;
     $mail->{from} = qq|"$myconfig->{name}" <$myconfig->{email}>|;
-    $mail->{subject} = "SQL-Ledger Backup / $myconfig->{dbname}-$form->{dbversion}-$t[5]$t[4]$t[3].sql$suffix";
+    $mail->{subject} = "SQL-Ledger Backup / $myconfig->{dbname}-$form->{dbversion}-$t[5]$t[4]$t[3]$suffix";
     @{ $mail->{attachments} } = ($tmpfile);
     $mail->{version} = $form->{version};
     $mail->{fileid} = "$boundary.";
 
-    $myconfig->{signature} =~ s/\\n/\r\n/g;
+    $myconfig->{signature} =~ s/\\n/\n/g;
     $mail->{message} = "-- \n$myconfig->{signature}";
     
     $err = $mail->send($out);
@@ -1290,7 +1546,7 @@ DROP SEQUENCE $item $restrict;\n|;
     open(OUT, ">-") or $form->error("STDOUT : $!");
    
     print OUT qq|Content-Type: application/file;
-Content-Disposition: attachment; filename="$myconfig->{dbname}-$form->{dbversion}-$t[5]$t[4]$t[3].sql$suffix"
+Content-Disposition: attachment; filename="$myconfig->{dbname}-$form->{dbversion}-$t[5]$t[4]$t[3]$suffix"
 
 |;
 
@@ -1401,7 +1657,7 @@ sub post_yearend {
 
   my $query;
   my $uid = time;
-  $uid .= $form->{login};
+  $uid .= $$;
 
   $query = qq|INSERT INTO gl (reference, employee_id)
 	      VALUES ('$uid', (SELECT id FROM employee
@@ -1431,16 +1687,16 @@ sub post_yearend {
     ($accno) = split(/--/, $form->{"accno_$i"});
     $amount = 0;
 
-    if ($form->{"credit_$i"} != 0) {
+    if ($form->{"credit_$i"}) {
       $amount = $form->{"credit_$i"};
     }
-    if ($form->{"debit_$i"} != 0) {
+    if ($form->{"debit_$i"}) {
       $amount = $form->{"debit_$i"} * -1;
     }
 
 
     # if there is an amount, add the record
-    if ($amount != 0) {
+    if ($amount) {
       $query = qq|INSERT INTO acc_trans (trans_id, chart_id, amount, transdate,
                   source)
 		  VALUES
