@@ -2,8 +2,7 @@
 # SQL-Ledger Accounting
 # Copyright (C) 2001
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
 #  Contributors:
@@ -161,22 +160,18 @@ sub all_transactions {
 
   if ($form->{accno} || $form->{gifi_accno}) {
     # get category for account
-    $query = qq|SELECT category, link
+    $query = qq|SELECT description, category, link, contra
                 FROM chart
 		WHERE accno = '$form->{accno}'|;
 
     if ($form->{accounttype} eq 'gifi') {
-      $query = qq|SELECT category, link
+      $query = qq|SELECT description, category, link, contra
                 FROM chart
 		WHERE gifi_accno = '$form->{gifi_accno}'
 		AND charttype = 'A'|;
     }
 
-    $sth = $dbh->prepare($query);
-
-    $sth->execute || $form->dberror($query);
-    ($form->{category}, $form->{link}) = $sth->fetchrow_array;
-    $sth->finish;
+    ($form->{description}, $form->{category}, $form->{link}, $form->{contra}) = $dbh->selectrow_array($query);
     
     if ($form->{fromdate}) {
 
@@ -191,58 +186,6 @@ sub all_transactions {
 		  $project
 		  |;
 
-      if ($project_id) {
-
-	$query .= qq|
-
-	       UNION
-
-	          SELECT SUM(ac.sellprice * ac.qty)
-		  FROM invoice ac
-		  JOIN ar a ON (ac.trans_id = a.id)
-		  JOIN parts p ON (ac.parts_id = p.id)
-		  JOIN chart c ON (p.income_accno_id = c.id)
-		  $dpt_join
-		  WHERE c.accno = '$form->{accno}'
-		  AND a.transdate < '$form->{fromdate}'
-		  AND c.category = 'I'
-		  $dpt_where
-		  $project
-
-	       UNION
-
-	          SELECT SUM(ac.sellprice * ac.qty)
-		  FROM invoice ac
-		  JOIN ap a ON (ac.trans_id = a.id)
-		  JOIN parts p ON (ac.parts_id = p.id)
-		  JOIN chart c ON (p.expense_accno_id = c.id)
-		  $dpt_join
-		  WHERE c.accno = '$form->{accno}'
-		  AND p.inventory_accno_id IS NULL
-		  AND p.assembly = '0'
-		  AND a.transdate < '$form->{fromdate}'
-		  AND c.category = 'E'
-		  $dpt_where
-		  $project
-
-	       UNION
-
-	          SELECT SUM(ac.sellprice * ac.allocated) * -1
-		  FROM invoice ac
-		  JOIN ap a ON (ac.trans_id = a.id)
-		  JOIN parts p ON (ac.parts_id = p.id)
-		  JOIN chart c ON (p.expense_accno_id = c.id)
-		  $dpt_join
-		  WHERE c.accno = '$form->{accno}'
-		  AND ac.assemblyitem = '0'
-		  AND a.transdate < '$form->{fromdate}'
-		  AND c.category = 'E'
-		  $dpt_where
-		  $project
-		  |;
-
-      }
-
       if ($form->{accounttype} eq 'gifi') {
         $query = qq|SELECT SUM(ac.amount)
 		  FROM acc_trans ac
@@ -254,57 +197,6 @@ sub all_transactions {
 		  $project
 		  |;
 		  
-	if ($project_id) {
-
-	  $query .= qq|
-
-	       UNION
-
-	          SELECT SUM(ac.sellprice * ac.qty)
-		  FROM invoice ac
-		  JOIN ar a ON (ac.trans_id = a.id)
-		  JOIN parts p ON (ac.parts_id = p.id)
-		  JOIN chart c ON (p.income_accno_id = c.id)
-		  $dpt_join
-		  WHERE c.gifi_accno = '$form->{gifi_accno}'
-		  AND a.transdate < '$form->{fromdate}'
-		  AND c.category = 'I'
-		  $dpt_where
-		  $project
-
-	       UNION
-
-	          SELECT SUM(ac.sellprice * ac.qty)
-		  FROM invoice ac
-		  JOIN ap a ON (ac.trans_id = a.id)
-		  JOIN parts p ON (ac.parts_id = p.id)
-		  JOIN chart c ON (p.expense_accno_id = c.id)
-		  $dpt_join
-		  WHERE c.gifi_accno = '$form->{gifi_accno}'
-		  AND p.inventory_accno_id IS NULL
-		  AND p.assembly = '0'
-		  AND a.transdate < '$form->{fromdate}'
-		  AND c.category = 'E'
-		  $dpt_where
-		  $project
-
-	       UNION
-
-	          SELECT SUM(ac.sellprice * ac.allocated) * -1
-		  FROM invoice ac
-		  JOIN ap a ON (ac.trans_id = a.id)
-		  JOIN parts p ON (ac.parts_id = p.id)
-		  JOIN chart c ON (p.expense_accno_id = c.id)
-		  $dpt_join
-		  WHERE c.gifi_accno = '$form->{gifi_accno}'
-		  AND ac.assemblyitem = '0'
-		  AND a.transdate < '$form->{fromdate}'
-		  AND c.category = 'E'
-		  $dpt_where
-		  $project
-		  |;
-
-	}
       }
       
       ($form->{balance}) = $dbh->selectrow_array($query);
@@ -321,7 +213,7 @@ sub all_transactions {
     $query .= qq|$union
                  SELECT a.id, a.reference, a.description, ac.transdate,
 	         $false AS invoice, ac.amount, 'gl' as module, ac.cleared,
-		 '' AS till
+		 '' AS till, ac.chart_id
 		 FROM gl a
 		 JOIN acc_trans ac ON (ac.trans_id = a.id)
 		 $dpt_join
@@ -335,7 +227,7 @@ sub all_transactions {
       
                  SELECT a.id, a.invnumber, c.name, ac.transdate,
 	         a.invoice, ac.amount, 'ar' as module, ac.cleared,
-		 a.till
+		 a.till, ac.chart_id
 		 FROM ar a
 		 JOIN acc_trans ac ON (ac.trans_id = a.id)
 		 JOIN customer c ON (a.customer_id = c.id)
@@ -350,7 +242,7 @@ sub all_transactions {
       
                  SELECT a.id, a.invnumber, v.name, ac.transdate,
 	         a.invoice, ac.amount, 'ap' as module, ac.cleared,
-		 a.till
+		 a.till, ac.chart_id
 		 FROM ap a
 		 JOIN acc_trans ac ON (ac.trans_id = a.id)
 		 JOIN vendor v ON (a.vendor_id = v.id)
@@ -362,78 +254,7 @@ sub all_transactions {
 		 $project
 		 |;
 
-    if ($project_id) {
-
-      $fromdate_where =~ s/ac\./a\./;
-      $todate_where =~ s/ac\./a\./;
-      
-      $query .= qq|
-
-             UNION ALL
-      
-             -- sold items
-	     
-                 SELECT a.id, a.invnumber, c.name, a.transdate,
-	         a.invoice, ac.sellprice * ac.qty, 'ar' as module, '0' AS cleared,
-		 a.till
-		 FROM ar a
-		 JOIN invoice ac ON (ac.trans_id = a.id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN customer c ON (a.customer_id = c.id)
-		 $dpt_join
-		 WHERE p.income_accno_id = $id
-		 $fromdate_where
-		 $todate_where
-		 $dpt_where
-		 $project
-      
-             UNION ALL
-             
-	     -- bought services
-	     
-                 SELECT a.id, a.invnumber, v.name, a.transdate,
-	         a.invoice, ac.sellprice * ac.qty, 'ap' as module, '0' AS cleared,
-		 a.till
-		 FROM ap a
-		 JOIN invoice ac ON (ac.trans_id = a.id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN vendor v ON (a.vendor_id = v.id)
-		 $dpt_join
-		 WHERE p.expense_accno_id = $id
-		 AND p.inventory_accno_id IS NULL
-		 AND p.assembly = '0'
-		 $fromdate_where
-		 $todate_where
-		 $dpt_where
-		 $project
-		 
-             UNION ALL
-
-	     -- cogs
-      
-                 SELECT a.id, a.invnumber, v.name, a.transdate,
-	         a.invoice, ac.sellprice * ac.allocated * -1, 'ap' as module, '0' AS cleared,
-		 a.till
-		 FROM ap a
-		 JOIN invoice ac ON (ac.trans_id = a.id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN vendor v ON (a.vendor_id = v.id)
-		 $dpt_join
-		 WHERE p.expense_accno_id = $id
-		 AND ac.assemblyitem = '0'
-		 $fromdate_where
-		 $todate_where
-		 $dpt_where
-		 $project
-
-		 |;
-		 
-      $fromdate_where =~ s/a\./ac\./;
-      $todate_where =~ s/a\./ac\./;
- 
-    }
-		 
-      $union = qq|
+    $union = qq|
              UNION ALL
                  |;
   }
@@ -444,6 +265,24 @@ sub all_transactions {
   $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
+  $query = qq|SELECT c.id, c.accno FROM chart c
+              JOIN acc_trans ac ON (ac.chart_id = c.id)
+              WHERE ac.amount >= 0
+	      AND (c.link = 'AR' OR c.link = 'AP')
+	      AND ac.trans_id = ?|;
+  my $dr = $dbh->prepare($query) || $form->dberror($query);
+  
+  $query = qq|SELECT c.id, c.accno FROM chart c
+              JOIN acc_trans ac ON (ac.chart_id = c.id)
+              WHERE ac.amount < 0
+	      AND (c.link = 'AR' OR c.link = 'AP')
+	      AND ac.trans_id = ?|;
+  my $cr = $dbh->prepare($query) || $form->dberror($query);
+  
+  my $accno;
+  my $chart_id;
+  my %accno;
+  
   while (my $ca = $sth->fetchrow_hashref(NAME_lc)) {
     
     # gl
@@ -464,12 +303,33 @@ sub all_transactions {
     }
 
     if ($ca->{amount}) {
+      %accno = ();
+
       if ($ca->{amount} < 0) {
 	$ca->{debit} = $ca->{amount} * -1;
 	$ca->{credit} = 0;
+	$dr->execute($ca->{id});
+	$ca->{accno} = ();
+	while (($chart_id, $accno) = $dr->fetchrow_array) {
+	  $accno{$accno} = 1 if $chart_id ne $ca->{chart_id};
+	}
+	$dr->finish;
+	
+	for (sort keys %accno) { push @{ $ca->{accno} }, "$_ " }
+
       } else {
 	$ca->{credit} = $ca->{amount};
 	$ca->{debit} = 0;
+	
+	$cr->execute($ca->{id});
+	$ca->{accno} = ();
+	while (($chart_id, $accno) = $cr->fetchrow_array) {
+	  $accno{$accno} = 1 if $chart_id ne $ca->{chart_id};
+	}
+	$cr->finish;
+
+	for (keys %accno) { push @{ $ca->{accno} }, "$_ " }
+
       }
 
       push @{ $form->{CA} }, $ca;
