@@ -2,8 +2,7 @@
 # SQL-Ledger Accounting
 # Copyright (C) 2003
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
 #  Contributors:
@@ -84,6 +83,23 @@ sub paymentaccounts {
               FROM defaults|;
   ($form->{currencies}, $form->{closedto}, $form->{datepaid}) = $dbh->selectrow_array($query);
 
+  if ($form->{payment} eq 'payments') {
+    # get language codes
+    $query = qq|SELECT *
+		FROM language
+		ORDER BY 2|;
+    $sth = $dbh->prepare($query);
+    $sth->execute || $self->dberror($query);
+
+    $form->{all_language} = ();
+    while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
+      push @{ $form->{all_language} }, $ref;
+    }
+    $sth->finish;
+
+    $form->all_departments($myconfig, $dbh, $form->{vc});
+  }
+  
   $dbh->disconnect;
 
 }
@@ -103,43 +119,33 @@ sub get_openvc {
 
   my $sth;
   my $ref;
+  my $i = 0;
 
+  my $where = qq|WHERE a.$form->{vc}_id = ct.id
+                 AND a.amount != a.paid|;
+
+  if ($form->{$form->{vc}}) {
+    my $var = $form->like(lc $form->{$form->{vc}});
+    $where .= " AND lower(name) LIKE '$var'";
+  }
+    
   # build selection list
-  if ($count < $myconfig->{vclimit}) {
-    $query = qq|SELECT DISTINCT ct.id, ct.name
-                FROM $form->{vc} ct, $arap a
-		WHERE a.$form->{vc}_id = ct.id
-		AND a.amount != a.paid
-		ORDER BY name|;
-    $sth = $dbh->prepare($query);
-    $sth->execute || $form->dberror($query);
-
-    while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
-      push @{ $form->{"all_$form->{vc}"} }, $ref;
-    }
-
-    $sth->finish;
-
-  }
-
-  if ($form->{ARAP} eq 'AR') {
-    $query = qq|SELECT id, description
-                FROM department
-		WHERE role = 'P'
-		ORDER BY 2|;
-  } else {
-    $query = qq|SELECT id, description
-                FROM department
-		ORDER BY 2|;
-  }
+  $query = qq|SELECT DISTINCT ct.*
+	      FROM $form->{vc} ct, $arap a
+	      $where
+	      ORDER BY name|;
   $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
   while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
-    push @{ $form->{all_departments} }, $ref;
+    $i++;
+    push @{ $form->{name_list} }, $ref;
   }
+
   $sth->finish;
 
+  $form->all_departments($myconfig, $dbh, $form->{vc});
+  
   # get language codes
   $query = qq|SELECT *
               FROM language
@@ -147,21 +153,24 @@ sub get_openvc {
   $sth = $dbh->prepare($query);
   $sth->execute || $self->dberror($query);
 
-  $form->{all_languages} = ();
+  $form->{all_language} = ();
   while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
-    push @{ $form->{all_languages} }, $ref;
+    push @{ $form->{all_language} }, $ref;
   }
   $sth->finish;
 
   # get currency for first name
-  if ($form->{"all_$form->{vc}"}) {
+  if (@{ $form->{name_list} }) {
     $query = qq|SELECT curr FROM $form->{vc}
-		WHERE id = $form->{"all_$form->{vc}"}->[0]->{id}|;
+		WHERE id = $form->{name_list}->[0]->{id}|;
     ($form->{currency}) = $dbh->selectrow_array($query);
+    $form->{currency} ||= $form->{defaultcurrency};
   }
 
   $dbh->disconnect;
 
+  $i;
+
 }
 
 
@@ -174,10 +183,14 @@ sub get_openinvoices {
   # connect to database
   my $dbh = $form->dbconnect($myconfig);
  
-  my $where = qq|WHERE $form->{vc}_id = $form->{"$form->{vc}_id"}
-                 AND curr = '$form->{currency}'
-	         AND amount != paid|;
+  my $where = qq|WHERE a.$form->{vc}_id = $form->{"$form->{vc}_id"}
+	         AND a.amount != a.paid|;
+
+  $where .= qq|
+                 AND a.curr = '$form->{currency}'| if $form->{currency};
   
+  my $sortorder = "transdate, invnumber";
+
   my ($buysell);
   if ($form->{vc} eq 'customer') {
     $buysell = "buy";
@@ -185,22 +198,56 @@ sub get_openinvoices {
     $buysell = "sell";
   }
   
+  if ($form->{payment} eq 'payments') {
+    $where = qq|WHERE a.amount != a.paid|;
+    $where .= qq| AND a.curr = '$form->{currency}'| if $form->{currency};
+
+    if ($form->{duedatefrom}) {
+      $where .= qq|
+                AND a.duedate >= '$form->{duedatefrom}'|;
+    }
+    if ($form->{duedateto}) {
+      $where .= qq|
+                AND a.duedate <= '$form->{duedateto}'|;
+    }
+    $sortorder = "name, transdate";
+  }
+
+ 
   ($null, $department_id) = split /--/, $form->{department};
   if ($department_id) {
     $where .= qq|
-                 AND department_id = $department_id|;
+                 AND a.department_id = $department_id|;
   }
 
-  my $query = qq|SELECT id, invnumber, transdate, amount, paid, curr
-	         FROM $form->{arap}
+  my $query = qq|SELECT a.id, a.invnumber, a.transdate, a.amount, a.paid,
+                 a.curr, c.name, a.$form->{vc}_id, c.language_code
+	         FROM $form->{arap} a
+		 JOIN $form->{vc} c ON (c.id = a.$form->{vc}_id)
 		 $where
-		 ORDER BY transdate, invnumber|;
+		 ORDER BY $sortorder|;
   my $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
+  $query = qq|SELECT s.spoolfile
+              FROM status s
+	      WHERE s.formname = '$form->{formname}'
+	      AND s.trans_id = ?|;
+  my $vth = $dbh->prepare($query);
+
+  my $spoolfile;
   while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
     # if this is a foreign currency transaction get exchangerate
     $ref->{exchangerate} = $form->get_exchangerate($dbh, $ref->{curr}, $ref->{transdate}, $buysell) if ($form->{currency} ne $form->{defaultcurrency});
+    
+    $vth->execute($ref->{id});
+    $ref->{queue} = "";
+    while (($spoolfile) = $vth->fetchrow_array) {
+      $ref->{queued} .= "$form->{formname} $spoolfile ";
+    }
+    $vth->finish;
+    $ref->{queued} =~ s/ +$//g;
+    
     push @{ $form->{PR} }, $ref;
   }
   
@@ -211,7 +258,7 @@ sub get_openinvoices {
 
 
 
-sub process_payment {
+sub post_payment {
   my ($self, $myconfig, $form) = @_;
 
   # connect to database, turn AutoCommit off
@@ -266,11 +313,6 @@ sub process_payment {
   
   my $paymentamount = $form->parse_amount($myconfig, $form->{amount});
   
-  my $null;
-  ($null, $form->{department_id}) = split /--/, $form->{department};
-  $form->{department_id} *= 1;
-
-
   # query to retrieve paid amount
   $query = qq|SELECT paid FROM $form->{arap}
               WHERE id = ?
@@ -329,19 +371,21 @@ sub process_payment {
       # add exchangerate difference if currency ne defaultcurrency
       $amount = $form->round_amount($form->{"paid_$i"} * ($form->{exchangerate} - 1), 2);
 
-      if ($amount != 0) {
+      if ($amount) {
         # exchangerate difference
 	$query = qq|INSERT INTO acc_trans (trans_id, chart_id, transdate,
-		    amount, cleared, fx_transaction)
+		    amount, cleared, fx_transaction, source, memo)
 		    VALUES ($form->{"id_$i"},
 		           (SELECT id FROM chart
 			    WHERE accno = '$paymentaccno'),
-		  '$form->{datepaid}', $amount * $ml * -1, '0', '1')|;
+		  '$form->{datepaid}', $amount * $ml * -1, '0', '1', |
+		  .$dbh->quote($form->{source}).qq|, |
+		  .$dbh->quote($form->{memo}).qq|)|;
 	$dbh->do($query) || $form->dberror($query);
 
         # gain/loss
-	$amount = $form->round_amount($form->{"paid_$i"} * ($exchangerate - $form->{exchangerate}) * $ml * -1, 2);
-	if ($amount != 0) {
+	$amount = ($form->round_amount($form->{"paid_$i"} * $exchangerate,2) - $form->round_amount($form->{"paid_$i"} * $form->{exchangerate},2)) * $ml * -1;
+	if ($amount) {
 	  my $accno_id = ($amount > 0) ? $fxgain_accno_id : $fxloss_accno_id;
 	  $query = qq|INSERT INTO acc_trans (trans_id, chart_id, transdate,
 		      amount, cleared, fx_transaction)
@@ -379,7 +423,7 @@ sub process_payment {
 
 
   # record a AR/AP with a payment
-  if ($form->round_amount($paymentamount, 2) != 0) {
+  if ($form->round_amount($paymentamount, 2)) {
     $form->{invnumber} = "";
     OP::overpayment("", $myconfig, $form, $dbh, $paymentamount, $ml, 1);
   }
@@ -392,5 +436,199 @@ sub process_payment {
 }
 
 
+sub post_payments {
+  my ($self, $myconfig, $form) = @_;
+
+  # connect to database, turn AutoCommit off
+  my $dbh = $form->dbconnect_noauto($myconfig);
+
+  my $sth;
+  
+  my ($paymentaccno) = split /--/, $form->{account};
+  
+  # if currency ne defaultcurrency update exchangerate
+  if ($form->{currency} ne $form->{defaultcurrency}) {
+    $form->{exchangerate} = $form->parse_amount($myconfig, $form->{exchangerate});
+
+    if ($form->{vc} eq 'customer') {
+      $form->update_exchangerate($dbh, $form->{currency}, $form->{datepaid}, $form->{exchangerate}, 0);
+    } else {
+      $form->update_exchangerate($dbh, $form->{currency}, $form->{datepaid}, 0, $form->{exchangerate});
+    }
+  } else {
+    $form->{exchangerate} = 1;
+  }
+
+  my $query = qq|SELECT fxgain_accno_id, fxloss_accno_id
+                 FROM defaults|;
+  my ($fxgain_accno_id, $fxloss_accno_id) = $dbh->selectrow_array($query);
+
+  my ($buysell);
+  
+  if ($form->{vc} eq 'customer') {
+    $buysell = "buy";
+  } else {
+    $buysell = "sell";
+  }
+
+  my $ml;
+  my $where;
+  
+  if ($form->{ARAP} eq 'AR') {
+    $ml = 1;
+    $where = qq|
+		(c.link = 'AR'
+		OR c.link LIKE 'AR:%')
+		|;
+  } else {
+    $ml = -1;
+    $where = qq|
+                (c.link = 'AP'
+                OR c.link LIKE '%:AP'
+		OR c.link LIKE '%:AP:%')
+		|;
+  }
+  
+  # query to retrieve paid amount
+  $query = qq|SELECT paid FROM $form->{arap}
+              WHERE id = ?
+ 	      FOR UPDATE|;
+  my $pth = $dbh->prepare($query) || $form->dberror($query);
+
+  my %audittrail;
+ 
+  my $overpayment = 0;
+  my $accno_id;
+  
+  # go through line by line
+  for my $i (1 .. $form->{rowcount}) {
+
+    $form->{"paid_$i"} = $form->parse_amount($myconfig, $form->{"paid_$i"});
+    $form->{"due_$i"} = $form->parse_amount($myconfig, $form->{"due_$i"});
+    
+    if ($form->{"$form->{vc}_id_$i"} ne $sameid) {
+      # record a AR/AP with a payment
+      if ($overpayment > 0) {
+	$form->{invnumber} = "";
+	OP::overpayment("", $myconfig, $form, $dbh, $overpayment, $ml, 1);
+      }
+      $overpayment = 0;
+      $form->{"$form->{vc}_id"} = $form->{"$form->{vc}_id_$i"};
+      for (qw(source memo)) { $form->{$_} = $form->{"${_}_$i"} }
+    }
+    
+    if ($form->{"checked_$i"} && $form->{"paid_$i"}) {
+     
+      $overpayment += ($form->{"paid_$i"} - $form->{"due_$i"});
+      
+      # get exchangerate for original 
+      $query = qq|SELECT $buysell
+                  FROM exchangerate e
+                  JOIN $form->{arap} a ON (a.transdate = e.transdate)
+		  WHERE e.curr = '$form->{currency}'
+		  AND a.id = $form->{"id_$i"}|;
+      my ($exchangerate) = $dbh->selectrow_array($query);
+
+      $exchangerate ||= 1;
+
+      $query = qq|SELECT c.id
+                  FROM chart c
+		  JOIN acc_trans a ON (a.chart_id = c.id)
+	  	  WHERE $where
+		  AND a.trans_id = $form->{"id_$i"}|;
+      my ($id) = $dbh->selectrow_array($query);
+
+      $paid = ($form->{"paid_$i"} > $form->{"due_$i"}) ? $form->{"due_$i"} : $form->{"paid_$i"};
+      $amount = $form->round_amount($paid * $exchangerate, 2);
+      
+      # add AR/AP
+      $query = qq|INSERT INTO acc_trans (trans_id, chart_id, transdate,
+                  amount)
+                  VALUES ($form->{"id_$i"}, $id, '$form->{datepaid}',
+		  $amount * $ml)|;
+      $dbh->do($query) || $form->dberror($query);
+
+      $query = qq|SELECT id
+                  FROM chart
+                  WHERE accno = '$paymentaccno'|;
+      ($accno_id) = $dbh->selectrow_array($query);
+      
+      # add payment
+      $query = qq|INSERT INTO acc_trans (trans_id, chart_id, transdate,
+                  amount, source, memo)
+                  VALUES ($form->{"id_$i"}, $accno_id, '$form->{datepaid}',
+		  $paid * $ml * -1, |
+		  .$dbh->quote($form->{source}).qq|, |
+		  .$dbh->quote($form->{memo}).qq|)|;
+      $dbh->do($query) || $form->dberror($query);
+
+      # add exchangerate difference if currency ne defaultcurrency
+      $amount = $form->round_amount($paid * ($form->{exchangerate} - 1) * $ml * -1, 2);
+
+      if ($amount) {
+        # exchangerate difference
+	$query = qq|INSERT INTO acc_trans (trans_id, chart_id, transdate,
+		    amount, source, memo)
+		    VALUES ($form->{"id_$i"}, $accno_id, '$form->{datepaid}',
+		    $amount, |
+		    .$dbh->quote($form->{source}).qq|, |
+		    .$dbh->quote($form->{memo}).qq|)|;
+	$dbh->do($query) || $form->dberror($query);
+
+        # gain/loss
+	$amount = ($form->round_amount($paid * $exchangerate,2) - $form->round_amount($paid * $form->{exchangerate},2)) * $ml * -1;
+	if ($amount) {
+	  $accno_id = ($amount > 0) ? $fxgain_accno_id : $fxloss_accno_id;
+	  $query = qq|INSERT INTO acc_trans (trans_id, chart_id, transdate,
+		      amount, fx_transaction)
+		      VALUES ($form->{"id_$i"}, $accno_id,
+		      '$form->{datepaid}', $amount, '1')|;
+	  $dbh->do($query) || $form->dberror($query);
+	}
+      }
+
+      $paid = $form->round_amount($paid * $exchangerate, 2);
+
+      $pth->execute($form->{"id_$i"}) || $form->dberror;
+      ($amount) = $pth->fetchrow_array;
+      $pth->finish;
+
+      $amount += $paid;
+		  
+      # update AR/AP transaction
+      $query = qq|UPDATE $form->{arap} set
+		  paid = $amount,
+		  datepaid = '$form->{datepaid}'
+		  WHERE id = $form->{"id_$i"}|;
+      $dbh->do($query) || $form->dberror($query);
+      
+      %audittrail = ( tablename  => $form->{arap},
+                      reference  => $form->{source},
+		      formname   => $form->{formname},
+		      action     => 'posted',
+		      id         => $form->{"id_$i"} );
+ 
+      $form->audittrail($dbh, "", \%audittrail);
+      
+    }
+
+    $sameid = $form->{"$form->{vc}_id_$i"};
+  
+  }
+
+  # record a AR/AP with a payment
+  if ($overpayment > 0) {
+    $form->{invnumber} = "";
+    OP::overpayment("", $myconfig, $form, $dbh, $overpayment, $ml, 1);
+  }
+
+  my $rc = $dbh->commit;
+  $dbh->disconnect;
+
+  $rc;
+
+}
+
+
 1;
 
