@@ -2,8 +2,7 @@
 # SQL-Ledger Accounting
 # Copyright (C) 2000
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
 #  Contributors:
@@ -36,7 +35,9 @@ sub create_links {
   my $query;
   my $sth;
   my $ref;
-
+  my $arap = ($form->{db} eq 'customer') ? "ar" : "ap";
+  my $ARAP = uc $arap;
+ 
   if ($form->{id}) {
     $query = qq|SELECT ct.*, b.description AS business, s.*,
                 e.name AS employee, g.pricegroup AS pricegroup,
@@ -52,13 +53,10 @@ sub create_links {
     $sth->execute || $form->dberror($query);
   
     $ref = $sth->fetchrow_hashref(NAME_lc);
-  
-    map { $form->{$_} = $ref->{$_} } keys %$ref;
-
+    for (keys %$ref) { $form->{$_} = $ref->{$_} }
     $sth->finish;
 
     # check if it is orphaned
-    my $arap = ($form->{db} eq 'customer') ? "ar" : "ap";
     $query = qq|SELECT a.id
               FROM $arap a
 	      JOIN $form->{db} ct ON (a.$form->{db}_id = ct.id)
@@ -76,7 +74,6 @@ sub create_links {
     }
     $sth->finish;
 
-
     # get taxes for customer/vendor
     $query = qq|SELECT c.accno
 		FROM chart c
@@ -103,7 +100,7 @@ sub create_links {
   $query = qq|SELECT c.accno, c.description
               FROM chart c
 	      JOIN tax t ON (t.chart_id = c.id)
-	      WHERE c.link LIKE '%CT_tax%'
+	      WHERE c.link LIKE '%${ARAP}_tax%'
 	      ORDER BY c.accno|;
   $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
@@ -127,19 +124,9 @@ sub create_links {
     push @{ $form->{all_business} }, $ref;
   }
   $sth->finish;
-  
+
   # this is for the salesperson
-  $query = qq|SELECT id, name
-              FROM employee
-	      WHERE sales = '1'
-	      ORDER BY 2|;
-  $sth = $dbh->prepare($query);
-  $sth->execute || $form->dberror($query);
-  
-  while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
-    push @{ $form->{all_employee} }, $ref;
-  }
-  $sth->finish;
+  $form->all_employees($myconfig, $dbh, $form->{startdate}, ($form->{vc} eq 'customer') ? 1 : 0);
 
   # get language
   $query = qq|SELECT *
@@ -168,11 +155,7 @@ sub create_links {
   # get currencies
   $query = qq|SELECT curr AS currencies
               FROM defaults|;
-  $sth = $dbh->prepare($query);
-  $sth->execute || $form->dberror($query);
-  
-  ($form->{currencies}) = $sth->fetchrow_array;
-  $sth->finish;
+  ($form->{currencies}) = $dbh->selectrow_array($query);
 
   $dbh->disconnect;
 
@@ -211,6 +194,14 @@ sub save_customer {
                 WHERE trans_id = $form->{id}|;
     $dbh->do($query) || $form->dberror($query);
 
+    $query = qq|SELECT id FROM customer
+                WHERE id = $form->{id}|;
+    if (! $dbh->selectrow_array($query)) {
+      $query = qq|INSERT INTO customer (id)
+                  VALUES ($form->{id})|;
+      $dbh->do($query) || $form->dberror($query);
+    }
+
     # retrieve enddate
     if ($form->{type} && $form->{enddate}) {
       my $now;
@@ -221,7 +212,7 @@ sub save_customer {
 
   } else {
     my $uid = time;
-    $uid .= $form->{login};
+    $uid .= $$;
 
     $query = qq|INSERT INTO customer (name)
                 VALUES ('$uid')|;
@@ -229,11 +220,7 @@ sub save_customer {
     
     $query = qq|SELECT id FROM customer
                 WHERE name = '$uid'|;
-    $sth = $dbh->prepare($query);
-    $sth->execute || $form->dberror($query);
-
-    ($form->{id}) = $sth->fetchrow_array;
-    $sth->finish;
+    ($form->{id}) = $dbh->selectrow_array($query);
 
   }
 
@@ -276,7 +263,7 @@ sub save_customer {
 	      taxincluded = '$form->{taxincluded}',
 	      business_id = $business_id,
 	      taxnumber = |.$dbh->quote($form->{taxnumber}).qq|,
-	      sic_code = '$form->{sic}',
+	      sic_code = '$form->{sic_code}',
 	      iban = '$form->{iban}',
 	      bic = '$form->{bic}',
 	      employee_id = $employee_id,
@@ -339,9 +326,26 @@ sub save_vendor {
     $query = qq|DELETE FROM shipto
                 WHERE trans_id = $form->{id}|;
     $dbh->do($query) || $form->dberror($query);
+    
+    $query = qq|SELECT id FROM vendor
+                WHERE id = $form->{id}|;
+    if (! $dbh->selectrow_array($query)) {
+      $query = qq|INSERT INTO vendor (id)
+                  VALUES ($form->{id})|;
+      $dbh->do($query) || $form->dberror($query);
+    }
+    
+    # retrieve enddate
+    if ($form->{type} && $form->{enddate}) {
+      my $now;
+      $query = qq|SELECT enddate, current_date AS now FROM vendor|;
+      ($form->{enddate}, $now) = $dbh->selectrow_array($query);
+      $form->{enddate} = $now if $form->{enddate} lt $now;
+    }
+
   } else {
     my $uid = time;
-    $uid .= $form->{login};
+    $uid .= $$;
     
     $query = qq|INSERT INTO vendor (name)
                 VALUES ('$uid')|;
@@ -349,11 +353,7 @@ sub save_vendor {
    
     $query = qq|SELECT id FROM vendor
                 WHERE name = '$uid'|;
-    $sth = $dbh->prepare($query);
-    $sth->execute || $form->dberror($query);
-
-    ($form->{id}) = $sth->fetchrow_array;
-    $sth->finish;
+    ($form->{id}) = $dbh->selectrow_array($query);
 
   }
    
@@ -397,7 +397,7 @@ sub save_vendor {
 	      gifi_accno = '$form->{gifi_accno}',
 	      business_id = $business_id,
 	      taxnumber = |.$dbh->quote($form->{taxnumber}).qq|,
-	      sic_code = '$form->{sic}',
+	      sic_code = '$form->{sic_code}',
 	      iban = '$form->{iban}',
 	      bic = '$form->{bic}',
 	      employee_id = $employee_id,
@@ -461,32 +461,50 @@ sub search {
   my $item;
 
   @a = ("$form->{db}number");
-  push @a, qw(name contact city state zipcode country notes email);
-  
+  push @a, qw(name contact city state zipcode country notes phone email);
+
+  if ($form->{employee}) {
+    $var = $form->like(lc $form->{employee});
+    $where .= " AND lower(e.name) LIKE '$var'";
+  }
+ 
   foreach $item (@a) {
-    if ($form->{$item}) {
+    if ($form->{$item} ne "") {
       $var = $form->like(lc $form->{$item});
       $where .= " AND lower(ct.$item) LIKE '$var'";
     }
   }
-  if ($form->{address}) {
+  if ($form->{address} ne "") {
     $var = $form->like(lc $form->{address});
     $where .= " AND (lower(ct.address1) LIKE '$var' OR lower(ct.address2) LIKE '$var')";
   }
+  if ($form->{startdatefrom}) {
+    $where .= " AND ct.startdate >= '$form->{startdatefrom}'";
+  }
+  if ($form->{startdateto}) {
+    $where .= " AND ct.startdate <= '$form->{startdateto}'";
+  }
+
+  if ($form->{status} eq 'active') {
+    $where .= " AND ct.enddate IS NULL";
+  }
+  if ($form->{status} eq 'inactive') {
+    $where .= " AND ct.enddate <= current_date";
+  }
 
   if ($form->{status} eq 'orphaned') {
     $where .= qq| AND ct.id NOT IN (SELECT o.$form->{db}_id
-                                    FROM oe o, $form->{db} cv
-		 	            WHERE cv.id = o.$form->{db}_id)|;
+                                    FROM oe o, $form->{db} vc
+		 	            WHERE vc.id = o.$form->{db}_id)|;
     if ($form->{db} eq 'customer') {
       $where .= qq| AND ct.id NOT IN (SELECT a.customer_id
-                                      FROM ar a, customer cv
-				      WHERE cv.id = a.customer_id)|;
+                                      FROM ar a, customer vc
+				      WHERE vc.id = a.customer_id)|;
     }
     if ($form->{db} eq 'vendor') {
       $where .= qq| AND ct.id NOT IN (SELECT a.vendor_id
-                                      FROM ap a, vendor cv
-				      WHERE cv.id = a.vendor_id)|;
+                                      FROM ap a, vendor vc
+				      WHERE vc.id = a.vendor_id)|;
     }
     $form->{l_invnumber} = $form->{l_ordnumber} = $form->{l_quonumber} = "";
   }
@@ -511,7 +529,7 @@ sub search {
     my $transwhere;
     my $openarap = "";
     my $openoe = "";
-    
+   
     if ($form->{open} || $form->{closed}) {
       unless ($form->{open} && $form->{closed}) {
 	$openarap = " AND a.amount != a.paid" if $form->{open};
@@ -533,10 +551,13 @@ sub search {
       $query = qq|SELECT ct.*, b.description AS business,
                   a.invnumber, a.ordnumber, a.quonumber, a.id AS invid,
 		  '$ar' AS module, 'invoice' AS formtype,
-		  (a.amount = a.paid) AS closed, a.amount, a.netamount
+		  (a.amount = a.paid) AS closed, a.amount, a.netamount,
+		  e.name AS employee, m.name AS manager
 		  FROM $form->{db} ct
 		JOIN $ar a ON (a.$form->{db}_id = ct.id)
 	        LEFT JOIN business b ON (ct.business_id = b.id)
+		LEFT JOIN employee e ON (a.employee_id = e.id)
+		LEFT JOIN employee m ON (m.id = e.managerid)
 		  WHERE $where
 		  AND a.invoice = '0'
 		  $transwhere
@@ -560,10 +581,13 @@ sub search {
                   SELECT ct.*, b.description AS business,
                   a.invnumber, a.ordnumber, a.quonumber, a.id AS invid,
 		  '$module' AS module, 'invoice' AS formtype,
-		  (a.amount = a.paid) AS closed, a.amount, a.netamount
+		  (a.amount = a.paid) AS closed, a.amount, a.netamount,
+		  e.name AS employee, m.name AS manager
 		  FROM $form->{db} ct
 		JOIN $ar a ON (a.$form->{db}_id = ct.id)
 	        LEFT JOIN business b ON (ct.business_id = b.id)
+		LEFT JOIN employee e ON (a.employee_id = e.id)
+		LEFT JOIN employee m ON (m.id = e.managerid)
 		  WHERE $where
 		  AND a.invoice = '1'
 		  $transwhere
@@ -584,10 +608,13 @@ sub search {
                   SELECT ct.*, b.description AS business,
 		  ' ' AS invnumber, o.ordnumber, o.quonumber, o.id AS invid,
 		  'oe' AS module, 'order' AS formtype,
-		  o.closed, o.amount, o.netamount
+		  o.closed, o.amount, o.netamount,
+		  e.name AS employee, m.name AS manager
 		  FROM $form->{db} ct
 		JOIN oe o ON (o.$form->{db}_id = ct.id)
 	        LEFT JOIN business b ON (ct.business_id = b.id)
+		LEFT JOIN employee e ON (o.employee_id = e.id)
+		LEFT JOIN employee m ON (m.id = e.managerid)
 		  WHERE $where
 		  AND o.quotation = '0'
 		  $transwhere
@@ -608,10 +635,13 @@ sub search {
                   SELECT ct.*, b.description AS business,
 		  ' ' AS invnumber, o.ordnumber, o.quonumber, o.id AS invid,
 		  'oe' AS module, 'quotation' AS formtype,
-		  o.closed, o.amount, o.netamount
+		  o.closed, o.amount, o.netamount,
+		  e.name AS employee, m.name AS manager
 		  FROM $form->{db} ct
 		JOIN oe o ON (o.$form->{db}_id = ct.id)
 	        LEFT JOIN business b ON (ct.business_id = b.id)
+		LEFT JOIN employee e ON (o.employee_id = e.id)
+		LEFT JOIN employee m ON (m.id = e.managerid)
 		  WHERE $where
 		  AND o.quotation = '1'
 		  $transwhere
@@ -629,9 +659,23 @@ sub search {
   my $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
+  # accounts
+  $query = qq|SELECT c.accno
+              FROM chart c
+	      JOIN $form->{db}tax t ON (t.chart_id = c.id)
+	      WHERE t.$form->{db}_id = ?|;
+  my $tth = $dbh->prepare($query);
+  
   while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
+    $tth->execute($ref->{id});
+    while (($item) = $tth->fetchrow_array) {
+      $ref->{taxaccount} .= "$item ";
+    }
+    $tth->finish;
+    chop $ref->{taxaccount};
+    
     $ref->{address} = "";
-    map { $ref->{address} .= "$ref->{$_} "; } qw(address1 address2 city state zipcode country);
+    for (qw(address1 address2 city state zipcode country)) { $ref->{address} .= "$ref->{$_} " }
     push @{ $form->{CT} }, $ref;
   }
 
@@ -658,43 +702,43 @@ sub get_history {
   # setup ASC or DESC
   $form->sort_order();
   
-  if ($form->{"$form->{db}number"}) {
+  if ($form->{"$form->{db}number"} ne "") {
     $var = $form->like(lc $form->{"$form->{db}number"});
     $where .= " AND lower(ct.$form->{db}number) LIKE '$var'";
   }
-  if ($form->{name}) {
+  if ($form->{name} ne "") {
     $var = $form->like(lc $form->{name});
     $where .= " AND lower(ct.name) LIKE '$var'";
   }
-  if ($form->{address}) {
+  if ($form->{address} ne "") {
     $var = $form->like(lc $form->{address});
     $where .= " AND lower(ct.address1) LIKE '$var'";
   }
-  if ($form->{city}) {
+  if ($form->{city} ne "") {
     $var = $form->like(lc $form->{city});
     $where .= " AND lower(ct.city) LIKE '$var'";
   }
-  if ($form->{state}) {
+  if ($form->{state} ne "") {
     $var = $form->like(lc $form->{state});
     $where .= " AND lower(ct.state) LIKE '$var'";
   }
-  if ($form->{zipcode}) {
+  if ($form->{zipcode} ne "") {
     $var = $form->like(lc $form->{zipcode});
     $where .= " AND lower(ct.zipcode) LIKE '$var'";
   }
-  if ($form->{country}) {
+  if ($form->{country} ne "") {
     $var = $form->like(lc $form->{country});
     $where .= " AND lower(ct.country) LIKE '$var'";
   }
-  if ($form->{contact}) {
+  if ($form->{contact} ne "") {
     $var = $form->like(lc $form->{contact});
     $where .= " AND lower(ct.contact) LIKE '$var'";
   }
-  if ($form->{notes}) {
+  if ($form->{notes} ne "") {
     $var = $form->like(lc $form->{notes});
     $where .= " AND lower(ct.notes) LIKE '$var'";
   }
-  if ($form->{email}) {
+  if ($form->{email} ne "") {
     $var = $form->like(lc $form->{email});
     $where .= " AND lower(ct.email) LIKE '$var'";
   }
@@ -837,7 +881,7 @@ sub get_history {
   while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
     $ref->{address} = "";
     $ref->{exchangerate} = 1 unless $ref->{exchangerate};
-    map { $ref->{address} .= "$ref->{$_} "; } qw(address1 address2 city state zipcode country);
+    for (qw(address1 address2 city state zipcode country)) { $ref->{address} .= "$ref->{$_} " }
     $ref->{id} = $ref->{ctid};
     push @{ $form->{CT} }, $ref;
   }
@@ -920,13 +964,12 @@ sub save_pricelist {
                  WHERE $form->{db}_id = $form->{id}|;
   $dbh->do($query) || $form->dberror($query);
 
-  
   foreach $i (1 .. $form->{rowcount}) {
 
     if ($form->{"id_$i"}) {
 
       if ($form->{db} eq 'customer') {
-	map { $form->{"${_}_$i"} = $form->parse_amount($myconfig, $form->{"${_}_$i"}) } qw(pricebreak sellprice);
+	for (qw(pricebreak sellprice)) { $form->{"${_}_$i"} = $form->parse_amount($myconfig, $form->{"${_}_$i"}) }
 	
 	$query = qq|INSERT INTO parts$form->{db} (parts_id, customer_id,
 		    pricebreak, sellprice, validfrom, validto, curr)
@@ -936,7 +979,7 @@ sub save_pricelist {
 		    .$form->dbquote($form->{"validto_$i"}, SQL_DATE) .qq|,
 		    '$form->{"curr_$i"}')|;
       } else {
-	map { $form->{"${_}_$i"} = $form->parse_amount($myconfig, $form->{"${_}_$i"}) } qw(leadtime lastcost);
+	for (qw(leadtime lastcost)) { $form->{"${_}_$i"} = $form->parse_amount($myconfig, $form->{"${_}_$i"}) }
 	
 	$query = qq|INSERT INTO parts$form->{db} (parts_id, vendor_id,
 		    partnumber, lastcost, leadtime, curr)
@@ -967,19 +1010,29 @@ sub retrieve_item {
   my $var;
   my $null;
 
-  my $where = "WHERE p.obsolete = '0' AND p.income_accno_id > 0";
+  my $where = "WHERE p.obsolete = '0'";
 
-  if ($form->{"partnumber_$i"}) {
+  if ($form->{db} eq 'vendor') {
+    # parts, services, labor
+    $where .= " AND p.assembly = '0'";
+  }
+  if ($form->{db} eq 'customer') {
+    # parts, assemblies, services
+    $where .= " AND p.income_accno_id > 0";
+  }
+
+  if ($form->{"partnumber_$i"} ne "") {
     $var = $form->like(lc $form->{"partnumber_$i"});
     $where .= " AND lower(p.partnumber) LIKE '$var'";
   }
-  if ($form->{"description_$i"}) {
+  if ($form->{"description_$i"} ne "") {
     $var = $form->like(lc $form->{"description_$i"});
     $where .= " AND lower(p.description) LIKE '$var'";
   }
 
-  if ($form->{"partsgroup_$i"}) {
+  if ($form->{"partsgroup_$i"} ne "") {
     ($null, $var) = split /--/, $form->{"partsgroup_$i"};
+    $var *= 1;
     $where .= qq| AND p.partsgroup_id = $var|;
   }
   
@@ -989,10 +1042,9 @@ sub retrieve_item {
 		 FROM parts p
 		 LEFT JOIN partsgroup pg ON (pg.id = p.partsgroup_id)
 		 $where
-		 |;
+		 ORDER BY partnumber|;
   my $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
-
   my $ref;
   $form->{item_list} = ();
   while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
