@@ -2,8 +2,7 @@
 # SQL-Ledger Accounting
 # Copyright (C) 2000
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
 #  Contributors:
@@ -69,27 +68,25 @@ sub post_transaction {
   # connect to database, turn off AutoCommit
   my $dbh = $form->dbconnect_noauto($myconfig);
 
-  # post the transaction
-  # make up a unique handle and store in reference field
-  # then retrieve the record based on the unique handle to get the id
-  # replace the reference field with the actual variable
-  # add records to acc_trans
-
-  # if there is a $form->{id} replace the old transaction
-  # delete all acc_trans entries and add the new ones
-
   my $query;
   my $sth;
-  
+
   if ($form->{id}) {
-    # delete individual transactions
-    $query = qq|DELETE FROM acc_trans 
-                WHERE trans_id = $form->{id}|;
-    $dbh->do($query) || $form->dberror($query);
-    
-  } else {
+    $query = qq|SELECT id FROM gl WHERE id = $form->{id}|;
+    ($form->{id}) = $dbh->selectrow_array($query);
+
+    if ($form->{id}) {
+      # delete individual transactions
+      $query = qq|DELETE FROM acc_trans 
+		  WHERE trans_id = $form->{id}|;
+      $dbh->do($query) || $form->dberror($query);
+    }
+  }
+  
+  if (! $form->{id}) {
+   
     my $uid = time;
-    $uid .= $form->{login};
+    $uid .= $$;
 
     $query = qq|INSERT INTO gl (reference, employee_id)
                 VALUES ('$uid', (SELECT id FROM employee
@@ -98,15 +95,14 @@ sub post_transaction {
     
     $query = qq|SELECT id FROM gl
                 WHERE reference = '$uid'|;
-    $sth = $dbh->prepare($query);
-    $sth->execute || $form->dberror($query);
-
-    ($form->{id}) = $sth->fetchrow_array;
-    $sth->finish;
+    ($form->{id}) = $dbh->selectrow_array($query);
   }
   
   ($null, $department_id) = split /--/, $form->{department};
   $department_id *= 1;
+
+  $form->{reference} = $form->update_defaults($myconfig, 'glnumber', $dbh) unless $form->{reference};
+  $form->{reference} ||= $form->{id};
   
   $query = qq|UPDATE gl SET 
 	      reference = |.$dbh->quote($form->{reference}).qq|,
@@ -121,42 +117,44 @@ sub post_transaction {
 
   my $amount = 0;
   my $posted = 0;
+  my $debit;
+  my $credit;
+  
   # insert acc_trans transactions
   for $i (1 .. $form->{rowcount}) {
 
-    $form->{"debit_$i"} = $form->parse_amount($myconfig, $form->{"debit_$i"});
-    $form->{"credit_$i"} = $form->parse_amount($myconfig, $form->{"credit_$i"});
+    $debit = $form->parse_amount($myconfig, $form->{"debit_$i"});
+    $credit = $form->parse_amount($myconfig, $form->{"credit_$i"});
 
     # extract accno
     ($accno) = split(/--/, $form->{"accno_$i"});
-    $amount = 0;
     
-    if ($form->{"credit_$i"} != 0) {
-      $amount = $form->{"credit_$i"};
+    if ($credit) {
+      $amount = $credit;
       $posted = 0;
     }
-    if ($form->{"debit_$i"} != 0) {
-      $amount = $form->{"debit_$i"} * -1;
+    if ($debit) {
+      $amount = $debit * -1;
       $posted = 0;
     }
 
-
     # add the record
     if (! $posted) {
       
       ($null, $project_id) = split /--/, $form->{"projectnumber_$i"};
-      $project_id *= 1;
+      $project_id ||= 'NULL';
       $form->{"fx_transaction_$i"} *= 1;
       
       $query = qq|INSERT INTO acc_trans (trans_id, chart_id, amount, transdate,
-		  source, project_id, fx_transaction)
+		  source, project_id, fx_transaction, memo)
 		  VALUES
 		  ($form->{id}, (SELECT id
 				 FROM chart
 				 WHERE accno = '$accno'),
-		   $amount, '$form->{transdate}', |
-		   .$dbh->quote($form->{reference}).qq|,
-		  $project_id, '$form->{"fx_transaction_$i"}')|;
+		   $amount, '$form->{transdate}', |.
+		   $dbh->quote($form->{"source_$i"}) .qq|,
+		  $project_id, '$form->{"fx_transaction_$i"}', |.
+		  $dbh->quote($form->{"memo_$i"}).qq|)|;
     
       $dbh->do($query) || $form->dberror($query);
 
@@ -173,6 +171,8 @@ sub post_transaction {
  
   $form->audittrail($dbh, "", \%audittrail);
 
+  $form->save_recurring($dbh, $myconfig);
+
   # commit and redirect
   my $rc = $dbh->commit;
   $dbh->disconnect;
@@ -195,25 +195,32 @@ sub all_transactions {
 
   my ($glwhere, $arwhere, $apwhere) = ("1 = 1", "1 = 1", "1 = 1");
   
-  if ($form->{reference}) {
+  if ($form->{reference} ne "") {
     $var = $form->like(lc $form->{reference});
     $glwhere .= " AND lower(g.reference) LIKE '$var'";
     $arwhere .= " AND lower(a.invnumber) LIKE '$var'";
     $apwhere .= " AND lower(a.invnumber) LIKE '$var'";
   }
-  if ($form->{department}) {
+  if ($form->{department} ne "") {
     ($null, $var) = split /--/, $form->{department};
     $glwhere .= " AND g.department_id = $var";
     $arwhere .= " AND a.department_id = $var";
     $apwhere .= " AND a.department_id = $var";
   }
 
-  if ($form->{source}) {
+  if ($form->{source} ne "") {
     $var = $form->like(lc $form->{source});
     $glwhere .= " AND lower(ac.source) LIKE '$var'";
     $arwhere .= " AND lower(ac.source) LIKE '$var'";
     $apwhere .= " AND lower(ac.source) LIKE '$var'";
   }
+  
+  if ($form->{memo} ne "") {
+    $var = $form->like(lc $form->{memo});
+    $glwhere .= " AND lower(ac.memo) LIKE '$var'";
+    $arwhere .= " AND lower(ac.memo) LIKE '$var'";
+    $apwhere .= " AND lower(ac.memo) LIKE '$var'";
+  }
 
   ($form->{datefrom}, $form->{dateto}) = $form->from_to($form->{year}, $form->{month}, $form->{interval}) if $form->{year} && $form->{month};
   
@@ -240,8 +247,16 @@ sub all_transactions {
   if ($form->{description}) {
     $var = $form->like(lc $form->{description});
     $glwhere .= " AND lower(g.description) LIKE '$var'";
-    $arwhere .= " AND lower(ct.name) LIKE '$var'";
-    $apwhere .= " AND lower(ct.name) LIKE '$var'";
+    $arwhere .= " AND (lower(ct.name) LIKE '$var'
+                       OR lower(ac.memo) LIKE '$var'
+                       OR a.id IN (SELECT DISTINCT trans_id
+		                   FROM invoice
+			           WHERE lower(description) LIKE '$var'))";
+    $apwhere .= " AND (lower(ct.name) LIKE '$var'
+                       OR lower(ac.memo) LIKE '$var'
+                       OR a.id IN (SELECT DISTINCT trans_id
+		               FROM invoice
+			       WHERE lower(description) LIKE '$var'))";
   }
   if ($form->{notes}) {
     $var = $form->like(lc $form->{notes});
@@ -267,10 +282,10 @@ sub all_transactions {
 
   if ($form->{accno}) {
     # get category for account
-    $query = qq|SELECT category, link
+    $query = qq|SELECT category, link, contra, description
                 FROM chart
 		WHERE accno = '$form->{accno}'|;
-    ($form->{ml}, $form->{link}) = $dbh->selectrow_array($query); 
+    ($form->{category}, $form->{link}, $form->{contra}, $form->{account_description}) = $dbh->selectrow_array($query); 
     
     if ($form->{datefrom}) {
       $query = qq|SELECT SUM(ac.amount)
@@ -285,10 +300,11 @@ sub all_transactions {
   
   if ($form->{gifi_accno}) {
     # get category for account
-    $query = qq|SELECT category, link
-                FROM chart
-		WHERE gifi_accno = '$form->{gifi_accno}'|;
-    ($form->{ml}, $form->{link}) = $dbh->selectrow_array($query); 
+    $query = qq|SELECT c.category, c.link, c.contra, g.description
+                FROM chart c
+		LEFT JOIN gifi g ON (g.accno = c.gifi_accno)
+		WHERE c.gifi_accno = '$form->{gifi_accno}'|;
+    ($form->{category}, $form->{link}, $form->{contra}, $form->{gifi_account_description}) = $dbh->selectrow_array($query); 
    
     if ($form->{datefrom}) {
       $query = qq|SELECT SUM(ac.amount)
@@ -304,11 +320,13 @@ sub all_transactions {
   my $false = ($myconfig->{dbdriver} =~ /Pg/) ? FALSE : q|'0'|;
 
   my %ordinal = ( id => 1,
-                  accno => 9,
-                  transdate => 6,
                   reference => 4,
+		  description => 5,
+                  transdate => 6,
                   source => 7,
-		  description => 5 );
+                  accno => 9,
+		  department => 15,
+		  memo => 16 );
   
   my @a = (id, transdate, reference, source, description, accno);
   my $sortorder = $form->sort_order(\@a, \%ordinal);
@@ -316,31 +334,37 @@ sub all_transactions {
   my $query = qq|SELECT g.id, 'gl' AS type, $false AS invoice, g.reference,
                  g.description, ac.transdate, ac.source,
 		 ac.amount, c.accno, c.gifi_accno, g.notes, c.link,
-		 '' AS till, ac.cleared
-                 FROM gl g, acc_trans ac, chart c
+		 '' AS till, ac.cleared, d.description AS department,
+		 ac.memo
+                 FROM gl g
+		 JOIN acc_trans ac ON (g.id = ac.trans_id)
+		 JOIN chart c ON (ac.chart_id = c.id)
+		 LEFT JOIN department d ON (d.id = g.department_id)
                  WHERE $glwhere
-		 AND ac.chart_id = c.id
-		 AND g.id = ac.trans_id
 	UNION ALL
 	         SELECT a.id, 'ar' AS type, a.invoice, a.invnumber,
 		 ct.name, ac.transdate, ac.source,
 		 ac.amount, c.accno, c.gifi_accno, a.notes, c.link,
-		 a.till, ac.cleared
-		 FROM ar a, acc_trans ac, chart c, customer ct
+		 a.till, ac.cleared, d.description AS department,
+		 ac.memo
+		 FROM ar a
+		 JOIN acc_trans ac ON (a.id = ac.trans_id)
+		 JOIN chart c ON (ac.chart_id = c.id)
+		 JOIN customer ct ON (a.customer_id = ct.id)
+		 LEFT JOIN department d ON (d.id = a.department_id)
 		 WHERE $arwhere
-		 AND ac.chart_id = c.id
-		 AND a.customer_id = ct.id
-		 AND a.id = ac.trans_id
 	UNION ALL
 	         SELECT a.id, 'ap' AS type, a.invoice, a.invnumber,
 		 ct.name, ac.transdate, ac.source,
 		 ac.amount, c.accno, c.gifi_accno, a.notes, c.link,
-		 a.till, ac.cleared
-		 FROM ap a, acc_trans ac, chart c, vendor ct
+		 a.till, ac.cleared, d.description AS department,
+		 ac.memo
+		 FROM ap a
+		 JOIN acc_trans ac ON (a.id = ac.trans_id)
+		 JOIN chart c ON (ac.chart_id = c.id)
+		 JOIN vendor ct ON (a.vendor_id = ct.id)
+		 LEFT JOIN department d ON (d.id = a.department_id)
 		 WHERE $apwhere
-		 AND ac.chart_id = c.id
-		 AND a.vendor_id = ct.id
-		 AND a.id = ac.trans_id
 	         ORDER BY $sortorder|;
   my $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
@@ -383,26 +407,7 @@ sub all_transactions {
     
   }
 
-
   $sth->finish;
-
-  if ($form->{accno}) {
-    $query = qq|SELECT description FROM chart WHERE accno = '$form->{accno}'|;
-    $sth = $dbh->prepare($query);
-    $sth->execute || $form->dberror($query);
-
-    ($form->{account_description}) = $sth->fetchrow_array;
-    $sth->finish;
-  }
-  if ($form->{gifi_accno}) {
-    $query = qq|SELECT description FROM gifi WHERE accno = '$form->{gifi_accno}'|;
-    $sth = $dbh->prepare($query);
-    $sth->execute || $form->dberror($query);
-
-    ($form->{gifi_account_description}) = $sth->fetchrow_array;
-    $sth->finish;
-  }
- 
   $dbh->disconnect;
 
 }
@@ -434,12 +439,11 @@ sub transaction {
     $sth->execute || $form->dberror($query);
 
     $ref = $sth->fetchrow_hashref(NAME_lc);
-    map { $form->{$_} = $ref->{$_} } keys %$ref;
+    for (keys %$ref) { $form->{$_} = $ref->{$_} }
     $sth->finish;
   
     # retrieve individual rows
-    $query = qq|SELECT c.accno, c.description, ac.amount, ac.project_id,
-                p.projectnumber, ac.fx_transaction
+    $query = qq|SELECT ac.*, c.accno, c.description, p.projectnumber
 	        FROM acc_trans ac
 	        JOIN chart c ON (ac.chart_id = c.id)
 	        LEFT JOIN project p ON (p.id = ac.project_id)
@@ -454,6 +458,10 @@ sub transaction {
       }
       push @{ $form->{GL} }, $ref;
     }
+
+    # get recurring transaction
+    $form->get_recurring($dbh);
+      
   } else {
     $query = "SELECT current_date AS transdate, closedto, revtrans
               FROM defaults";
@@ -467,16 +475,14 @@ sub transaction {
 
   my $paid;
   if ($form->{transfer}) {
-    $paid = "AND link LIKE '%_paid%'
-             AND NOT (category = 'I'
-	          OR category = 'E')
+    $paid = qq|AND link LIKE '%_paid%'
 	     
 	  UNION
 	  
-	     SELECT accno,description
-	     FROM chart
-	     WHERE id IN (SELECT fxgain_accno_id FROM defaults)
-	     OR id IN (SELECT fxloss_accno_id FROM defaults)";
+	       SELECT accno,description
+	       FROM chart
+	       WHERE id IN (SELECT fxgain_accno_id FROM defaults)
+	       OR id IN (SELECT fxloss_accno_id FROM defaults)|;
   }
   
   # get chart of accounts
@@ -492,18 +498,12 @@ sub transaction {
     push @{ $form->{all_accno} }, $ref;
   }
   $sth->finish;
+
+  # get departments
+  $form->all_departments($myconfig, $dbh);
   
   # get projects
-  $query = qq|SELECT *
-              FROM project
-	      ORDER BY projectnumber|;
-  $sth = $dbh->prepare($query);
-  $sth->execute || $form->dberror($query);
-
-  while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
-    push @{ $form->{all_projects} }, $ref;
-  }
-  $sth->finish;
+  $form->all_projects($myconfig, $dbh, $form->{transdate});
   
   $dbh->disconnect;
 
