@@ -2,8 +2,7 @@
 # SQL-Ledger Accounting
 # Copyright (C) 2002
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
 # Contributors:
@@ -46,7 +45,7 @@ sub send {
   my $msgid = "$boundary\@$domain";
   
   $self->{charset} = "ISO-8859-1" unless $self->{charset};
-  
+
   if ($out) {
     open(OUT, $out) or return "$out : $!";
   } else {
@@ -55,32 +54,33 @@ sub send {
 
   $self->{contenttype} = "text/plain" unless $self->{contenttype};
   
-  my ($cc, $bcc);
-  $cc = "Cc: $self->{cc}\n" if $self->{cc};
-  $bcc = "Bcc: $self->{bcc}\n" if $self->{bcc};
-
-  foreach my $item (qw(from to cc bcc)) {
-    $self->{$item} =~ s/\\_/_/g;
-    $self->{$item} =~ s/\&lt;/</g;
-    $self->{$item} =~ s/\$<\$/</g;
-    $self->{$item} =~ s/\&gt;/>/g;
-    $self->{$item} =~ s/\$>\$/>/g;
+  my %h;
+  for (qw(from to cc bcc)) {
+    $self->{$_} =~ s/\&lt;/</g;
+    $self->{$_} =~ s/\&gt;/>/g;
+    $self->{$_} =~ s/(\\|\$)//g;
+    $h{$_} = $self->{$_};
   }
-
-  print OUT qq|From: $self->{from}
-To: $self->{to}
-${cc}${bcc}Subject: $self->{subject}
+ 
+  $h{cc} = "Cc: $h{cc}\n" if $self->{cc};
+  $h{bcc} = "Bcc: $h{bcc}\n" if $self->{bcc};
+  $h{notify} = "Disposition-Notification-To: $h{from}\n" if $self->{notify};
+  $h{subject} = ($self->{subject} =~ /([\x00-\x1F]|[\x7B-\xFFFF])/) ? "Subject: =?$self->{charset}?B?".&encode_base64($self->{subject},"")."?=" : "Subject: $self->{subject}";
+  
+  print OUT qq|From: $h{from}
+To: $h{to}
+$h{cc}$h{bcc}$h{subject}
 Message-ID: <$msgid>
-X-Mailer: SQL-Ledger $self->{version}
+$h{notify}X-Mailer: SQL-Ledger $self->{version}
 MIME-Version: 1.0
 |;
 
 
-  if ($self->{attachments}) {
+  if (@{ $self->{attachments} }) {
     print OUT qq|Content-Type: multipart/mixed; boundary="$boundary"
 
 |;
-    if ($self->{message}) {
+    if ($self->{message} ne "") {
       print OUT qq|--${boundary}
 Content-Type: $self->{contenttype}; charset="$self->{charset}"
 
@@ -90,11 +90,10 @@ $self->{message}
     }
 
     foreach my $attachment (@{ $self->{attachments} }) {
-      
+
       my $application = ($attachment =~ /(^\w+$)|\.(html|text|txt|sql)$/) ? "text" : "application";
       
-      open(IN, $attachment);
-      if ($?) {
+      unless (open IN, $attachment) {
 	close(OUT);
 	return "$attachment : $!";
       }
