@@ -2,11 +2,10 @@
 # SQL-Ledger Accounting
 # Copyright (C) 2002
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
-#  Contributors:
+#  Contributors: Tony Fraser <tony@sybaspace.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,37 +27,19 @@
 
 package Menu;
 
-
-sub new {
-  my ($type, $menufile, $level) = @_;
-
-  use SL::Inifile;
-  my $self = Inifile->new($menufile, $level);
-  
-  bless $self, $type if $self;
-
-}
+use SL::Inifile;
+@ISA = qw/Inifile/;
 
 
 sub menuitem {
-  my ($self, $myconfig, $form, $item, $level) = @_;
+  my ($self, $myconfig, $form, $item) = @_;
 
-  my $module = $form->{script};
-  my $action = "section_menu";
-  my $target = "";
+  my $module = ($self->{$item}{module}) ? $self->{$item}{module} : $form->{script};
+  my $action = ($self->{$item}{action}) ? $self->{$item}{action} : "section_menu";
+  my $target = ($self->{$item}{target}) ? $self->{$item}{target} : "";
 
-  if ($self->{$item}{module}) {
-    $module = $self->{$item}{module};
-  }
-  if ($self->{$item}{action}) {
-    $action = $self->{$item}{action};
-  }
-  if ($self->{$item}{target}) {
-    $target = $self->{$item}{target};
-  }
-  
-  $level = $form->escape($item);
-  my $str = qq|<a href=$module?path=$form->{path}&action=$action&level=$level&login=$form->{login}&timeout=$form->{timeout}&sessionid=$form->{sessionid}|;
+  my $level = $form->escape($item);
+  my $str = qq|<a href=$module?path=$form->{path}&action=$action&level=$level&login=$form->{login}&timeout=$form->{timeout}&sessionid=$form->{sessionid}&js=$form->{js}|;
 
   my @vars = qw(module action target href);
   
@@ -67,7 +48,7 @@ sub menuitem {
     @vars = qw(module target href);
   }
 
-  map { delete $self->{$item}{$_} } @vars;
+  for (@vars) { delete $self->{$item}{$_} }
   
   delete $self->{$item}{submenu};
  
@@ -75,7 +56,8 @@ sub menuitem {
   foreach my $key (keys %{ $self->{$item} }) {
     $str .= "&".$form->escape($key)."=";
     ($value, $conf) = split /=/, $self->{$item}{$key}, 2;
-    $value = $myconfig->{$value}."/$conf" if ($conf);
+    $value = "$myconfig->{$value}$conf" if $self->{$item}{$key} =~ /=/;
+    
     $str .= $form->escape($value);
   }
 
@@ -98,7 +80,7 @@ sub access_control {
   if ($menulevel eq "") {
     @menu = grep { !/--/ } @{ $self->{ORDER} };
   } else {
-    @menu = grep { /^${menulevel}--/ } @{ $self->{ORDER} };
+    @menu = grep { /^${menulevel}--/; } @{ $self->{ORDER} };
   }
 
   my @a = split /;/, $myconfig->{acs};
@@ -107,10 +89,10 @@ sub access_control {
   # remove --AR, --AP from array
   grep { ($a, $b) = split /--/; s/--$a$//; } @a;
 
-  map { $excl{$_} = 1 } @a;
+  for (@a) { $excl{$_} = 1 }
 
   @a = ();
-  map { push @a, $_ unless $excl{$_} } (@menu);
+  for (@menu) { push @a, $_ unless $excl{$_} }
 
   @a;
 
