@@ -2,8 +2,7 @@
 # SQL-Ledger Accounting
 # Copyright (C) 2002
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
 #  Contributors:
@@ -48,7 +47,7 @@ sub paymentaccounts {
   }
   $sth->finish;
 
-  $form->all_years($dbh, $myconfig);
+  $form->all_years($myconfig, $dbh);
 
   $dbh->disconnect;
 
@@ -82,8 +81,8 @@ sub payment_transactions {
   # get beginning balance
   $query = qq|SELECT sum(ac.amount)
 	      FROM acc_trans ac
-	      JOIN chart c ON (c.id = ac.chart_id)
-	      WHERE c.accno = '$form->{accno}'
+	      JOIN chart ch ON (ch.id = ac.chart_id)
+	      WHERE ch.accno = '$form->{accno}'
 	      $transdate
 	      $cleared
 	      |;
@@ -92,8 +91,8 @@ sub payment_transactions {
   # fx balance
   $query = qq|SELECT sum(ac.amount)
 	      FROM acc_trans ac
-	      JOIN chart c ON (c.id = ac.chart_id)
-	      WHERE c.accno = '$form->{accno}'
+	      JOIN chart ch ON (ch.id = ac.chart_id)
+	      WHERE ch.accno = '$form->{accno}'
 	      AND ac.fx_transaction = '1'
 	      $transdate
 	      $cleared
@@ -109,8 +108,8 @@ sub payment_transactions {
   # get statement balance
   $query = qq|SELECT sum(ac.amount)
 	      FROM acc_trans ac
-	      JOIN chart c ON (c.id = ac.chart_id)
-	      WHERE c.accno = '$form->{accno}'
+	      JOIN chart ch ON (ch.id = ac.chart_id)
+	      WHERE ch.accno = '$form->{accno}'
 	      $transdate
 	      |;
   ($form->{endingbalance}) = $dbh->selectrow_array($query);
@@ -118,8 +117,8 @@ sub payment_transactions {
   # fx balance
   $query = qq|SELECT sum(ac.amount)
 	      FROM acc_trans ac
-	      JOIN chart c ON (c.id = ac.chart_id)
-	      WHERE c.accno = '$form->{accno}'
+	      JOIN chart ch ON (ch.id = ac.chart_id)
+	      WHERE ch.accno = '$form->{accno}'
 	      AND ac.fx_transaction = '1'
 	      $transdate
 	      |;
@@ -142,6 +141,19 @@ sub payment_transactions {
     }
   }
   
+  my $fx_transaction;
+  if ($form->{fx_transaction}) {
+    $fx_transaction = qq|
+	      AND NOT
+		 (ac.chart_id IN
+		  (SELECT fxgain_accno_id FROM defaults
+		   UNION
+		   SELECT fxloss_accno_id FROM defaults))|;
+  } else {
+    $fx_transaction = qq|
+	      AND ac.fx_transaction = '0'|;
+  }
+ 
   
   if ($form->{summary}) {
     $query = qq|SELECT ac.transdate, ac.source,
@@ -150,20 +162,20 @@ sub payment_transactions {
 		JOIN chart ch ON (ac.chart_id = ch.id)
 		WHERE ch.accno = '$form->{accno}'
 		AND ac.amount >= 0
-		AND ac.fx_transaction = '0'
+		$fx_transaction
 		$cleared|;
     $query .= " AND ac.transdate >= '$form->{fromdate}'" if $form->{fromdate};
     $query .= " AND ac.transdate <= '$form->{todate}'" if $form->{todate};
     $query .= " GROUP BY ac.source, ac.transdate, ac.cleared";
     $query .= qq|
-                UNION
+                UNION ALL
 		SELECT ac.transdate, ac.source,
 		sum(ac.amount) AS amount, ac.cleared
 		FROM acc_trans ac
 		JOIN chart ch ON (ac.chart_id = ch.id)
 		WHERE ch.accno = '$form->{accno}'
 		AND ac.amount < 0
-		AND ac.fx_transaction = '0'
+		$fx_transaction
 		$cleared|;
     $query .= " AND ac.transdate >= '$form->{fromdate}'" if $form->{fromdate};
     $query .= " AND ac.transdate <= '$form->{todate}'" if $form->{todate};
@@ -179,13 +191,12 @@ sub payment_transactions {
 		JOIN chart ch ON (ac.chart_id = ch.id)
 		JOIN gl g ON (g.id = ac.trans_id)
 		WHERE ch.accno = '$form->{accno}'
-		AND ac.fx_transaction = '0'
+		$fx_transaction
 		$cleared|;
     $query .= " AND ac.transdate >= '$form->{fromdate}'" if $form->{fromdate};
     $query .= " AND ac.transdate <= '$form->{todate}'" if $form->{todate};
-    
     $query .= qq|
-                UNION
+                UNION ALL
 		SELECT ac.transdate, ac.source, ac.fx_transaction,
 		ac.amount, ac.cleared, a.id, n.name
 		FROM acc_trans ac
@@ -193,13 +204,12 @@ sub payment_transactions {
 		JOIN ar a ON (a.id = ac.trans_id)
 		JOIN customer n ON (n.id = a.customer_id)
 		WHERE ch.accno = '$form->{accno}'
-		AND ac.fx_transaction = '0'
+		$fx_transaction
 		$cleared|;
     $query .= " AND ac.transdate >= '$form->{fromdate}'" if $form->{fromdate};
     $query .= " AND ac.transdate <= '$form->{todate}'" if $form->{todate};
-    
     $query .= qq|
-                UNION
+                UNION ALL
 		SELECT ac.transdate, ac.source, ac.fx_transaction,
 		ac.amount, ac.cleared, a.id, n.name
 		FROM acc_trans ac
@@ -207,7 +217,7 @@ sub payment_transactions {
 		JOIN ap a ON (a.id = ac.trans_id)
 		JOIN vendor n ON (n.id = a.vendor_id)
 		WHERE ch.accno = '$form->{accno}'
-		AND ac.fx_transaction = '0'
+		$fx_transaction
 		$cleared|;
     $query .= " AND ac.transdate >= '$form->{fromdate}'" if $form->{fromdate};
     $query .= " AND ac.transdate <= '$form->{todate}'" if $form->{todate};
@@ -220,104 +230,52 @@ sub payment_transactions {
 
   my $dr;
   my $cr;
-  my $fxs;
   
-  if ($form->{summary}) {
-    $query = qq|SELECT ac.amount, ac.cleared
-		FROM acc_trans ac
-		JOIN ar a ON (a.id = ac.trans_id)
-		JOIN customer n ON (n.id = a.customer_id)
-		WHERE ac.fx_transaction = '1'
-		AND n.name = ?
-		AND ac.transdate = ?
-		AND ac.trans_id IN (SELECT id FROM ar a
-				    JOIN acc_trans ac ON (a.id = ac.trans_id)
-				    WHERE ac.source = ?)
-		AND ac.cleared = ?
-		AND NOT
-		   (ac.chart_id IN
-		    (SELECT fxgain_accno_id FROM defaults
-		     UNION
-		     SELECT fxloss_accno_id FROM defaults))
-		|;
-		
-    $query .= qq|
-		UNION
-		SELECT ac.amount, ac.cleared
-		FROM acc_trans ac
-		JOIN ap a ON (a.id = ac.trans_id)
-		JOIN vendor n ON (n.id = a.vendor_id)
-		WHERE ac.fx_transaction = '1'
-		AND n.name = ?
-		AND ac.transdate = ?
-		AND ac.trans_id IN (SELECT id FROM ap a
-				    JOIN acc_trans ac ON (a.id = ac.trans_id)
-				    WHERE ac.source = ?)
-		AND ac.cleared = ?
-		AND NOT
-		   (ac.chart_id IN
-		    (SELECT fxgain_accno_id FROM defaults
-		     UNION
-		     SELECT fxloss_accno_id FROM defaults))
-		|;
-
-  } else {
-
-    $query = qq|SELECT ac.amount, ac.cleared
-		FROM acc_trans ac
-		WHERE ac.trans_id = ?
-		AND ac.fx_transaction = '1'
-		$cleared
-		AND NOT
-		   (ac.chart_id IN
-		    (SELECT fxgain_accno_id FROM defaults
-		     UNION
-		     SELECT fxloss_accno_id FROM defaults))
-		|;
-	
-  }
-
-  $fxs = $dbh->prepare($query);
-
-
   if ($form->{summary}) {
     $query = qq|SELECT c.name
 		FROM customer c
 		JOIN ar a ON (c.id = a.customer_id)
 		JOIN acc_trans ac ON (a.id = ac.trans_id)
+		JOIN chart ch ON (ac.chart_id = ch.id)
 		WHERE ac.transdate = ?
-		AND ac.source = ?
-		AND ac.amount > 0
+		AND ch.accno = '$form->{accno}'
+		AND (ac.source = ? OR ac.source IS NULL)
+		AND ac.amount >= 0
 		$cleared
 	UNION
 		SELECT v.name
 		FROM vendor v
 		JOIN ap a ON (v.id = a.vendor_id)
 		JOIN acc_trans ac ON (a.id = ac.trans_id)
+		JOIN chart ch ON (ac.chart_id = ch.id)
 		WHERE ac.transdate = ?
-		AND ac.source = ?
+		AND ch.accno = '$form->{accno}'
+		AND (ac.source = ? OR ac.source IS NULL)
 		AND ac.amount > 0
 		$cleared
 	UNION
 		SELECT g.description
 		FROM gl g
 		JOIN acc_trans ac ON (g.id = ac.trans_id)
+		JOIN chart ch ON (ac.chart_id = ch.id)
 		WHERE ac.transdate = ?
-		AND ac.source = ?
-		AND ac.amount > 0
+		AND ch.accno = '$form->{accno}'
+		AND (ac.source = ? OR ac.source IS NULL)
+		AND ac.amount >= 0
 		$cleared
 		|;
     
     $query .= " ORDER BY 1";
     $dr = $dbh->prepare($query);
 
-
     $query = qq|SELECT c.name
 		FROM customer c
 		JOIN ar a ON (c.id = a.customer_id)
 		JOIN acc_trans ac ON (a.id = ac.trans_id)
+		JOIN chart ch ON (ac.chart_id = ch.id)
 		WHERE ac.transdate = ?
-		AND ac.source = ?
+		AND ch.accno = '$form->{accno}'
+		AND (ac.source = ? OR ac.source IS NULL)
 		AND ac.amount < 0
 		$cleared
 	UNION
@@ -325,16 +283,20 @@ sub payment_transactions {
 		FROM vendor v
 		JOIN ap a ON (v.id = a.vendor_id)
 		JOIN acc_trans ac ON (a.id = ac.trans_id)
+		JOIN chart ch ON (ac.chart_id = ch.id)
 		WHERE ac.transdate = ?
-		AND ac.source = ?
+		AND ch.accno = '$form->{accno}'
+		AND (ac.source = ? OR ac.source IS NULL)
 		AND ac.amount < 0
 		$cleared
 	UNION
 		SELECT g.description
 		FROM gl g
 		JOIN acc_trans ac ON (g.id = ac.trans_id)
+		JOIN chart ch ON (ac.chart_id = ch.id)
 		WHERE ac.transdate = ?
-		AND ac.source = ?
+		AND ch.accno = '$form->{accno}'
+		AND (ac.source = ? OR ac.source IS NULL)
 		AND ac.amount < 0
 		$cleared
 		|;
@@ -343,10 +305,8 @@ sub payment_transactions {
     $cr = $dbh->prepare($query);
   }
  
-
   my $name;
   my $ref;
-  my $xfref;
 
   while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
 
@@ -356,6 +316,7 @@ sub payment_transactions {
 	$dr->execute($ref->{transdate}, $ref->{source}, $ref->{transdate}, $ref->{source}, $ref->{transdate}, $ref->{source});
 	$ref->{oldcleared} = $ref->{cleared};
 	$ref->{name} = ();
+
 	while (($name) = $dr->fetchrow_array) {
 	  push @{ $ref->{name} }, $name;
 	}
@@ -378,39 +339,6 @@ sub payment_transactions {
 
     push @{ $form->{PR} }, $ref;
 
-    # include fx transactions
-    $amount = 0;
-    $addfx = 0;
-    $ref->{oldcleared} = $ref->{cleared};
-    if ($form->{summary}) {
-      foreach $name (@{ $ref->{name} }) {
-	$fxs->execute($name, $ref->{transdate}, $ref->{source}, $ref->{cleared}, $name, $ref->{transdate}, $ref->{source}, $ref->{cleared});
-	while ($fxref = $fxs->fetchrow_hashref(NAME_lc)) {
-	  $addfx = 1;
-	  $amount += $fxref->{amount};
-	}
-	$fxs->finish;
-      }
-    } else {
-      $fxs->execute($ref->{id});
-      while ($fxref = $fxs->fetchrow_hashref(NAME_lc)) {
-	$addfx = 1;
-	$amount += $fxref->{amount};
-      }
-      $fxs->finish;
-    }
-    
-    if ($addfx) {
-      $fxref = ();
-      map { $fxref->{$_} = $ref->{$_} } keys %$ref;
-      $fxref->{fx_transaction} = 1;
-      $fxref->{name} = ();
-      $fxref->{source} = "";
-      $fxref->{transdate} = "";
-      $fxref->{amount} = $amount;
-      push @{ $form->{PR} }, $fxref;
-    }
- 
   }
   $sth->finish;
 
