@@ -2,8 +2,7 @@
 # SQL-Ledger Accounting
 # Copyright (C) 2001
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
 #  Contributors: Benjamin Lee <benjaminlee@consultant.com>
@@ -60,8 +59,10 @@ sub yearend_statement {
   foreach $id (@trans_id) {
     $sth->execute($id);
     $ath->execute($id);
+
+    $sth->finish;
+    $ath->finish;
   }
-  $sth->finish;
   
   
   my $last_period = 0;
@@ -256,11 +257,17 @@ sub balance_sheet {
   my $dbh = $form->dbconnect($myconfig);
 
   my $last_period = 0;
-  my @categories = qw(A C L Q);
+  my @categories = qw(A L Q);
 
   my $null;
   
-  if (! $form->{asofdate}) {
+  if ($form->{asofdate}) {
+    if ($form->{asofyear} && $form->{asofmonth}) {
+      if ($form->{asofdate} !~ /\W/) {
+	$form->{asofdate} = "$form->{asofyear}$form->{asofmonth}$form->{asofdate}";
+      }
+    }
+  } else {
     if ($form->{asofyear} && $form->{asofmonth}) {
       ($null, $form->{asofdate}) = $form->from_to($form->{asofyear}, $form->{asofmonth});
     }
@@ -278,7 +285,13 @@ sub balance_sheet {
 
   &get_accounts($dbh, $last_period, "", $form->{asofdate}, $form, \@categories, 1);
   
-  if (! $form->{compareasofdate}) {
+  if ($form->{compareasofdate}) {
+    if ($form->{compareasofyear} && $form->{compareasofmonth}) {
+      if ($form->{compareasofdate} !~ /\W/) {
+	$form->{compareasofdate} = "$form->{compareasofyear}$form->{compareasofmonth}$form->{compareasofdate}";
+      }
+    }
+  } else {
     if ($form->{compareasofyear} && $form->{compareasofmonth}) {
       ($null, $form->{compareasofdate}) = $form->from_to($form->{compareasofyear}, $form->{compareasofmonth});
     }
@@ -317,8 +330,9 @@ sub balance_sheet {
 		            'labels' => 'equity',
 			    'ml' => 1 }
 		);	    
-			    
-  foreach $category (grep { !/C/ } @categories) {
+
+
+   foreach $category (@categories) {			    
 
     foreach $key (sort keys %{ $form->{$category} }) {
 
@@ -418,11 +432,11 @@ sub balance_sheet {
   }
 
   
-  $form->{total_liabilities_last_period} = $form->format_amount($myconfig, $form->{total_liabilities_last_period}, $form->{decimalplaces}, "- ") if ($form->{total_liabilities_last_period} != 0);
+  $form->{total_liabilities_last_period} = $form->format_amount($myconfig, $form->{total_liabilities_last_period}, $form->{decimalplaces}, "- ") if ($form->{total_liabilities_last_period});
   
-  $form->{total_equity_last_period} = $form->format_amount($myconfig, $form->{total_equity_last_period}, $form->{decimalplaces}, "- ") if ($form->{total_equity_last_period} != 0);
+  $form->{total_equity_last_period} = $form->format_amount($myconfig, $form->{total_equity_last_period}, $form->{decimalplaces}, "- ") if ($form->{total_equity_last_period});
   
-  $form->{total_assets_last_period} = $form->format_amount($myconfig, $form->{total_assets_last_period}, $form->{decimalplaces}, "- ") if ($form->{total_assets_last_period} != 0);
+  $form->{total_assets_last_period} = $form->format_amount($myconfig, $form->{total_assets_last_period}, $form->{decimalplaces}, "- ") if ($form->{total_assets_last_period});
   
   $form->{total_assets_this_period} = $form->format_amount($myconfig, $form->{total_assets_this_period}, $form->{decimalplaces}, "- ");
   
@@ -434,14 +448,14 @@ sub balance_sheet {
 
 
 sub get_accounts {
-  my ($dbh, $last_period, $fromdate, $todate, $form, $categories, $yearend) = @_;
+  my ($dbh, $last_period, $fromdate, $todate, $form, $categories, $excludeyearend) = @_;
 
   my $department_id;
   my $project_id;
   
   ($null, $department_id) = split /--/, $form->{department};
   ($null, $project_id) = split /--/, $form->{projectnumber};
-  
+
   my $query;
   my $dpt_where;
   my $dpt_join;
@@ -492,6 +506,9 @@ sub get_accounts {
 
   $sth->finish;
 
+  if ($form->{method} eq 'cash' && !$todate) {
+    ($todate) = $dbh->selectrow_array(qq|SELECT current_date FROM defaults|);
+  }
 
   if ($fromdate) {
     $where .= " AND ac.transdate >= '$fromdate'";
@@ -509,7 +526,7 @@ sub get_accounts {
     $yearendwhere = "ac.transdate < '$todate'";
   }
 
-  if ($yearend) {
+  if ($excludeyearend) {
     $ywhere = " AND ac.trans_id NOT IN
                (SELECT trans_id FROM yearend)";
 	       
@@ -532,8 +549,7 @@ sub get_accounts {
     }
   }
 
-  if ($department_id)
-  {
+  if ($department_id) {
     $dpt_join = qq|
                JOIN department t ON (a.department_id = t.id)
 		  |;
@@ -542,19 +558,16 @@ sub get_accounts {
 	           |;
   }
 
-  if ($project_id)
-  {
+  if ($project_id) {
     $project = qq|
                  AND ac.project_id = $project_id
 		 |;
   }
 
 
-  if ($form->{accounttype} eq 'gifi')
-  {
+  if ($form->{accounttype} eq 'gifi') {
     
-    if ($form->{method} eq 'cash')
-    {
+    if ($form->{method} eq 'cash') {
 
 	$query = qq|
 	
@@ -691,7 +704,7 @@ sub get_accounts {
 		 GROUP BY c.category
 		 |;
 
-      if ($yearend) {
+      if ($excludeyearend) {
 
          # this is for the yearend
 
@@ -702,6 +715,7 @@ sub get_accounts {
 	         SELECT g.accno, sum(ac.amount) AS amount,
 		 g.description, c.category
 		 FROM yearend y
+		 JOIN gl a ON (a.id = y.trans_id)
 		 JOIN acc_trans ac ON (ac.trans_id = y.trans_id)
 		 JOIN chart c ON (c.id = ac.chart_id)
 		 JOIN gifi g ON (g.accno = c.accno)
@@ -714,101 +728,15 @@ sub get_accounts {
 		 |;
       }
 
-      if ($project_id) {
-
-	$query .= qq|
-	  
-       UNION ALL
-       
-		 SELECT g.accno AS accno, SUM(ac.sellprice * ac.qty) AS amount,
-		 g.description AS description, c.category
-		 FROM invoice ac
-	         JOIN ar a ON (a.id = ac.trans_id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN chart c on (p.income_accno_id = c.id)
-	         JOIN gifi g ON (g.accno = c.gifi_accno)
-	         $dpt_join
-		 WHERE 1 = 1 $projectwhere
-		 $ywhere
-		 AND c.category = 'I'
-		 $dpt_where
-		 AND ac.trans_id IN
-		   (
-		     SELECT trans_id
-		     FROM acc_trans
-		     JOIN chart ON (chart_id = id)
-		     WHERE link LIKE '%AR_paid%'
-		     $subwhere
-		   )
-		 $project
-		 GROUP BY g.accno, g.description, c.category
-
-       UNION ALL
-       
-		 SELECT g.accno AS accno, SUM(ac.sellprice * ac.qty) AS amount,
-		 g.description AS description, c.category
-		 FROM invoice ac
-	         JOIN ap a ON (a.id = ac.trans_id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN chart c on (p.expense_accno_id = c.id)
-	         JOIN gifi g ON (g.accno = c.gifi_accno)
-	         $dpt_join
-		 WHERE 1 = 1 $projectwhere
-		 AND p.inventory_accno_id IS NULL
-		 AND p.assembly = '0'
-		 $ywhere
-		 AND c.category = 'E'
-		 $dpt_where
-		 AND ac.trans_id IN
-		   (
-		     SELECT trans_id
-		     FROM acc_trans
-		     JOIN chart ON (chart_id = id)
-		     WHERE link LIKE '%AP_paid%'
-		     $subwhere
-		   )
-		 $project
-		 GROUP BY g.accno, g.description, c.category
-
-       UNION ALL
-       
-		 SELECT g.accno AS accno, SUM(ac.sellprice * ac.allocated) * -1 AS amount,
-		 g.description AS description, c.category
-		 FROM invoice ac
-	         JOIN ap a ON (a.id = ac.trans_id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN chart c on (p.expense_accno_id = c.id)
-	         JOIN gifi g ON (g.accno = c.gifi_accno)
-	         $dpt_join
-		 WHERE 1 = 1 $projectwhere
-		 AND ac.assemblyitem = '0'
-		 $ywhere
-		 AND c.category = 'E'
-		 $dpt_where
-		 AND ac.trans_id IN
-		   (
-		     SELECT trans_id
-		     FROM acc_trans
-		     JOIN chart ON (chart_id = id)
-		     WHERE link LIKE '%AP_paid%'
-		     $subwhere
-		   )
-		 $project
-		 GROUP BY g.accno, g.description, c.category
-		 |;
-	}
-
     } else {
 
-      if ($department_id)
-      {
+      if ($department_id) {
 	$dpt_join = qq|
 	      JOIN dpt_trans t ON (t.trans_id = ac.trans_id)
 	      |;
 	$dpt_where = qq|
                AND t.department_id = $department_id
 	      |;
-
       }
 
       $query = qq|
@@ -842,7 +770,7 @@ sub get_accounts {
 	      GROUP BY c.category
 	      |;
 
-	if ($yearend) {
+	if ($excludeyearend) {
 
 	  # this is for the yearend
 
@@ -853,6 +781,7 @@ sub get_accounts {
 	         SELECT g.accno, sum(ac.amount) AS amount,
 		 g.description, c.category
 		 FROM yearend y
+		 JOIN gl a ON (a.id = y.trans_id)
 		 JOIN acc_trans ac ON (ac.trans_id = y.trans_id)
 		 JOIN chart c ON (c.id = ac.chart_id)
 		 JOIN gifi g ON (g.accno = c.accno)
@@ -864,74 +793,11 @@ sub get_accounts {
 		 GROUP BY g.accno, g.description, c.category
 	      |;
 	}
-
-       if ($project_id)
-       {
-
-	 $query .= qq|
-	  
-	 UNION ALL
-       
-		 SELECT g.accno AS accno, SUM(ac.sellprice * ac.qty) AS amount,
-		 g.description AS description, c.category
-		 FROM invoice ac
-	         JOIN ar a ON (a.id = ac.trans_id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN chart c on (p.income_accno_id = c.id)
-		 JOIN gifi g ON (c.gifi_accno = g.accno)
-	         $dpt_join
-		 WHERE 1 = 1 $projectwhere
-	         $ywhere
-		 AND c.category = 'I'
-		 $dpt_where
-		 $project
-		 GROUP BY g.accno, g.description, c.category
-
-       UNION ALL
-       
-		 SELECT g.accno AS accno, SUM(ac.sellprice * ac.qty) AS amount,
-		 g.description AS description, c.category
-		 FROM invoice ac
-	         JOIN ap a ON (a.id = ac.trans_id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN chart c on (p.expense_accno_id = c.id)
-		 JOIN gifi g ON (c.gifi_accno = g.accno)
-	         $dpt_join
-		 WHERE 1 = 1 $projectwhere
-		 AND p.inventory_accno_id IS NULL
-		 AND p.assembly = '0'
-	         $ywhere
-		 AND c.category = 'E'
-		 $dpt_where
-		 $project
-		 GROUP BY g.accno, g.description, c.category
-
-       UNION ALL
-       
-		 SELECT g.accno AS accno, SUM(ac.sellprice * ac.allocated) * -1 AS amount,
-		 g.description AS description, c.category
-		 FROM invoice ac
-	         JOIN ap a ON (a.id = ac.trans_id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN chart c on (p.expense_accno_id = c.id)
-		 JOIN gifi g ON (c.gifi_accno = g.accno)
-	         $dpt_join
-		 WHERE 1 = 1 $projectwhere
-		 AND ac.assemblyitem = '0'
-	         $ywhere
-		 AND c.category = 'E'
-		 $dpt_where
-		 $project
-		 GROUP BY g.accno, g.description, c.category
-		 |;
-	}
-
     }
     
   } else {    # standard account
 
-    if ($form->{method} eq 'cash')
-    {
+    if ($form->{method} eq 'cash') {
 
       $query = qq|
 	
@@ -999,7 +865,7 @@ sub get_accounts {
 		 GROUP BY c.accno, c.description, c.category
 		 |;
 
-      if ($yearend) {
+      if ($excludeyearend) {
 
         # this is for the yearend
 	
@@ -1010,6 +876,7 @@ sub get_accounts {
 	         SELECT c.accno, sum(ac.amount) AS amount,
 		 c.description, c.category
 		 FROM yearend y
+		 JOIN gl a ON (a.id = y.trans_id)
 		 JOIN acc_trans ac ON (ac.trans_id = y.trans_id)
 		 JOIN chart c ON (c.id = ac.chart_id)
 	         $dpt_join
@@ -1021,96 +888,9 @@ sub get_accounts {
 		 |;
       }
 
-		 
-      if ($project_id)
-      {
-
-	  $query .= qq|
-	  
-	 UNION ALL
-       
-		 SELECT c.accno AS accno, SUM(ac.sellprice * ac.qty) AS amount,
-		 c.description AS description, c.category
-		 FROM invoice ac
-	         JOIN ar a ON (a.id = ac.trans_id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN chart c on (p.income_accno_id = c.id)
-	         $dpt_join
-		 WHERE 1 = 1 $projectwhere
-	         $ywhere
-		 AND c.category = 'I'
-		 $dpt_where
-		 AND ac.trans_id IN
-		   (
-		     SELECT trans_id
-		     FROM acc_trans
-		     JOIN chart ON (chart_id = id)
-		     WHERE link LIKE '%AR_paid%'
-		     $subwhere
-		   )
-
-		 $project
-		 GROUP BY c.accno, c.description, c.category
-
-	 UNION ALL
-       
-		 SELECT c.accno AS accno, SUM(ac.sellprice * ac.qty) AS amount,
-		 c.description AS description, c.category
-		 FROM invoice ac
-	         JOIN ap a ON (a.id = ac.trans_id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN chart c on (p.expense_accno_id = c.id)
-	         $dpt_join
-		 WHERE 1 = 1 $projectwhere
-		 AND p.inventory_accno_id IS NULL
-		 AND p.assembly = '0'
-	         $ywhere
-		 AND c.category = 'E'
-		 $dpt_where
-		 AND ac.trans_id IN
-		   (
-		     SELECT trans_id
-		     FROM acc_trans
-		     JOIN chart ON (chart_id = id)
-		     WHERE link LIKE '%AP_paid%'
-		     $subwhere
-		   )
-
-		 $project
-		 GROUP BY c.accno, c.description, c.category
-		 
-	 UNION ALL
-       
-		 SELECT c.accno AS accno, SUM(ac.sellprice * ac.allocated) * -1 AS amount,
-		 c.description AS description, c.category
-		 FROM invoice ac
-	         JOIN ap a ON (a.id = ac.trans_id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN chart c on (p.expense_accno_id = c.id)
-	         $dpt_join
-		 WHERE 1 = 1 $projectwhere
-		 AND ac.assemblyitem = '0'
-	         $ywhere
-		 AND c.category = 'E'
-		 $dpt_where
-		 AND ac.trans_id IN
-		   (
-		     SELECT trans_id
-		     FROM acc_trans
-		     JOIN chart ON (chart_id = id)
-		     WHERE link LIKE '%AP_paid%'
-		     $subwhere
-		   )
-
-		 $project
-		 GROUP BY c.accno, c.description, c.category
-		 |;
-      }
-
     } else {
      
-      if ($department_id)
-      {
+      if ($department_id) {
 	$dpt_join = qq|
 	      JOIN dpt_trans t ON (t.trans_id = ac.trans_id)
 	      |;
@@ -1135,7 +915,7 @@ sub get_accounts {
 		 GROUP BY c.accno, c.description, c.category
 		 |;
 
-      if ($yearend) {
+      if ($excludeyearend) {
 
         # this is for the yearend
 	
@@ -1146,6 +926,7 @@ sub get_accounts {
 	         SELECT c.accno, sum(ac.amount) AS amount,
 		 c.description, c.category
 		 FROM yearend y
+		 JOIN gl a ON (a.id = y.trans_id)
 		 JOIN acc_trans ac ON (ac.trans_id = y.trans_id)
 		 JOIN chart c ON (c.id = ac.chart_id)
 	         $dpt_join
@@ -1156,66 +937,6 @@ sub get_accounts {
 		 GROUP BY c.accno, c.description, c.category
 		 |;
       }
-
-
-      if ($project_id)
-      {
-
-	$query .= qq|
-	  
-	UNION ALL
-       
-		 SELECT c.accno AS accno, SUM(ac.sellprice * ac.qty) AS amount,
-		 c.description AS description, c.category
-		 FROM invoice ac
-	         JOIN ar a ON (a.id = ac.trans_id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN chart c on (p.income_accno_id = c.id)
-	         $dpt_join
-		 WHERE 1 = 1 $projectwhere
-	         $ywhere
-		 AND c.category = 'I'
-		 $dpt_where
-		 $project
-		 GROUP BY c.accno, c.description, c.category
-
-	UNION ALL
-       
-		 SELECT c.accno AS accno, SUM(ac.sellprice * ac.qty) AS amount,
-		 c.description AS description, c.category
-		 FROM invoice ac
-	         JOIN ap a ON (a.id = ac.trans_id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN chart c on (p.expense_accno_id = c.id)
-	         $dpt_join
-		 WHERE 1 = 1 $projectwhere
-		 AND p.inventory_accno_id IS NULL
-		 AND p.assembly = '0'
-	         $ywhere
-		 AND c.category = 'E'
-		 $dpt_where
-		 $project
-		 GROUP BY c.accno, c.description, c.category
-		 
-	UNION ALL
-       
-		 SELECT c.accno AS accno, SUM(ac.sellprice * ac.allocated) * -1 AS amount,
-		 c.description AS description, c.category
-		 FROM invoice ac
-	         JOIN ap a ON (a.id = ac.trans_id)
-		 JOIN parts p ON (ac.parts_id = p.id)
-		 JOIN chart c on (p.expense_accno_id = c.id)
-	         $dpt_join
-		 WHERE 1 = 1 $projectwhere
-		 AND ac.assemblyitem = '0'
-	         $ywhere
-		 AND c.category = 'E'
-		 $dpt_where
-		 $project
-		 GROUP BY c.accno, c.description, c.category
-		 |;
-
-      }
     }
   }
 
@@ -1226,12 +947,7 @@ sub get_accounts {
   my $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror($query);
 
-  while ($ref = $sth->fetchrow_hashref(NAME_lc))
-  {
-
-    if ($ref->{category} eq 'C') {
-      $ref->{category} = 'A';
-    }
+  while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
 
     # get last heading account
     @accno = grep { $_ le "$ref->{accno}" } @headingaccounts;
@@ -1249,8 +965,7 @@ sub get_accounts {
     $form->{$ref->{category}}{$ref->{accno}}{description} = $ref->{description};
     $form->{$ref->{category}}{$ref->{accno}}{charttype} = "A";
     
-    if ($last_period)
-    {
+    if ($last_period) {
       $form->{$ref->{category}}{$ref->{accno}}{last} += $ref->{amount};
     } else {
       $form->{$ref->{category}}{$ref->{accno}}{this} += $ref->{amount};
@@ -1305,10 +1020,6 @@ sub trial_balance {
   }
   
   
-  # project_id only applies to getting transactions
-  # it has nothing to do with a trial balance
-  # but we use the same function to collect information
-  
   if ($project_id) {
     $project = qq|
                 AND ac.project_id = $project_id
@@ -1323,7 +1034,7 @@ sub trial_balance {
     if ($form->{accounttype} eq 'gifi') {
       
       $query = qq|SELECT g.accno, c.category, SUM(ac.amount) AS amount,
-                  g.description
+                  g.description, c.contra
 		  FROM acc_trans ac
 		  JOIN chart c ON (ac.chart_id = c.id)
 		  JOIN gifi g ON (c.gifi_accno = g.accno)
@@ -1331,20 +1042,20 @@ sub trial_balance {
 		  WHERE ac.transdate < '$form->{fromdate}'
 		  $dpt_where
 		  $project
-		  GROUP BY g.accno, c.category, g.description
+		  GROUP BY g.accno, c.category, g.description, c.contra
 		  |;
    
     } else {
       
       $query = qq|SELECT c.accno, c.category, SUM(ac.amount) AS amount,
-                  c.description
+                  c.description, c.contra
 		  FROM acc_trans ac
 		  JOIN chart c ON (ac.chart_id = c.id)
 		  $dpt_join
 		  WHERE ac.transdate < '$form->{fromdate}'
 		  $dpt_where
 		  $project
-		  GROUP BY c.accno, c.category, c.description
+		  GROUP BY c.accno, c.category, c.description, c.contra
 		  |;
 		  
     }
@@ -1353,19 +1064,21 @@ sub trial_balance {
     $sth->execute || $form->dberror($query);
 
     while (my $ref = $sth->fetchrow_hashref(NAME_lc)) {
+      $ref->{amount} = $form->round_amount($ref->{amount}, 2);
       $balance{$ref->{accno}} = $ref->{amount};
 
-      if ($ref->{amount} != 0 && $form->{all_accounts}) {
+      if ($form->{all_accounts}) {
 	$trb{$ref->{accno}}{description} = $ref->{description};
 	$trb{$ref->{accno}}{charttype} = 'A';
 	$trb{$ref->{accno}}{category} = $ref->{category};
+	$trb{$ref->{accno}}{contra} = $ref->{contra};
       }
 
     }
     $sth->finish;
 
   }
-
+  
 
   # get headings
   $query = qq|SELECT c.accno, c.description, c.category
@@ -1375,7 +1088,7 @@ sub trial_balance {
 
   if ($form->{accounttype} eq 'gifi')
   {
-    $query = qq|SELECT g.accno, g.description, c.category
+    $query = qq|SELECT g.accno, g.description, c.category, c.contra
 		FROM gifi g
 		JOIN chart c ON (c.gifi_accno = g.accno)
 		WHERE c.charttype = 'H'
@@ -1390,6 +1103,7 @@ sub trial_balance {
     $trb{$ref->{accno}}{description} = $ref->{description};
     $trb{$ref->{accno}}{charttype} = 'H';
     $trb{$ref->{accno}}{category} = $ref->{category};
+    $trb{$ref->{accno}}{contra} = $ref->{contra};
    
     push @headingaccounts, $ref->{accno};
   }
@@ -1412,7 +1126,7 @@ sub trial_balance {
   if ($form->{accounttype} eq 'gifi') {
 
     $query = qq|SELECT g.accno, g.description, c.category,
-                SUM(ac.amount) AS amount
+                SUM(ac.amount) AS amount, c.contra
 		FROM acc_trans ac
 		JOIN chart c ON (c.id = ac.chart_id)
 		JOIN gifi g ON (c.gifi_accno = g.accno)
@@ -1420,145 +1134,20 @@ sub trial_balance {
 		WHERE $where
 		$dpt_where
 		$project
-		GROUP BY g.accno, g.description, c.category
-		|;
-
-    if ($project_id) {
-
-      $query .= qq|
-
-	-- sold items
-	
-	UNION ALL
-	
-	        SELECT g.accno, g.description, c.category,
-		SUM(ac.sellprice * ac.qty) AS amount
-		FROM invoice ac
-		JOIN ar a ON (ac.trans_id = a.id)
-		JOIN parts p ON (ac.parts_id = p.id)
-		JOIN chart c ON (p.income_accno_id = c.id)
-		JOIN gifi g ON (c.gifi_accno = g.accno)
-		$dpt_join
-		WHERE $invwhere
-		$dpt_where
-		$project
-		GROUP BY g.accno, g.description, c.category
-
-	UNION ALL
-
-	-- bought services
-	
-	        SELECT g.accno, g.description, c.category,
-		SUM(ac.sellprice * ac.qty) AS amount
-		FROM invoice ac
-		JOIN ap a ON (ac.trans_id = a.id)
-		JOIN parts p ON (ac.parts_id = p.id)
-		JOIN chart c ON (p.expense_accno_id = c.id)
-		JOIN gifi g ON (c.gifi_accno = g.accno)
-		$dpt_join
-		WHERE $invwhere
-		AND p.inventory_accno_id IS NULL
-		AND p.assembly = '0'
-		$dpt_where
-		$project
-		GROUP BY g.accno, g.description, c.category
-
-        -- COGS
-
-	UNION ALL
-	
-	        SELECT g.accno, g.description, c.category,
-		SUM(ac.sellprice * ac.allocated) * -1 AS amount
-		FROM invoice ac
-		JOIN ap a ON (ac.trans_id = a.id)
-		JOIN parts p ON (ac.parts_id = p.id)
-		JOIN chart c ON (p.expense_accno_id = c.id)
-		JOIN gifi g ON (c.gifi_accno = g.accno)
-		$dpt_join
-		WHERE $invwhere
-		AND ac.assemblyitem = '0'
-		$dpt_where
-		$project
-		GROUP BY g.accno, g.description, c.category
-
-		|;
-    }
-
-    $query .= qq|
+		GROUP BY g.accno, g.description, c.category, c.contra
 		ORDER BY accno|;
     
   } else {
 
     $query = qq|SELECT c.accno, c.description, c.category,
-                SUM(ac.amount) AS amount
+                SUM(ac.amount) AS amount, c.contra
 		FROM acc_trans ac
 		JOIN chart c ON (c.id = ac.chart_id)
 		$dpt_join
 		WHERE $where
 		$dpt_where
 		$project
-		GROUP BY c.accno, c.description, c.category
-		|;
-
-    if ($project_id) {
-
-      $query .= qq|
-
-	-- sold items
-	
-	UNION ALL
-	
-	        SELECT c.accno, c.description, c.category,
-		SUM(ac.sellprice * ac.qty) AS amount
-		FROM invoice ac
-		JOIN ar a ON (ac.trans_id = a.id)
-		JOIN parts p ON (ac.parts_id = p.id)
-		JOIN chart c ON (p.income_accno_id = c.id)
-		$dpt_join
-		WHERE $invwhere
-		$dpt_where
-		$project
-		GROUP BY c.accno, c.description, c.category
-
-	UNION ALL
-	
-	-- bought services
-	
-	        SELECT c.accno, c.description, c.category,
-		SUM(ac.sellprice * ac.qty) AS amount
-		FROM invoice ac
-		JOIN ap a ON (ac.trans_id = a.id)
-		JOIN parts p ON (ac.parts_id = p.id)
-		JOIN chart c ON (p.expense_accno_id = c.id)
-		$dpt_join
-		WHERE $invwhere
-		AND p.inventory_accno_id IS NULL
-		AND p.assembly = '0'
-		$dpt_where
-		$project
-		GROUP BY c.accno, c.description, c.category
-
-        -- cogs
-	
-	UNION ALL
-	
-	        SELECT c.accno, c.description, c.category,
-		SUM(ac.sellprice * ac.allocated) * -1 AS amount
-		FROM invoice ac
-		JOIN ap a ON (ac.trans_id = a.id)
-		JOIN parts p ON (ac.parts_id = p.id)
-		JOIN chart c ON (p.expense_accno_id = c.id)
-		$dpt_join
-		WHERE $invwhere
-		AND ac.assemblyitem = '0'
-		$dpt_where
-		$project
-		GROUP BY c.accno, c.description, c.category
- 
-		|;
-    }
-
-    $query .= qq|
+		GROUP BY c.accno, c.description, c.category, c.contra
                 ORDER BY accno|;
 
   }
@@ -1619,6 +1208,7 @@ sub trial_balance {
     $trb{$ref->{accno}}{description} = $ref->{description};
     $trb{$ref->{accno}}{charttype} = 'A';
     $trb{$ref->{accno}}{category} = $ref->{category};
+    $trb{$ref->{accno}}{contra} = $ref->{contra};
     $trb{$ref->{accno}}{amount} += $ref->{amount};
   }
   $sth->finish;
@@ -1629,9 +1219,9 @@ sub trial_balance {
     $ref = ();
     
     $ref->{accno} = $accno;
-    map { $ref->{$_} = $trb{$accno}{$_} } qw(description category charttype amount);
+    for (qw(description category contra charttype amount)) { $ref->{$_} = $trb{$accno}{$_} }
     
-    $ref->{balance} = $form->round_amount($balance{$ref->{accno}}, 2);
+    $ref->{balance} = $balance{$ref->{accno}};
 
     if ($trb{$accno}{charttype} eq 'A') {
       if ($project_id) {
@@ -1695,48 +1285,53 @@ sub aging {
   # connect to database
   my $dbh = $form->dbconnect($myconfig);
   my $invoice = ($form->{arap} eq 'ar') ? 'is' : 'ir';
+
+  my $query = qq|SELECT curr FROM defaults|;
+  ($form->{currencies}) = $dbh->selectrow_array($query);
   
   ($null, $form->{todate}) = $form->from_to($form->{year}, $form->{month}) if $form->{year} && $form->{month};
   
-  $form->{todate} = $form->current_date($myconfig) unless ($form->{todate});
-
-
+  if (! $form->{todate}) {
+    $query = qq|SELECT current_date FROM defaults|;
+    ($form->{todate}) = $dbh->selectrow_array($query);
+  }
+    
   my $where = "1 = 1";
   my $name;
   my $null;
   my $ref;
+  my $transdate = ($form->{overdue}) ? "duedate" : "transdate";
 
   if ($form->{"$form->{ct}_id"}) {
     $where .= qq| AND ct.id = $form->{"$form->{ct}_id"}|;
   } else {
-    if ($form->{$form->{ct}}) {
+    if ($form->{$form->{ct}} ne "") {
       $name = $form->like(lc $form->{$form->{ct}});
       $where .= qq| AND lower(ct.name) LIKE '$name'| if $form->{$form->{ct}};
     }
   }
 
-  my $dpt_join;
   if ($form->{department}) {
     ($null, $department_id) = split /--/, $form->{department};
-    $dpt_join = qq|
-               JOIN department d ON (a.department_id = d.id)
-	          |;
-
     $where .= qq| AND a.department_id = $department_id|;
   }
   
   # select outstanding vendors or customers, depends on $ct
-  my $query = qq|SELECT DISTINCT ct.id, ct.name, ct.language_code
-                 FROM $form->{ct} ct
-		 JOIN $form->{arap} a ON (a.$form->{ct}_id = ct.id)
-		 $dpt_join
-		 WHERE $where
-                 AND a.paid != a.amount
-                 AND (a.transdate <= '$form->{todate}')
-                 ORDER BY ct.name|;
-
+  $query = qq|SELECT DISTINCT ct.id, ct.name, ct.language_code
+              FROM $form->{ct} ct
+	      JOIN $form->{arap} a ON (a.$form->{ct}_id = ct.id)
+	      WHERE $where
+              AND a.paid != a.amount
+              AND (a.$transdate <= '$form->{todate}')
+              ORDER BY ct.name|;
   my $sth = $dbh->prepare($query);
   $sth->execute || $form->dberror;
+  
+  my @ot = ();
+  while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
+    push @ot, $ref;
+  }
+  $sth->finish;
 
   my $buysell = ($form->{arap} eq 'ar') ? 'buy' : 'sell';
 
@@ -1756,110 +1351,167 @@ sub aging {
   
 		    
   # for each company that has some stuff outstanding
-  my $id;
-  while (($id, $null, $language_code) = $sth->fetchrow_array ) {
+  $form->{currencies} ||= ":";
   
-    $query = qq|
-	SELECT c.id AS ctid, c.name,
-	c.address1, c.address2, c.city, c.state, c.zipcode, c.country,
-	c.contact, c.email,
-	c.phone as customerphone, c.fax as customerfax, c.$form->{ct}number,
-	a.invnumber, a.transdate, a.till, a.ordnumber, a.notes,
-	(a.amount - a.paid) as c0, 0.00 as c30, 0.00 as c60, 0.00 as c90,
-	a.duedate, a.invoice, a.id,
-	  (SELECT $buysell FROM exchangerate e
-	   WHERE a.curr = e.curr
-	   AND e.transdate = a.transdate) AS exchangerate
-  FROM $form->{arap} a
-  JOIN $form->{ct} c ON (a.$form->{ct}_id = c.id)
-	WHERE a.paid != a.amount
-	AND c.id = $id
-	AND (
-	        a.transdate <= $interval{$myconfig->{dbdriver}}{c0}
-	        AND a.transdate >= $interval{$myconfig->{dbdriver}}{c30}
-	    )
-	
-	UNION
-
-	SELECT c.id AS ctid, c.name,
-	c.address1, c.address2, c.city, c.state, c.zipcode, c.country,
-	c.contact, c.email,
-	c.phone as customerphone, c.fax as customerfax, c.$form->{ct}number,
-	a.invnumber, a.transdate, a.till, a.ordnumber, a.notes,
-	0.00 as c0, (a.amount - a.paid) as c30, 0.00 as c60, 0.00 as c90,
-	a.duedate, a.invoice, a.id,
-	  (SELECT $buysell FROM exchangerate e
-	   WHERE a.curr = e.curr
-	   AND e.transdate = a.transdate) AS exchangerate
-  FROM $form->{arap} a
-  JOIN $form->{ct} c ON (a.$form->{ct}_id = c.id)
-	WHERE a.paid != a.amount 
-	AND c.id = $id
-	AND (
-		a.transdate < $interval{$myconfig->{dbdriver}}{c30}
-		AND a.transdate >= $interval{$myconfig->{dbdriver}}{c60}
-		)
-
-	UNION
+      
+       $where = qq|
+	    a.paid != a.amount
+	    AND c.id = ?
+	    AND a.curr = ?|;
+	    
+      if ($department_id) {
+	$where .= qq| AND a.department_id = $department_id|;
+      }
   
-	SELECT c.id AS ctid, c.name,
-	c.address1, c.address2, c.city, c.state, c.zipcode, c.country,
-	c.contact, c.email,
-	c.phone as customerphone, c.fax as customerfax, c.$form->{ct}number,
-	a.invnumber, a.transdate, a.till, a.ordnumber, a.notes,
-	0.00 as c0, 0.00 as c30, (a.amount - a.paid) as c60, 0.00 as c90,
-	a.duedate, a.invoice, a.id,
-	  (SELECT $buysell FROM exchangerate e
-	   WHERE a.curr = e.curr
-	   AND e.transdate = a.transdate) AS exchangerate
-	FROM $form->{arap} a
-	JOIN $form->{ct} c ON (a.$form->{ct}_id = c.id)
-	WHERE a.paid != a.amount
-	AND c.id = $id
-	AND (
-		a.transdate < $interval{$myconfig->{dbdriver}}{c60}
-		AND a.transdate >= $interval{$myconfig->{dbdriver}}{c90}
-		)
+      $query = "";
+      my $union = "";
 
-	UNION
-  
-	SELECT c.id AS ctid, c.name,
-	c.address1, c.address2, c.city, c.state, c.zipcode, c.country,
-	c.contact, c.email,
-	c.phone as customerphone, c.fax as customerfax, c.$form->{ct}number,
-	a.invnumber, a.transdate, a.till, a.ordnumber, a.notes,
-	0.00 as c0, 0.00 as c30, 0.00 as c60, (a.amount - a.paid) as c90,
-	a.duedate, a.invoice, a.id,
-	  (SELECT $buysell FROM exchangerate e
-	   WHERE a.curr = e.curr
-	   AND e.transdate = a.transdate) AS exchangerate
-	FROM $form->{arap} a
-	JOIN $form->{ct} c ON (a.$form->{ct}_id = c.id)
-	WHERE a.paid != a.amount
-	AND c.id = $id
-	AND a.transdate < $interval{$myconfig->{dbdriver}}{c90}
+      if ($form->{c0}) {
+	$query .= qq|
+	  SELECT c.id AS ctid, c.$form->{ct}number, c.name,
+	  c.address1, c.address2, c.city, c.state, c.zipcode, c.country,
+	  c.contact, c.email,
+	  c.phone as $form->{ct}phone, c.fax as $form->{ct}fax,
+	  c.$form->{ct}number, c.taxnumber as $form->{ct}taxnumber,
+	  a.invnumber, a.transdate, a.till, a.ordnumber, a.ponumber, a.notes,
+	  (a.amount - a.paid) as c0, 0.00 as c30, 0.00 as c60, 0.00 as c90,
+	  a.duedate, a.invoice, a.id, a.curr,
+	    (SELECT $buysell FROM exchangerate e
+	     WHERE a.curr = e.curr
+	     AND e.transdate = a.transdate) AS exchangerate
+    FROM $form->{arap} a
+    JOIN $form->{ct} c ON (a.$form->{ct}_id = c.id)
+	  WHERE $where
+	  AND (
+		  a.$transdate <= $interval{$myconfig->{dbdriver}}{c0}
+		  AND a.$transdate >= $interval{$myconfig->{dbdriver}}{c30}
+	      )
+|;
 
-	ORDER BY
-  
-  ctid, transdate, invnumber
-  
-		|;
+        $union = qq|
+	  UNION
+|;
 
-    my $sth = $dbh->prepare($query);
-    $sth->execute || $form->dberror;
+      }
+	  
+      if ($form->{c30}) {
 
-    while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
-      $ref->{module} = ($ref->{invoice}) ? $invoice : $form->{arap};
-      $ref->{module} = 'ps' if $ref->{till};
-      $ref->{exchangerate} = 1 unless $ref->{exchangerate};
-      $ref->{language_code} = $language_code;
-      push @{ $form->{AG} }, $ref;
-    }
+	$query .= qq|
+
+	  $union
+
+	  SELECT c.id AS ctid, c.$form->{ct}number, c.name,
+	  c.address1, c.address2, c.city, c.state, c.zipcode, c.country,
+	  c.contact, c.email,
+	  c.phone as $form->{ct}phone, c.fax as $form->{ct}fax,
+	  c.$form->{ct}number, c.taxnumber as $form->{ct}taxnumber,
+	  a.invnumber, a.transdate, a.till, a.ordnumber, a.ponumber, a.notes,
+	  0.00 as c0, (a.amount - a.paid) as c30, 0.00 as c60, 0.00 as c90,
+	  a.duedate, a.invoice, a.id, a.curr,
+	    (SELECT $buysell FROM exchangerate e
+	     WHERE a.curr = e.curr
+	     AND e.transdate = a.transdate) AS exchangerate
+    FROM $form->{arap} a
+    JOIN $form->{ct} c ON (a.$form->{ct}_id = c.id)
+	  WHERE $where
+	  AND (
+		  a.$transdate < $interval{$myconfig->{dbdriver}}{c30}
+		  AND a.$transdate >= $interval{$myconfig->{dbdriver}}{c60}
+		  )
+|;
+
+        $union = qq|
+	  UNION
+|;
+
+      }
+
+      if ($form->{c60}) {
+
+	$query .= qq|
+
+	  $union
     
-    $sth->finish;
+	  SELECT c.id AS ctid, c.$form->{ct}number, c.name,
+	  c.address1, c.address2, c.city, c.state, c.zipcode, c.country,
+	  c.contact, c.email,
+	  c.phone as $form->{ct}phone, c.fax as $form->{ct}fax,
+	  c.$form->{ct}number, c.taxnumber as $form->{ct}taxnumber,
+	  a.invnumber, a.transdate, a.till, a.ordnumber, a.ponumber, a.notes,
+	  0.00 as c0, 0.00 as c30, (a.amount - a.paid) as c60, 0.00 as c90,
+	  a.duedate, a.invoice, a.id, a.curr,
+	    (SELECT $buysell FROM exchangerate e
+	     WHERE a.curr = e.curr
+	     AND e.transdate = a.transdate) AS exchangerate
+	  FROM $form->{arap} a
+	  JOIN $form->{ct} c ON (a.$form->{ct}_id = c.id)
+	  WHERE $where
+	  AND (
+		  a.$transdate < $interval{$myconfig->{dbdriver}}{c60}
+		  AND a.$transdate >= $interval{$myconfig->{dbdriver}}{c90}
+		  )
+|;
 
+        $union = qq|
+	  UNION
+|;
+
+      }
+
+      if ($form->{c90}) {
+
+	$query .= qq|
+
+	  $union
+
+	  SELECT c.id AS ctid, c.$form->{ct}number, c.name,
+	  c.address1, c.address2, c.city, c.state, c.zipcode, c.country,
+	  c.contact, c.email,
+	  c.phone as $form->{ct}phone, c.fax as $form->{ct}fax,
+	  c.$form->{ct}number, c.taxnumber as $form->{ct}taxnumber,
+	  a.invnumber, a.transdate, a.till, a.ordnumber, a.ponumber, a.notes,
+	  0.00 as c0, 0.00 as c30, 0.00 as c60, (a.amount - a.paid) as c90,
+	  a.duedate, a.invoice, a.id, a.curr,
+	    (SELECT $buysell FROM exchangerate e
+	     WHERE a.curr = e.curr
+	     AND e.transdate = a.transdate) AS exchangerate
+	  FROM $form->{arap} a
+	  JOIN $form->{ct} c ON (a.$form->{ct}_id = c.id)
+	  WHERE $where
+	  AND a.$transdate < $interval{$myconfig->{dbdriver}}{c90}
+|;
+      }
+
+      $query .= qq|
+
+	  ORDER BY ctid, $transdate, invnumber|;
+
+      $sth = $dbh->prepare($query) || $form->dberror($query);
+
+  my @var = ();
+  
+  if ($form->{c0} + $form->{c30} + $form->{c60} + $form->{c90}) {
+    foreach $curr (split /:/, $form->{currencies}) {
+    
+      foreach $item (@ot) {
+    
+	@var = ();
+	for (qw(c0 c30 c60 c90)) { push @var, ($item->{id}, $curr) if $form->{$_} }
+	
+	$sth->execute(@var);
+
+	while ($ref = $sth->fetchrow_hashref(NAME_lc)) {
+	  $ref->{module} = ($ref->{invoice}) ? $invoice : $form->{arap};
+	  $ref->{module} = 'ps' if $ref->{till};
+	  $ref->{exchangerate} = 1 unless $ref->{exchangerate};
+	  $ref->{language_code} = $item->{language_code};
+	  push @{ $form->{AG} }, $ref;
+	}
+	$sth->finish;
+
+      }
+    }
   }
-  $sth->finish;
 
   # get language
   my $query = qq|SELECT *
@@ -1900,11 +1552,12 @@ sub get_taxaccounts {
 
   # connect to database
   my $dbh = $form->dbconnect($myconfig);
-
+  my $ARAP = uc $form->{db};
+  
   # get tax accounts
   my $query = qq|SELECT c.accno, c.description, t.rate, c.link
                  FROM chart c, tax t
-		 WHERE c.link LIKE '%CT_tax%'
+		 WHERE c.link LIKE '%${ARAP}_tax%'
 		 AND c.id = t.chart_id
                  ORDER BY c.accno|;
   my $sth = $dbh->prepare($query);
@@ -1922,7 +1575,7 @@ sub get_taxaccounts {
                  FROM gifi g, chart c, tax t
 		 WHERE g.accno = c.gifi_accno
 		 AND c.id = t.chart_id
-		 AND c.link LIKE '%CT_tax%'
+		 AND c.link LIKE '%${ARAP}_tax%'
 		 GROUP BY g.accno, g.description
                  ORDER BY accno|;
   my $sth = $dbh->prepare($query);
@@ -2005,7 +1658,10 @@ sub tax_report {
   if ($form->{method} eq 'cash') {
     $transdate = "a.datepaid";
 
-    my $todate = ($form->{todate}) ? $form->{todate} : $form->current_date($myconfig);
+    my $todate = $form->{todate};
+    if (! $todate) {
+      ($todate) = $dbh->selectrow_array(qq|SELECT current_date FROM defaults|);
+    }
     
     $cashwhere = qq|
 		 AND ac.trans_id IN
@@ -2365,7 +2021,11 @@ sub tax_report {
 
   while ( my $ref = $sth->fetchrow_hashref(NAME_lc)) {
     $ref->{tax} = $form->round_amount($ref->{tax}, 2);
-    push @{ $form->{TR} }, $ref if $ref->{netamount} != 0;
+    if ($form->{report} =~ /nontaxable/) {
+      push @{ $form->{TR} }, $ref if $ref->{netamount};
+    } else {
+      push @{ $form->{TR} }, $ref if $ref->{tax};
+    }
   }
 
   $sth->finish;
@@ -2395,7 +2055,7 @@ sub paymentaccounts {
   }
   $sth->finish;
 
-  $form->all_years($dbh, $myconfig);
+  $form->all_years($myconfig, $dbh);
   
   $dbh->disconnect;
 
@@ -2446,15 +2106,15 @@ sub payments {
     $where .= " AND ac.fx_transaction = '0'";
   }
   
-  if ($form->{description}) {
+  if ($form->{description} ne "") {
     $var = $form->like(lc $form->{description});
     $where .= " AND lower(c.name) LIKE '$var'";
   }
-  if ($form->{source}) {
+  if ($form->{source} ne "") {
     $var = $form->like(lc $form->{source});
     $where .= " AND lower(ac.source) LIKE '$var'";
   }
-  if ($form->{memo}) {
+  if ($form->{memo} ne "") {
     $var = $form->like(lc $form->{memo});
     $where .= " AND lower(ac.memo) LIKE '$var'";
   }
@@ -2495,7 +2155,7 @@ sub payments {
 		WHERE ac.chart_id = $ref->{id}
 		$where|;
 
-    if ($form->{till}) {
+    if ($form->{till} ne "") {
       $query .= " AND a.invoice = '1' 
                   AND NOT a.till IS NULL";
       
@@ -2509,7 +2169,7 @@ sub payments {
 		e.name, a.till, a.curr
 		|;
 		
-    if (! $form->{till}) {
+    if ($form->{till} eq "") {
 # don't need gl for a till
       
       $query .= qq|
