@@ -2,8 +2,7 @@
 # SQL-Ledger Accounting
 # Copyright (C) 2000
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
 #  Contributors: Jim Rawlings <jim@your-dba.com>
@@ -95,11 +94,11 @@ sub country_codes {
 sub login {
   my ($self, $form, $userspath) = @_;
 
-  my $rc = -3;
+  my $rc = -1;
   
-  if ($self->{login}) {
+  if ($self->{login} ne "") {
 
-    if ($self->{password}) {
+    if ($self->{password} ne "") {
       my $password = crypt $form->{password}, substr($self->{login}, 0, 2);
       if ($self->{password} ne $password) {
 	return -1;
@@ -150,12 +149,12 @@ sub login {
 
     
     if ($form->{dbversion} ne $dbversion) {
-      $rc = -4;
+      $rc = -3;
       $dbupdate = (calc_version($dbversion) < calc_version($form->{dbversion}));
     }
 
     if ($dbupdate) {
-      $rc = -5;
+      $rc = -4;
 
       # if DB2 bale out
       if ($myconfig{dbdriver} eq 'DB2') {
@@ -169,6 +168,23 @@ sub login {
 }
 
 
+sub check_recurring {
+  my ($self, $form) = @_;
+
+  $self->{dbpasswd} = unpack 'u', $self->{dbpasswd};
+
+  my $dbh = DBI->connect($self->{dbconnect}, $self->{dbuser}, $self->{dbpasswd}) or $form->dberror;
+
+  my $query = qq|SELECT count(*) FROM recurring
+                 WHERE enddate >= current_date AND nextdate <= current_date|;
+  ($_) = $dbh->selectrow_array($query);
+  
+  $dbh->disconnect;
+
+  $_;
+
+}
+
 
 sub dbconnect_vars {
   my ($form, $db) = @_;
@@ -327,7 +343,7 @@ sub dbcreate {
   &dbconnect_vars($form, $form->{dbdefault});
   my $dbh = DBI->connect($form->{dbconnect}, $form->{dbuser}, $form->{dbpasswd}) or $form->dberror;
   my $query = qq|$dbcreate{$form->{dbdriver}}|;
-  $dbh->do($query) || $form->dberror($query);
+  $dbh->do($query);
 
   if ($form->{dbdriver} eq 'Oracle') {
     $query = qq|GRANT CONNECT,RESOURCE TO "$form->{db}"|;
@@ -792,7 +808,7 @@ sub save_member {
 
   print CONF qq|[$self->{login}]\n|;
   
-  if ($self->{root}) {
+  if ($self->{packpw}) {
     $self->{dbpasswd} = pack 'u', $self->{dbpasswd};
     chop $self->{dbpasswd};
   }
@@ -802,17 +818,15 @@ sub save_member {
   }
   
   if ($self->{'root login'}) {
-    @config = ("password");
+    @config = qw(password);
   } else {
     @config = &config_vars;
   }
  
   # replace \r\n with \n
-  map { $self->{$_} =~ s/\r\n/\\n/g } qw(address signature);
+  for (qw(address signature)) { $self->{$_} =~ s/\r?\n/\\n/g }
 
-  foreach $key (sort @config) {
-    print CONF qq|$key=$self->{$key}\n|;
-  }
+  for (sort @config) { print CONF qq|$_=$self->{$_}\n| }
 
   print CONF "\n";
   close CONF;
@@ -826,11 +840,10 @@ sub save_member {
     $self->{dbpasswd} =~ s/\\\\/\\/g;
     $self->{dbpasswd} = unpack 'u', $self->{dbpasswd};
     
-    # check if login is in database
-    my $dbh = DBI->connect($self->{dbconnect}, $self->{dbuser}, $self->{dbpasswd}, {AutoCommit => 0}) or $self->error($DBI::errstr);
+    # check if login is in database, no error checking
+    my $dbh = DBI->connect($self->{dbconnect}, $self->{dbuser}, $self->{dbpasswd}, {AutoCommit => 0});
 
     # add login to employee table if it does not exist
-    # no error check for employee table, ignore if it does not exist
     my $login = $self->{login};
     $login =~ s/@.*//;
     my $query = qq|SELECT id FROM employee WHERE login = '$login'|;
@@ -850,9 +863,9 @@ sub save_member {
     } else {
       my ($employeenumber) = Form::update_defaults("", \%$self, "employeenumber", $dbh);
       $query = qq|INSERT INTO employee (login, employeenumber, name, workphone,
-                  role, email)
+                  role, email, sales)
 		  VALUES ('$login', '$employeenumber', '$self->{name}',
-		  '$self->{tel}', '$self->{role}', '$self->{email}')|;
+		  '$self->{tel}', '$self->{role}', '$self->{email}', '1')|;
     }
     
     $dbh->do($query);
@@ -888,15 +901,15 @@ sub delete_login {
   $dbh->disconnect;
 
 }
-  
+
 
 sub config_vars {
   
-  my @conf = qw(acs address businessnumber charset company countrycode
-             currency dateformat dbconnect dbdriver dbhost dbport dboptions
-	     dbname dbuser dbpasswd email fax name numberformat password
-	     printer role sid signature stylesheet tel templates vclimit
-	     menuwidth timeout);
+  my @conf = qw(acs address businessnumber company countrycode
+             currency dateformat dbconnect dbdriver dbhost dbname dboptions
+	     dbpasswd dbport dbuser email fax menuwidth name numberformat
+	     password printer role sid signature stylesheet tel
+	     templates timeout vclimit);
 
   @conf;
 
