@@ -1,12 +1,10 @@
 #=====================================================================
 # SQL-Ledger Accounting
-# Copyright (c) 1998-2003
+# Copyright (c) 2002
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
-# Contributors: Jim Rawlings <jim@your-dba.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,10 +34,16 @@ use SL::User;
 $form = new Form;
 
 $locale = new Locale $language, "admin";
-  
+$form->{charset} = $locale->{charset};
+
 eval { require DBI; };
 $form->error($locale->text('DBI not installed!')) if ($@);
 
+$form->{stylesheet} = "sql-ledger.css";
+$form->{favicon} = "favicon.ico";
+$form->{timeout} = 600;
+
+require "$form->{path}/pw.pl";
 
 # customization
 if (-f "$form->{path}/custom_$form->{script}") {
@@ -48,24 +52,11 @@ if (-f "$form->{path}/custom_$form->{script}") {
 }
 
 
-if (-f "css/sql-ledger.css") {
-  $form->{stylesheet} = "sql-ledger.css";
-}
-
-
 if ($form->{action}) {
 
-  $subroutine = $locale->findsub($form->{action});
+  &check_password unless $form->{action} eq 'logout';
   
-  if ($subroutine eq 'login') {
-    if ($form->{rpw}) {
-      $form->{rpw} = crypt $form->{rpw}, "ro";
-    }
-  }
-  
-  &check_password;
-  
-  &$subroutine;
+  &{ $locale->findsub($form->{action}) };
     
 } else {
 
@@ -81,7 +72,7 @@ if ($form->{action}) {
 password=
 
 |;
-    close(FH);
+    close FH;
   }
 
   &adminlogin;
@@ -95,32 +86,37 @@ password=
 sub adminlogin {
 
   $form->{title} = qq|SQL-Ledger $form->{version} |.$locale->text('Administration');
-
+  
   $form->header;
   
-  print qq|
-<body class=admin>
+    print qq|
+<script language="JavaScript" type="text/javascript">
+<!--
+function sf(){
+  document.admin.password.focus();
+}
+// End -->
+</script>
 
+<body class=admin onload="sf()">
 
 <div align=center>
 
-<a href="http://www.sql-ledger.org"><img src=sql-ledger.png border=0></a>
+<a href="http://www.sql-ledger.org"><img src=sql-ledger.gif border=0>&trade;</a>
 <h1 class=login>|.$locale->text('Version').qq| $form->{version}<p>|.$locale->text('Administration').qq|</h1>
 
-<form method=post action="$form->{script}">
+<form method=post action="$form->{script}" name=admin>
 
 <table>
   <tr>
     <th>|.$locale->text('Password').qq|</th>
-    <td><input type=password name=rpw></td>
+    <td><input type=password name=password></td>
     <td><input type=submit class=submit name=action value="|.$locale->text('Login').qq|"></td>
   </tr>
 <input type=hidden name=action value=login>
-<input type=hidden name=root value="root login">
 <input type=hidden name=path value=$form->{path}>
 </table>
 
-
 </form>
 
 <a href=http://www.sql-ledger.org>SQL-Ledger |.$locale->text('website').qq|</a>
@@ -134,8 +130,6 @@ sub adminlogin {
 }
 
 
-
-
 sub login {
 
   &list_users;
@@ -143,6 +137,13 @@ sub login {
 }
 
 
+sub logout {
+
+  $form->{callback} = "$form->{script}?path=$form->{path}&endsession=1";
+  $form->redirect($locale->text('You are logged out'));
+
+}
+
 
 sub add_user {
   
@@ -155,7 +156,9 @@ sub add_user {
   if (-f "css/sql-ledger.css") {
     $myconfig->{stylesheet} = "sql-ledger.css";
   }
-  $myconfig->{vclimit} = 500;
+  $myconfig->{vclimit} = 1000;
+  $myconfig->{menuwidth} = 155;
+  $myconfig->{timeout} = 3600;
   
   &form_header;
   &form_footer;
@@ -184,10 +187,10 @@ sub form_footer {
 
   print qq|
 
-<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&root=$form->{root}&rpw=$form->{rpw}">
-<input type=hidden name=root value="$form->{root}">
+<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&sessionid=$form->{sessionid}">
+
 <input type=hidden name=path value=$form->{path}>
-<input type=hidden name=rpw value=$form->{rpw}>
+<input type=hidden name=sessionid value=$form->{sessionid}>
 
 <input type=submit class=submit name=action value="|.$locale->text('Save').qq|">
 $delete
@@ -203,14 +206,12 @@ $delete
 
 sub list_users {
 
-  $form->error("$memberfile : ".$locale->text('locked!')) if (-f "${memberfile}.LCK");
-
   open(FH, "$memberfile") or $form->error("$memberfile : $!");
 
   $nologin = qq|
 <input type=submit class=submit name=action value="|.$locale->text('Lock System').qq|">|;
 
-  if (-f "$userspath/nologin") {
+  if (-e "$userspath/nologin") {
     $nologin = qq|
 <input type=submit class=submit name=action value="|.$locale->text('Unlock System').qq|">|;
   }
@@ -236,8 +237,7 @@ sub list_users {
 # type=submit $locale->text('PgPP Database Administration')
 # type=submit $locale->text('Oracle Database Administration')
 
-# delete DB2 driver, database administration is done externally
-  foreach $item (grep !/DB2/, User->dbdrivers) {
+  foreach $item (User->dbdrivers) {
     $dbdrivers .= qq|<input name=action type=submit class=submit value="|.$locale->text("$item Database Administration").qq|">|;
   }
 
@@ -254,6 +254,7 @@ sub list_users {
 
   $form->{title} = "SQL-Ledger ".$locale->text('Accounting')." ".$locale->text('Administration');
 
+  $form->{login} = "root login";
   $form->header;
 
   print qq|
@@ -272,14 +273,14 @@ sub list_users {
       <table width=100%>
         <tr class=listheading>|;
 
-  map { print "$column_header{$_}\n" } @column_index;
+  for (@column_index) { print "$column_header{$_}\n" }
   
   print qq|
         </tr>
 |;
 
 foreach $key (sort keys %member) {
-  $href = "$script?action=edit&login=$key&path=$form->{path}&root=$form->{root}&rpw=$form->{rpw}";
+  $href = "$script?action=edit&login=$key&path=$form->{path}&sessionid=$form->{sessionid}";
   $href =~ s/ /%20/g;
   
   $member{$key}{templates} =~ s/^$templates\///;
@@ -298,7 +299,7 @@ foreach $key (sort keys %member) {
   print qq|
         <tr class=listrow$i>|;
 
-  map { print "$column_data{$_}\n" } @column_index;
+  for (@column_index) { print "$column_data{$_}\n" }
 
   print qq|
         </tr>|;
@@ -315,8 +316,7 @@ print qq|
 </table>
 
 <input type=hidden name=path value=$form->{path}>
-<input type=hidden name=rpw value=$form->{rpw}>
-<input type=hidden name=root value="$form->{root}">
+<input type=hidden name=sessionid value=$form->{sessionid}>
 
 <br><input type=submit class=submit name=action value="|.$locale->text('Add User').qq|">
 <input type=submit class=submit name=action value="|.$locale->text('Change Admin Password').qq|">
@@ -324,41 +324,14 @@ print qq|
 $dbdrivers
 $nologin
 
+<input type=submit class=submit name=action value="|.$locale->text('Logout').qq|">
+
 </form>
 
 |.$locale->text('Click on login name to edit!').qq|
 <br>
 |.$locale->text('To add a user to a group edit a name, change the login name and save.  A new user with the same variables will then be saved under the new login name.').qq|
 
-<p>
-
-<form method=post action=login.pl>
-
-<table border=0 width=100%>
-  <tr class=listheading>
-    <th>SQL-Ledger |.$locale->text('Accounting')." ".$locale->text('Login').qq|</th>
-  </tr>
-  <tr>
-    <td>
-      <table>
-        <tr>
-	  <th align=right>|.$locale->text('Name').qq|</th>
-	  <td><input class=login name=login></td>
-	  <td>&nbsp;</td>
-	</tr>
-	<tr>
-	  <th align=right>|.$locale->text('Password').qq|</th>
-	  <td><input class=login type=password name=password></td>
-	  <td><input type=submit name=action value="|.$locale->text('Login').qq|"></td>
-	</tr>
-<input type=hidden name=path value=$form->{path}>
-      </table>
-    </td>
-  </tr>
-</table>
-
-</form>
-
 </body>
 </html>
 |;
@@ -367,7 +340,6 @@ $nologin
 
 
 
-
 sub form_header {
 
   # if there is a login, get user
@@ -375,8 +347,8 @@ sub form_header {
     # get user
     $myconfig = new User "$memberfile", "$form->{login}";
 
-    $myconfig->{signature} =~ s/\\n/\r\n/g;
-    $myconfig->{address} =~ s/\\n/\r\n/g;
+    for (qw(company address signature)) { $myconfig->{$_} = $form->quote($myconfig->{$_}) }
+    for (qw(address signature)) { $myconfig->{$_} =~ s/\\n/\n/g }
 
     # strip basedir from templates directory
     $myconfig->{templates} =~ s/^$templates\///;
@@ -384,7 +356,6 @@ sub form_header {
     $myconfig->{dbpasswd} = unpack 'u', $myconfig->{dbpasswd};
   }
 
-
   foreach $item (qw(mm-dd-yy mm/dd/yy dd-mm-yy dd/mm/yy dd.mm.yy yyyy-mm-dd)) {
     $dateformat .= ($item eq $myconfig->{dateformat}) ? "<option selected>$item\n" : "<option>$item\n";
   }
@@ -396,16 +367,11 @@ sub form_header {
 
   %countrycodes = User->country_codes;
   $countrycodes = "";
-  $gb = $countrycodes{en_GB};
-  delete $countrycodes{en_GB};
   
   foreach $key (sort { $countrycodes{$a} cmp $countrycodes{$b} } keys %countrycodes) {
     $countrycodes .= ($myconfig->{countrycode} eq $key) ? "<option selected value=$key>$countrycodes{$key}" : "<option value=$key>$countrycodes{$key}";
   }
-  if ($gb) {
-    $countrycodes = ($myconfig{countrycode} eq 'en_GB') ? qq|<option selected value="en_GB">English (GB)\n$countrycodes| : qq|<option value="en_GB">English (GB)\n$countrycodes|;
-  }
-  $countrycodes = qq|<option value="">English (US)\n$countrycodes|;
+  $countrycodes = qq|<option value="">English\n$countrycodes|;
 
   # is there a templates basedir
   if (! -d "$templates") {
@@ -417,7 +383,13 @@ sub form_header {
   closedir TEMPLATEDIR;
 
   @allhtml = sort grep /\.html/, @all;
-  @alldir = grep !/\.(html|tex|txt)$/, @all;
+
+  @alldir = ();
+  for (@all) {
+    if (-d "$templates/$_") {
+      push @alldir, $_;
+    }
+  }
   
   @allhtml = reverse grep !/Default/, @allhtml;
   push @allhtml, 'Default';
@@ -443,8 +415,42 @@ sub form_header {
     }
   }
 
+  opendir CSS, "css/.";
+  @all = grep /.*\.css$/, readdir CSS;
+  closedir CSS;
   
+  foreach $item (@all) {
+    if ($item eq $myconfig->{stylesheet}) {
+      $selectstylesheet .= qq|<option selected>$item\n|;
+    } else {
+      $selectstylesheet .= qq|<option>$item\n|;
+    }
+  }
+  $selectstylesheet .= "<option>\n";
+  
+  if (%printer && $latex) {
+    $selectprinter = "<option>\n";
+    foreach $item (sort keys %printer) {
+      if ($myconfig->{printer} eq $item) {
+	$selectprinter .= qq|<option value="$item" selected>$item\n|;
+      } else {
+	$selectprinter .= qq|<option value="$item">$item\n|;
+      }
+    }
+
+    $printer = qq|
+	<tr>
+	  <th align=right>|.$locale->text('Printer').qq|</th>
+	  <td><select name=printer>$selectprinter</select></td>
+	</tr>
+|;
+
+  }
+  
+  $user = $form->{login};
+  $form->{login} = "root login";
   $form->header;
+  $form->{login} = $user;
  
   print qq|
 <body class=admin>
@@ -510,26 +516,24 @@ sub form_header {
 	  <th align=right>|.$locale->text('Dropdown Limit').qq|</th>
 	  <td><input name=vclimit value="$myconfig->{vclimit}"></td>
 	</tr>
+	<tr>
+	  <th align=right>|.$locale->text('Menu Width').qq|</th>
+	  <td><input name=menuwidth value="$myconfig->{menuwidth}"></td>
+	</tr>
 	<tr>
 	  <th align=right>|.$locale->text('Language').qq|</th>
 	  <td><select name=countrycode>$countrycodes</select></td>
 	</tr>
 	<tr>
-	  <th align=right>|.$locale->text('Character Set').qq|</th>
-	  <td><input name=charset value="$myconfig->{charset}"></td>
+	  <th align=right>|.$locale->text('Session Timeout').qq|</th>
+	  <td><input name=newtimeout value="$myconfig->{timeout}"></td>
 	</tr>
+
 	<tr>
 	  <th align=right>|.$locale->text('Stylesheet').qq|</th>
-	  <td><input name=userstylesheet value="$myconfig->{stylesheet}"></td>
-	</tr>
-	<tr>
-	  <th align=right>|.$locale->text('Printer').qq|</th>
-	  <td><input name=printer size=20 value="$myconfig->{printer}"></td>
-	</tr>
-	<tr>
-	  <th align=right>|.$locale->text('Ship via').qq|</th>
-	  <td><input name=shippingpoint size=15 value="$myconfig->{shippingpoint}"></td>
+	  <td><select name=userstylesheet>$selectstylesheet</select></td>
 	</tr>
+	$printer
 	<tr>
 	  <th align=right>|.$locale->text('Use Templates').qq|</th>
 	  <td><select name=usetemplates>$usetemplates</select></td>
@@ -561,7 +565,7 @@ sub form_header {
 
     $checked = "";
     if ($myconfig->{dbdriver} eq $item) {
-      map { $form->{"${item}_$_"} = $myconfig->{$_} } qw(dbhost dbport dbuser dbpasswd dbname sid);
+      for (qw(dbhost dbport dbuser dbpasswd dbname sid)) { $form->{"${item}_$_"} = $myconfig->{$_} }
       $checked = "checked";
     }
 
@@ -569,20 +573,20 @@ sub form_header {
 	  <th align=right>|.$locale->text('Driver').qq|</th>
 	  <td><input name=dbdriver type=radio class=radio value=$item $checked>&nbsp;$item</td>
 	  <th align=right>|.$locale->text('Host').qq|</th>
-	  <td><input name="${item}_dbhost" value=$form->{"${item}_dbhost"}></td>
+	  <td><input name="${item}_dbhost" size=30 value=$form->{"${item}_dbhost"}></td>
 	</tr>
 	<tr>|;
 
-    if ($item =~ /(Pg|DB2)/) {
+    if ($item =~ /Pg/) {
       print qq|
 	  <th align=right>|.$locale->text('Dataset').qq|</th>
-	  <td><input name="${item}_dbname" size=10 value=$form->{"${item}_dbname"}></td>
+	  <td><input name="${item}_dbname" size=15 value=$form->{"${item}_dbname"}></td>
 	  <th align=right>|.$locale->text('Port').qq|</th>
 	  <td><input name="${item}_dbport" size=4 value=$form->{"${item}_dbport"}></td>
 	</tr>
 	<tr>
 	  <th align=right>|.$locale->text('User').qq|</th>
-	  <td><input name="${item}_dbuser" size=10 value=$form->{"${item}_dbuser"}></td>
+	  <td><input name="${item}_dbuser" size=15 value=$form->{"${item}_dbuser"}></td>
 	  <th align=right>|.$locale->text('Password').qq|</th>
 	  <td><input name="${item}_dbpasswd" type=password size=10 value=$form->{"${item}_dbpasswd"}></td>
 	</tr>|;
@@ -594,17 +598,18 @@ sub form_header {
 	  <th align=right>SID</th>
 	  <td><input name=Oracle_sid value=$form->{Oracle_sid}></td>
 	  <th align=right>|.$locale->text('Port').qq|</th>
-	  <td><input name=Oracle_dbport size=4 value=$form->{Oracle_dbport}></td>
+	  <td><input name="${item}_dbport size=4 value=$form->{"${item}_dbport"}></td>
 	</tr>
 	<tr>
 	  <th align=right>|.$locale->text('Dataset').qq|</th>
-	  <td><input name="${item}_dbuser" size=10 value=$form->{"${item}_dbuser"}></td>
+	  <td><input name="${item}_dbuser" size=15 value=$form->{"${item}_dbuser"}></td>
 	  <th align=right>|.$locale->text('Password').qq|</th>
 	  <td><input name="${item}_dbpasswd" type=password size=10 value=$form->{"${item}_dbpasswd"}></td>
 	  
 	</tr>|;
     }
     
+      
     print qq|
 	<input type=hidden name=old_dbpasswd value=$myconfig->{dbpasswd}>
       </table>
@@ -648,14 +653,24 @@ sub form_header {
 
   }
   
-  $admincheck = "checked" if $myconfig->{admin};
+  %role = ( 'admin' => $locale->text('Administrator'),
+            'user' => $locale->text('User'),
+            'supervisor' => $locale->text('Supervisor'),
+	    'manager' => $locale->text('Manager')
+
+	   );
+	    
+  $selectrole = "";
+  foreach $item (qw(user admin supervisor manager)) {
+    $selectrole .= ($myconfig->{role} eq $item) ? "<option selected value=$item>$role{$item}\n" : "<option value=$item>$role{$item}\n";
+  }
   
   print qq|
   <tr class=listheading>
     <th colspan=2>|.$locale->text('Access Control').qq|</th>
   </tr>
   <tr>
-    <td><input name=admin type=checkbox class=checkbox value=1 $admincheck>&nbsp;<b>|.$locale->text('Administrator').qq|</b></td>
+    <td><select name=role>$selectrole</select></td>
   </tr>
 |;
   
@@ -671,20 +686,20 @@ sub form_header {
       $checked = ($excl{$key}{$key}) ? "" : "checked";
     }
     
-    # can't have variable names with spaces
-    # the 1 is for apache 2
-    $item = $form->escape("${key}--$key", 1);
-    
+    # can't have variable names with & and spaces
+    $item = $form->escape("${key}--$key",1);
+
     $acsheading = $key;
     $acsheading =~ s/ /&nbsp;/g;
-    
+
     $acsheading = qq|
-    <th align=left><input name="$item" class=checkbox type=checkbox value=1 $checked>&nbsp;$acsheading</th>\n|;
+    <th align=left nowrap><input name="$item" class=checkbox type=checkbox value=1 $checked>&nbsp;$acsheading</th>\n|;
     $menuitems .= "$item;";
     $acsdata = "
     <td>";
 
     foreach $item (@{ $acs{$key} }) {
+
       next if ($key eq $item);
 
       $checked = "checked";
@@ -692,8 +707,8 @@ sub form_header {
 	$checked = ($excl{$key}{$item}) ? "" : "checked";
       }
 
-      $acsitem = $form->escape("${key}--$item", 1);
-      
+      $acsitem = $form->escape("${key}--$item",1);
+
       $acsdata .= qq|
     <br><input name="$acsitem" class=checkbox type=checkbox value=1 $checked>&nbsp;$item|;
       $menuitems .= "$acsitem;";
@@ -713,11 +728,8 @@ sub form_header {
    <tr>
     <td colspan=2><hr size=3 noshade></td>
   </tr>
-  <tr>
-    <td>
-    </td>
-  </tr>
 </table>
+</div>
 |;
 
 }
@@ -729,7 +741,7 @@ sub save {
   $form->error($locale->text('Database Driver not checked!')) unless $form->{dbdriver};
 
   # no spaces allowed in login name
-  ($form->{login}) = split / /, $form->{login};
+  $form->{login} =~ s/ //g;
 
   $form->isblank("login", $locale->text('Login name missing!'));
   
@@ -742,21 +754,15 @@ sub save {
     }
   }
   
-  # does stylesheet exist
-  if ($form->{userstylesheet}) {
-    $form->error($locale->text('Stylesheet').": css/$form->{userstylesheet} ".$locale->text('does not exist')) unless (-f "css/$form->{userstylesheet}");
-  }
-  
   # no spaces allowed in directories
-  ($form->{newtemplates}) = split / /, $form->{newtemplates};
+  $form->{newtemplates} =~ s/( |\.\.|\*)//g;
   
-  if ($form->{newtemplates}) {
+  if ($form->{newtemplates} ne "") {
     $form->{templates} = $form->{newtemplates};
   } else {
     $form->{templates} = ($form->{usetemplates}) ? $form->{usetemplates} : $form->{login};
   }
   
-  
   # is there a basedir
   if (! -d "$templates") {
     $form->error($locale->text('Directory').": $templates ".$locale->text('does not exist'));
@@ -773,56 +779,53 @@ sub save {
 
   $form->{acs} = "";
   foreach $item (@acs) {
-    $item = $form->escape($item, 1);
-
+    $item = $form->escape($item,1);
     if (!$form->{$item}) {
-      $form->{acs} .= $form->unescape($form->unescape($item)).";";
+      $form->{acs} .= $form->unescape($form->unescape("$item")).";";
     }
     delete $form->{$item};
   }
 
   # check which database was filled in
+  
+  $form->{dbhost} = $form->{"$form->{dbdriver}_dbhost"};
+  $form->{dbport} = $form->{"$form->{dbdriver}_dbport"};
+  $form->{dbpasswd} = $form->{"$form->{dbdriver}_dbpasswd"};
+  $form->{dbuser} = $form->{"$form->{dbdriver}_dbuser"};
+  $form->{dbname} = $form->{"$form->{dbdriver}_dbname"};
+
   if ($form->{dbdriver} eq 'Oracle') {
     $form->{sid} = $form->{Oracle_sid}, ;
-    $form->{dbhost} = $form->{Oracle_dbhost}, ;
-    $form->{dbport} = $form->{Oracle_dbport};
-    $form->{dbpasswd} = $form->{Oracle_dbpasswd};
-    $form->{dbuser} = $form->{Oracle_dbuser};
-    $form->{dbname} = $form->{Oracle_dbuser};
 
     $form->isblank("dbhost", $locale->text('Hostname missing!'));
     $form->isblank("dbport", $locale->text('Port missing!'));
     $form->isblank("dbuser", $locale->text('Dataset missing!'));
   }
-  if ($form->{dbdriver} =~ /(Pg|DB2)/) {
-    $form->{dbhost} = $form->{"$form->{dbdriver}_dbhost"};
-    $form->{dbport} = $form->{"$form->{dbdriver}_dbport"};
-    $form->{dbpasswd} = $form->{"$form->{dbdriver}_dbpasswd"};
-    $form->{dbuser} = $form->{"$form->{dbdriver}_dbuser"};
-    $form->{dbname} = $form->{"$form->{dbdriver}_dbname"};
-    
+  if ($form->{dbdriver} =~ /Pg/) {
     $form->isblank("dbname", $locale->text('Dataset missing!'));
     $form->isblank("dbuser", $locale->text('Database User missing!'));
   }
     
-  # set admin
-  $form->{admin} = "" unless $form->{admin};
-
   foreach $item (keys %{$form}) {
     $myconfig->{$item} = $form->{$item};
   }
 
+  $myconfig->{password} = $form->{old_password};
   $myconfig->{password} = $form->{new_password} if $form->{new_password} ne $form->{old_password};
-  
+  $myconfig->{timeout} = $form->{newtimeout};
+
   delete $myconfig->{stylesheet};
   if ($form->{userstylesheet}) {
     $myconfig->{stylesheet} = $form->{userstylesheet};
   }
   
+  $myconfig->{packpw} = 1;
+  
   $myconfig->save_member($memberfile, $userspath);
 
   # create user template directory and copy master files
   if (! -d "$form->{templates}") {
+    
     umask(002);
     
     if (mkdir "$form->{templates}", oct("771")) {
@@ -831,7 +834,7 @@ sub save {
       
       # copy templates to the directory
       opendir TEMPLATEDIR, "$templates/." or $form->error("$templates : $!");
-      @templates = grep /$form->{mastertemplates}.*?\.(html|tex|txt)$/, readdir TEMPLATEDIR;
+      @templates = grep /$form->{mastertemplates}-/, readdir TEMPLATEDIR;
       closedir TEMPLATEDIR;
 
       foreach $file (@templates) {
@@ -847,7 +850,7 @@ sub save {
 	close(NEW);
       }
     } else {
-      $form->error("$!: $form->{templates}");
+      $form->error("$form->{templates} : $!");
     }
   }
 
@@ -860,11 +863,15 @@ sub delete {
 
   $form->{templates} = ($form->{templates}) ? "$templates/$form->{templates}" : "$templates/$form->{login}";
   
-  $form->error("$memberfile : ".$locale->text('locked!')) if (-f ${memberfile}.LCK);
+  $form->error("$memberfile ".$locale->text('locked!')) if (-f ${memberfile}.LCK);
+
   open(FH, ">${memberfile}.LCK") or $form->error("${memberfile}.LCK : $!");
   close(FH);
   
-  open(CONF, "+<$memberfile") or $form->error("$memberfile : $!");
+  if (! open(CONF, "+<$memberfile")) {
+    unlink "${memberfile}.LCK";
+    $form->error("$memberfile : $!");
+  }
 
   @config = <CONF>;
 
@@ -876,12 +883,12 @@ sub delete {
     chop $line;
 
     if ($line =~ /^\[/) {
-      last if ($line =~ /\[$form->{login}\]/);
+      last if ($line eq "[$form->{login}]");
       $login = &login_name($line);
     }
     
     if ($line =~ /^templates=/) {
-      $user{$login} = &get_value($line);
+      ($null, $user{$login}) = split /=/, $line, 2;
     }
 
     print CONF "$line\n";
@@ -893,12 +900,10 @@ sub delete {
 
     chop $line;
     
-    if ($line =~ /^templates=/) {
-      $templatedir = &get_value($line);
-    }
+    ($key, $value) = split /=/, $line, 2;
+    $myconfig{$key} = $value;
     
     last if ($line =~ /^\[/);
-    
   }
 
   # this one is either the next login or EOF
@@ -910,13 +915,13 @@ sub delete {
   while ($line = shift @config) {
 
     chop $line;
-    
+
     if ($line =~ /^\[/) {
       $login = &login_name($line);
     }
     
     if ($line =~ /^templates=/) {
-      $user{$login} = &get_value($line);
+      ($null, $user{$login}) = split /=/, $line, 2;
     }
     
     print CONF "$line\n";
@@ -927,7 +932,7 @@ sub delete {
 
   # scan %user for $templatedir
   foreach $login (keys %user) {
-    last if ($found = ($templatedir eq $user{$login}));
+    last if ($found = ($form->{templates} eq $user{$login}));
   }
 
   # if found keep directory otherwise delete
@@ -935,16 +940,19 @@ sub delete {
     # delete it if there is a template directory
     $dir = "$form->{templates}";
     if (-d "$dir") {
-      unlink <$dir/*.html>;
-      unlink <$dir/*.tex>;
-      unlink <$dir/*.txt>;
+      unlink <$dir/*>;
       rmdir "$dir";
     }
   }
+
+  if ($myconfig{login}) {
+    for (keys %myconfig) { $form->{$_} = $myconfig{$_} }
+    User->delete_login(\%$form);
   
-  # delete config file for user
-  unlink "$userspath/$form->{login}.conf";
-    
+    # delete config file for user
+    unlink "$userspath/$form->{login}.conf";
+  }
+
   $form->redirect($locale->text('User deleted!'));
   
 }
@@ -959,42 +967,45 @@ sub login_name {
 }
 
 
-
-sub get_value {
-  my $line = shift;
-  
-  my ($null, $value) = split(/=/, $line, 2);
-
-  # remove comments
-  $value =~ s/\s#.*//g;
-      
-  # remove any trailing whitespace
-  $value =~ s/^\s*(.*?)\s*$/$1/;
-
-  $value;
-}
-
-
-
 sub change_admin_password {
 
   $form->{title} = qq|SQL-Ledger |.$locale->text('Accounting')." ".$locale->text('Administration')." / ".$locale->text('Change Admin Password');
-  
+
+  $form->{login} = "root login";
   $form->header;
 
   print qq|
 <body class=admin>
 
-
-<h2>|.$locale->text('Change Admin Password').qq|</h2>
-
 <form method=post action=$form->{script}>
 
-<b>|.$locale->text('Password').qq|</b> <input type=password name=password size=8>
+<table>
+  <tr>
+  <tr class=listheading>
+    <th>|.$locale->text('Change Password').qq|</th>
+  </tr>
+  <tr size=5></tr>
+  <tr>
+    <td>
+      <table width=100%>
+        <tr>
+          <th align=right>|.$locale->text('Password').qq|</th>
+	  <td><input type=password name=new_password></td>
+	</tr>
+	<tr>
+	  <th align=right>|.$locale->text('Confirm').qq|</th>
+	  <td><input type=password name=confirm_password></td>
+	</tr>
+      </table>
+    </td>
+  </tr>
 
-<input type=hidden name=root value="$form->{root}">
+</table>
+
+<br>
+<hr size=3 noshade>
 <input type=hidden name=path value=$form->{path}>
-<input type=hidden name=rpw value=$form->{rpw}>
+<input type=hidden name=sessionid value=$form->{sessionid}>
 
 <p>
 <input type=submit class=submit name=action value="|.$locale->text('Change Password').qq|">
@@ -1010,12 +1021,14 @@ sub change_admin_password {
 
 sub change_password {
 
-  $root->{password} = $form->{password};
+  $form->error($locale->text('Passwords do not match!')) if $form->{new_password} ne $form->{confirm_password};
+  
+  $root->{password} = $form->{new_password};
   
   $root->{'root login'} = 1;
   $root->save_member($memberfile);
 
-  $form->{callback} = "$form->{script}?action=list_users&path=$form->{path}&root=$form->{root}&rpw=$root->{password}";
+  $form->{callback} = "$form->{script}?action=list_users&path=$form->{path}&sessionid=$form->{sessionid}";
 
   $form->redirect($locale->text('Password changed!'));
 
@@ -1024,11 +1037,26 @@ sub change_password {
 
 sub check_password {
 
-  $root = new User "$memberfile", $form->{root};
+  $root = new User "$memberfile", "root login";
 
   if ($root->{password}) {
-    if ($root->{password} ne $form->{rpw}) {
-      $form->error($locale->text('Incorrect Password!'));
+      
+    if ($form->{password}) {
+      $form->{callback} .= "&password=$form->{password}" if $form->{callback};
+      $form->{sessionid} = time;
+      if ($root->{password} ne crypt $form->{password}, 'ro') {
+	&getpassword;
+	exit;
+      }
+    } else {
+      if ($ENV{HTTP_USER_AGENT}) {
+	$ENV{HTTP_COOKIE} =~ s/;\s*/;/g;
+	%cookie = split /[=;]/, $ENV{HTTP_COOKIE};
+	if ((! $cookie{"SQL-Ledger-root login"}) || $cookie{"SQL-Ledger-root login"} ne $form->{sessionid}) {
+	  &getpassword;
+	  exit;
+	}
+      }
     }
   }
 
@@ -1078,8 +1106,8 @@ sub dbdriver_defaults {
 
   $driverdefaults{PgPP} = $driverdefaults{Pg};
 
-  map { $form->{$_} = $driverdefaults{$form->{dbdriver}}{$_} } keys %{ $driverdefaults{Pg} };
-
+  for (keys %{ $driverdefaults{Pg} }) { $form->{$_} = $driverdefaults{$form->{dbdriver}}{$_} }
+  
 }
   
 
@@ -1087,75 +1115,57 @@ sub dbselect_source {
 
   &dbdriver_defaults;
 
-  $msg{Pg} = $locale->text('Leave host and port field empty unless you want to make a remote connection.');
-  $msg{PgPP} = $msg{Pg};
-  $msg{Oracle} = $locale->text('You must enter a host and port for local and remote connections!');
-  
-
  $form->{title} = "SQL-Ledger ".$locale->text('Accounting')." / ".$locale->text('Database Administration');
   
-
+  $form->{login} = "root login";
   $form->header;
 
   print qq|
 <body class=admin>
 
-
 <center>
 <h2>$form->{title}</h2>
 
 <form method=post action=$form->{script}>
 
 <table>
-<tr><td>
-
-<table>
-
-  <tr class=listheading>
-    <th colspan=4>|.$locale->text('Database').qq|</th>
-  </tr>
-
-<input type=hidden name=dbdriver value=$form->{dbdriver}>
-
-  <tr><td>
-   <table>
-
   <tr>
+    <td>
+      <table>
+	<tr class=listheading>
+	  <th colspan=4>|.$locale->text('Database').qq|</th>
+	</tr>
+	<input type=hidden name=dbdriver value=$form->{dbdriver}>
+	<tr>
+	  <td>
+	    <table>
+	      <tr>
+		<th align=right>|.$locale->text('Host').qq|</th>
+		<td><input name=dbhost size=25 value=$form->{dbhost}></td>
+		<th align=right>|.$locale->text('Port').qq|</th>
+		<td><input name=dbport size=5 value=$form->{dbport}></td>
+	      </tr>
+	      <tr>
+		<th align=right>|.$locale->text('User').qq|</th>
+		<td><input name=dbuser size=10 value=$form->{dbuser}></td>
+		<th align=right>|.$locale->text('Password').qq|</th>
+		<td><input type=password name=dbpasswd size=10></td>
+	      </tr>
+	      <tr>
 
-    <th align=right>|.$locale->text('Host').qq|</th>
-    <td><input name=dbhost size=25 value=$form->{dbhost}></td>
-    <th align=right>|.$locale->text('Port').qq|</th>
-    <td><input name=dbport size=5 value=$form->{dbport}></td>
+		<th align=right>$form->{connectstring}</th>
+		<td colspan=3><input name=dbdefault size=10 value=$form->{dbdefault}></td>
 
-  </tr>
+	      </tr>
+	    </table>
+	  </td>
+	</tr>
+      </table>
 
-  <tr>
-
-    <th align=right>|.$locale->text('User').qq|</th>
-    <td><input name=dbuser size=10 value=$form->{dbuser}></td>
-    <th align=right>|.$locale->text('Password').qq|</th>
-    <td><input type=password name=dbpasswd size=10></td>
-
-  </tr>
-  
-  <tr>
-
-    <th align=right>$form->{connectstring}</th>
-    <td colspan=3><input name=dbdefault size=10 value=$form->{dbdefault}></td>
-
-  </tr>
-
-</table>
-
-</td></tr>
-</table>
-
-<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&root=$form->{root}&rpw=$form->{rpw}">
-<input type=hidden name=root value="$form->{root}">
+<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&sessionid=$form->{sessionid}">
 <input type=hidden name=path value=$form->{path}>
-<input type=hidden name=rpw value=$form->{rpw}>
+<input type=hidden name=sessionid value=$form->{sessionid}>
 
-<hr size=3 noshade>
 <br>
 
 <input type=submit class=submit name=action value="|.$locale->text('Create Dataset').qq|">
@@ -1164,15 +1174,13 @@ sub dbselect_source {
 
 </form>
 
-</td></tr>
+    </td>
+  </tr>
 </table>
 
 <p>|.$locale->text('This is a preliminary check for existing sources. Nothing will be created or deleted at this stage!')
 
 .qq|
-<br>$msg{$form->{dbdriver}}
-
-
 </body>
 </html>
 |;
@@ -1193,6 +1201,7 @@ sub update_dataset {
 
   $form->{title} = "SQL-Ledger ".$locale->text('Accounting')." ".$locale->text('Database Administration')." / ".$locale->text('Update Dataset');
   
+  $form->{login} = "root login";
   $form->header;
 
   print qq|
@@ -1205,8 +1214,8 @@ sub update_dataset {
 
 
   foreach $key (sort keys %needsupdate) {
-    if ($needsupdate{$key} lt $form->{dbversion}) {
-      $upd .= qq|<br><input name="db$key" type=checkbox value=1 checked> $key\n|;
+    if ($needsupdate{$key} ne $form->{dbversion}) {
+      $upd .= qq|<input name="db$key" class=checkbox type=checkbox value=1 checked> $key\n|;
       $form->{dbupdate} .= "db$key ";
     }
   }
@@ -1217,7 +1226,7 @@ sub update_dataset {
   if ($form->{dbupdate}) {
 
     print qq|
-<table>
+<table width=100%>
 <form method=post action=$form->{script}>
 
 <input type=hidden name=dbdriver value=$form->{dbdriver}>
@@ -1235,19 +1244,23 @@ sub update_dataset {
 
 $upd
 
-</td></tr>
-<tr><td>
+</td>
+</tr>
+<tr>
+<td>
 
 <input name=dbupdate type=hidden value="$form->{dbupdate}">
 
-<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&root=$form->{root}&rpw=$form->{rpw}">
+<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&sessionid=$form->{sessionid}">
 
-<input type=hidden name=root value="$form->{root}">
 <input type=hidden name=path value=$form->{path}>
-<input type=hidden name=rpw value=$form->{rpw}>
+<input type=hidden name=sessionid value=$form->{sessionid}>
 
 <input type=hidden name=nextsub value=dbupdate>
 
+<hr size=3 noshade>
+
+<br>
 <input type=submit class=submit name=action value="|.$locale->text('Continue').qq|">
 
 </td></tr>
@@ -1257,7 +1270,7 @@ $upd
 
   } else {
 
-    print $locale->text('All Datasets up to date!');
+    print $locale->text('DBA')." : $form->{dbuser} : " .$locale->text('All Datasets up to date!');
 
   }
   
@@ -1287,12 +1300,12 @@ sub create_dataset {
   foreach $item (sort grep /-chart\.sql/, readdir SQLDIR) {
     next if ($item eq 'Default-chart.sql');
     $item =~ s/-chart\.sql//;
-    push @charts, qq|<td><input name=chart class=radio type=radio value="$item">&nbsp;$item</td>|;
+    push @charts, qq|<input name=chart class=radio type=radio value="$item">$item|;
   }
   closedir SQLDIR;
 
   # add Default at beginning
-  unshift @charts, (qq|<td><input name=chart class=radio type=radio value="Default" checked>&nbsp;Default</td>|);
+  unshift @charts, qq|<input name=chart class=radio type=radio value="Default" checked>Default|;
 
   $selectencoding = qq|<option>
   <option value=SQL_ASCII>ASCII
@@ -1327,6 +1340,7 @@ sub create_dataset {
   
   $form->{title} = "SQL-Ledger ".$locale->text('Accounting')." ".$locale->text('Database Administration')." / ".$locale->text('Create Dataset');
   
+  $form->{login} = "root login";
   $form->header;
 
   print qq|
@@ -1338,7 +1352,7 @@ sub create_dataset {
 
 <form method=post action=$form->{script}>
 
-<table>
+<table width=100%>
   <tr class=listheading>
     <th colspan=2>&nbsp;</th>
   </tr>
@@ -1348,9 +1362,9 @@ sub create_dataset {
     <th align=right nowrap>|.$locale->text('Existing Datasets').qq|</th>
     <td>
 |;
-     
-    map { print "[&nbsp;$_&nbsp;] " } @dbsources;
-      
+
+    for (@dbsources) { print "[&nbsp;$_&nbsp;] " }
+    
     print qq|
     </td>
   </tr>
@@ -1361,7 +1375,7 @@ sub create_dataset {
     <td><input name=db></td>
 
   </tr>
-  
+
   <tr>
 
     <th align=right nowrap>|.$locale->text('Multibyte Encoding').qq|</th>
@@ -1371,21 +1385,24 @@ sub create_dataset {
  
   <tr>
 
-    <th align=right>|.$locale->text('Create Chart of Accounts').qq|</th>
+    <th align=right nowrap>|.$locale->text('Create Chart of Accounts').qq|</th>
     <td>
-      <table>\n|;
+      <table>
+|;
 
-      while (@charts) {
-	print qq|
-	<tr>\n|;
-	
-        map { print "$charts[$_]\n" } (0 .. 2);
+  while (@charts) {
+    print qq|
+        <tr>
+|;
 
-	print qq|
-	</tr>\n|;
-	
-	splice @charts, 0, 3;
-      }
+    for (0 .. 2) { print "<td>$charts[$_]</td>\n" }
+
+    print qq|
+        </tr>
+|;
+
+    splice @charts, 0, 3;
+  }
 
   print qq|
       </table>
@@ -1393,29 +1410,23 @@ sub create_dataset {
   </tr>
   <tr>
     <td colspan=2>
-      <hr size=3 noshade>
+    <hr size=3 noshade>
     </td>
   </tr>
 </table>
+|;
 
-<br>
-<input type=hidden name=dbdriver value=$form->{dbdriver}>
-<input type=hidden name=dbuser value=$form->{dbuser}>
-<input type=hidden name=dbhost value=$form->{dbhost}>
-<input type=hidden name=dbport value=$form->{dbport}>
-<input type=hidden name=dbpasswd value=$form->{dbpasswd}>
-<input type=hidden name=dbdefault value=$form->{dbdefault}>
-
-<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&root=$form->{root}&rpw=$form->{rpw}">
-
-<input type=hidden name=root value="$form->{root}">
-<input type=hidden name=path value=$form->{path}>
-<input type=hidden name=rpw value=$form->{rpw}>
-
+  $form->hide_form(qw(dbdriver dbuser dbhost dbport dbpasswd dbdefault path sessionid));
+  
+  print qq|
+  
+<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&sessionid=$form->{sessionid}">
 <input type=hidden name=nextsub value=dbcreate>
-
+  
+<br>
 <input type=submit class=submit name=action value="|.$locale->text('Continue').qq|">
 
+
 </form>
 
 </body>
@@ -1433,6 +1444,7 @@ sub dbcreate {
   
   $form->{title} = "SQL-Ledger ".$locale->text('Accounting')." ".$locale->text('Database Administration')." / ".$locale->text('Create Dataset');
 
+  $form->{login} = "root login";
   $form->header;
 
   print qq|
@@ -1448,9 +1460,8 @@ sub dbcreate {
 
 .qq|
 
-<input type=hidden name=root value="$form->{root}">
 <input type=hidden name=path value="$form->{path}">
-<input type=hidden name=rpw value="$form->{rpw}">
+<input type=hidden name=sessionid value=$form->{sessionid}>
 
 <input type=hidden name=nextsub value=list_users>
 
@@ -1469,7 +1480,7 @@ sub delete_dataset {
 
   if (@dbsources = User->dbsources_unused(\%$form, $memberfile)) {
     foreach $item (sort @dbsources) {
-      $dbsources .= qq|<input name=db class=radio type=radio value=$item>&nbsp;$item\n<br>|;
+      $dbsources .= qq|<input name=db class=radio type=radio value=$item>&nbsp;$item |;
     }
   } else {
     $form->error($locale->text('Nothing to delete!'));
@@ -1477,18 +1488,17 @@ sub delete_dataset {
 
   $form->{title} = "SQL-Ledger ".$locale->text('Accounting')." ".$locale->text('Database Administration')." / ".$locale->text('Delete Dataset');
 
+  $form->{login} = "root login";
   $form->header;
 
   print qq|
 <body class=admin>
 
-
-<center>
 <h2>$form->{title}</h2>
 
 <form method=post action=$form->{script}>
 
-<table>
+<table width=100%>
   <tr class=listheading>
     <th>|.$locale->text('The following Datasets are not in use and can be deleted').qq|</th>
   </tr>
@@ -1500,9 +1510,7 @@ sub delete_dataset {
   </tr>
   
   <tr><td>
-
-<hr size=3 noshade>
-<br>
+<p>
 <input type=hidden name=dbdriver value=$form->{dbdriver}>
 <input type=hidden name=dbuser value=$form->{dbuser}>
 <input type=hidden name=dbhost value=$form->{dbhost}>
@@ -1510,14 +1518,16 @@ sub delete_dataset {
 <input type=hidden name=dbpasswd value=$form->{dbpasswd}>
 <input type=hidden name=dbdefault value=$form->{dbdefault}>
 
-<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&root=$form->{root}&rpw=$form->{rpw}">
+<input name=callback type=hidden value="$form->{script}?action=list_users&path=$form->{path}&sessionid=$form->{sessionid}">
 
-<input type=hidden name=root value="$form->{root}">
 <input type=hidden name=path value="$form->{path}">
-<input type=hidden name=rpw value="$form->{rpw}">
+<input type=hidden name=sessionid value=$form->{sessionid}>
 
 <input type=hidden name=nextsub value=dbdelete>
 
+<hr size=3 noshade>
+
+<br>
 <input type=submit class=submit name=action value="|.$locale->text('Continue').qq|">
 
   </td></tr>
@@ -1525,11 +1535,6 @@ sub delete_dataset {
 
 </form>
 
-<p>|.$locale->text('Select a Dataset to delete and press "Continue"')
-
-.qq|
-
-
 </body>
 </html>
 |;
@@ -1547,6 +1552,7 @@ sub dbdelete {
 
   $form->{title} = "SQL-Ledger ".$locale->text('Accounting')." ".$locale->text('Database Administration')." / ".$locale->text('Delete Dataset');
 
+  $form->{login} = "root login";
   $form->header;
 
   print qq|
@@ -1562,9 +1568,8 @@ $form->{db} |.$locale->text('successfully deleted!')
 
 .qq|
 
-<input type=hidden name=root value="$form->{root}">
 <input type=hidden name=path value="$form->{path}">
-<input type=hidden name=rpw value="$form->{rpw}">
+<input type=hidden name=sessionid value=$form->{sessionid}>
 
 <input type=hidden name=nextsub value=list_users>
 
@@ -1582,8 +1587,8 @@ $form->{db} |.$locale->text('successfully deleted!')
 sub unlock_system {
 
   unlink "$userspath/nologin";
-
-  $form->{callback} = "$form->{script}?action=list_users&path=$form->{path}&root=$form->{root}&rpw=$root->{password}";
+  
+  $form->{callback} = "$form->{script}?action=list_users&path=$form->{path}&sessionid=$form->{sessionid}";
 
   $form->redirect($locale->text('Lockfile removed!'));
 
@@ -1592,10 +1597,10 @@ sub unlock_system {
 
 sub lock_system {
 
-  open(FH, ">$userspath/nologin") or $form->error($locale->text('Cannot create Lockfile!'));
+  open(FH, ">$userspath/nologin") or $form->error($locale->text('Cannot create Lock!'));
   close(FH);
-
-  $form->{callback} = "$form->{script}?action=list_users&path=$form->{path}&root=$form->{root}&rpw=$root->{password}";
+  
+  $form->{callback} = "$form->{script}?action=list_users&path=$form->{path}&sessionid=$form->{sessionid}";
 
   $form->redirect($locale->text('Lockfile created!'));
 
