@@ -7,6 +7,7 @@
 #     Web: http://www.sql-ledger.org
 #
 #  Contributors: Christopher Browne <cbrowne@acm.org>
+#                Tony Fraser <tony@sybaspace.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,8 +39,8 @@ use SL::Menu;
 sub display {
 
   $menu = new Menu "$menufile";
-  $menu = new Menu "custom_$menufile" if (-f "custom_$menufile");
-  $menu = new Menu "$form->{login}_$menufile" if (-f "$form->{login}_$menufile");
+  $menu->add_file("custom_$menufile") if -f "custom_$menufile";
+  $menu->add_file("$form->{login}_$menufile") if -f "$form->{login}_$menufile";
   
   @menuorder = $menu->access_control(\%myconfig);
 
@@ -70,9 +71,10 @@ sub display {
 sub section_menu {
 
   $menu = new Menu "$menufile", $form->{level};
-  $menu = new Menu "custom_$menufile", $form->{level} if (-f "custom_$menufile");
-  $menu = new Menu "$form->{login}_$menufile", $form->{level} if (-f "$form->{login}_$menufile");
-
+  
+  $menu->add_file("custom_$menufile") if -f "custom_$menufile";
+  $menu->add_file("$form->{login}_$menufile") if -f "$form->{login}_$menufile";
+  
   # build tiered menus
   @menuorder = $menu->access_control(\%myconfig, $form->{level});
 
