@@ -2,8 +2,7 @@
 # SQL-Ledger Accounting
 # Copyright (c) 2003
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
 # Contributors: Steve Doerr <sdoerr907@everestkc.net>
@@ -49,7 +48,8 @@ sub add {
   $ENV{REMOTE_ADDR} =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/;
   $form->{till} = $4;
 
-  map { $form->{partsgroup} .= "$_->{partsgroup}--$_->{translation}\n" } @{ $form->{all_partsgroup} };
+  $form->{partsgroup} = "";
+  for (@{ $form->{all_partsgroup} }) { $form->{partsgroup} .= "$_->{partsgroup}--$_->{translation}\n"; }
 
   &display_form;
 
@@ -63,14 +63,14 @@ sub openinvoices {
   
   $form->{sort} = 'transdate';
 
-  map { $form->{$_} = 'Y' } qw(open l_invnumber l_transdate l_name l_amount l_curr l_till l_subtotal);
+  for (qw(open l_invnumber l_transdate l_name l_amount l_curr l_till l_subtotal)) { $form->{$_} = 'Y'; }
 
   if ($myconfig{role} ne 'user') {
     $form->{l_employee} = 'Y';
   }
 
   $form->{title} = $locale->text('Open');
-  &ar_transactions;
+  transactions;
   
 }
 
@@ -90,7 +90,8 @@ sub edit {
 
   $form->{readonly} = ($myconfig{acs} =~ /POS--Sale/) ? 1 : 0;
 
-  map { $form->{partsgroup} .= "$_->{partsgroup}--$_->{translation}\n" } @{ $form->{all_partsgroup} };
+  $form->{partsgroup} = "";
+  for (@{ $form->{all_partsgroup} }) { $form->{partsgroup} .= "$_->{partsgroup}--$_->{translation}\n"; }
   
   &display_form;
 
@@ -100,21 +101,29 @@ sub edit {
 sub form_header {
 
   # set option selected
-  foreach $item (qw(AR currency)) {
-    $form->{"select$item"} =~ s/ selected//;
-    $form->{"select$item"} =~ s/option>\Q$form->{$item}\E/option selected>$form->{$item}/;
+  for (qw(AR currency)) {
+    $form->{"select$_"} =~ s/ selected//;
+    $form->{"select$_"} =~ s/option>\Q$form->{$_}\E/option selected>$form->{$_}/;
   }
 
-  foreach $item (qw(customer department employee)) {
-    $form->{"select$item"} = $form->unescape($form->{"select$item"});
-    $form->{"select$item"} =~ s/ selected//;
-    $form->{"select$item"} =~ s/(<option value="\Q$form->{$item}\E")/$1 selected/;
+  for (qw(customer department employee)) {
+    $form->{"select$_"} = $form->unescape($form->{"select$_"});
+    $form->{"select$_"} =~ s/ selected//;
+    $form->{"select$_"} =~ s/(<option value="\Q$form->{$_}\E")/$1 selected/;
   }
     
   $form->{exchangerate} = $form->format_amount(\%myconfig, $form->{exchangerate});
 
-  $exchangerate = "";
-  if ($form->{currency} ne $form->{defaultcurrency}) {
+  $exchangerate = qq|<tr>|;
+  $exchangerate .= qq|
+                <th align=right nowrap>|.$locale->text('Currency').qq|</th>
+		<td><select name=currency>$form->{selectcurrency}</select></td> | if $form->{defaultcurrency};
+  $exchangerate .= qq|
+                <input type=hidden name=selectcurrency value="$form->{selectcurrency}"> 
+		<input type=hidden name=defaultcurrency value=$form->{defaultcurrency}>
+|;
+		
+  if ($form->{defaultcurrency} && $form->{currency} ne $form->{defaultcurrency}) {
     if ($form->{forex}) {
       $exchangerate .= qq|<th align=right>|.$locale->text('Exchange Rate').qq|</th><td>$form->{exchangerate}<input type=hidden name=exchangerate value=$form->{exchangerate}></td>|;
     } else {
@@ -123,6 +132,7 @@ sub form_header {
   }
   $exchangerate .= qq|
 <input type=hidden name=forex value=$form->{forex}>
+</tr>
 |;
 
   if ($form->{selectcustomer}) {
@@ -142,15 +152,39 @@ sub form_header {
 		</td>
 	      </tr>
 | if $form->{selectdepartment};
-	
+
+   $employee = qq|
+	      <tr>
+	        <th align=right nowrap>|.$locale->text('Salesperson').qq|</th>
+		<td colspan=3><select name=employee>$form->{selectemployee}</select></td>
+		<input type=hidden name=selectemployee value="|.
+		$form->escape($form->{selectemployee},1).qq|">
+	      </tr>
+| if $form->{selectemployee};
+
+  if ($form->{change} != $form->{oldchange}) {
+    $form->{creditremaining} -= $form->{oldchange};
+  }
   $n = ($form->{creditremaining} < 0) ? "0" : "1";
 
+  if ($form->{business}) {
+    $business = qq|
+              <tr>
+	        <th align=right nowrap>|.$locale->text('Business').qq|</th>
+		<td>$form->{business}</td>
+		<td width=10></td>
+		<th align=right nowrap>|.$locale->text('Trade Discount').qq|</th>
+		<td>|.$form->format_amount(\%myconfig, $form->{tradediscount} * 100).qq| %</td>
+	      </tr>
+|;
+  }
+
   if ($form->{selectlanguage}) {
     if ($form->{language_code} ne $form->{oldlanguage_code}) {
       # rebuild partsgroup
-      $form->get_partsgroup(\%myconfig, { language_code => $form->{language_code} });
+      $form->get_partsgroup(\%myconfig, { language_code => $form->{language_code}, searchitems => 'nolabor'});
       $form->{partsgroup} = "";
-      map { $form->{partsgroup} .= "$_->{partsgroup}--$_->{translation}\n" } @{ $form->{all_partsgroup} };
+      for (@{ $form->{all_partsgroup} }) { $form->{partsgroup} .= "$_->{partsgroup}--$_->{translation}\n"; }
       $form->{oldlanguage_code} = $form->{language_code};
     }
 
@@ -167,37 +201,23 @@ sub form_header {
     <input type=hidden name=selectlanguage value="|.
     $form->escape($form->{selectlanguage},1).qq|">|;
   }
- 
 
+  $i = $form->{rowcount} + 1;
+  $focus = "partnumber_$i";
+  
   $form->header;
-
  
   print qq|
-<body>
+<body onLoad="document.forms[0].${focus}.focus()" />
 
-<form method=post action=$form->{script}>
+<form method=post action="$form->{script}">
+|;
 
-<input type=hidden name=id value=$form->{id}>
+  $form->hide_form(qw(id till type format printed title discount creditlimit creditremaining tradediscount business closedto locked oldtransdate customer_id oldcustomer));
 
-<input type=hidden name=till value=$form->{till}>
-
-<input type=hidden name=type value=$form->{type}>
-<input type=hidden name=media value=$form->{media}>
-<input type=hidden name=format value=$form->{format}>
-
-<input type=hidden name=printed value="$form->{printed}">
-
-<input type=hidden name=title value="$form->{title}">
+  print qq|
 <input type=hidden name=vc value="customer">
 
-<input type=hidden name=discount value=$form->{discount}>
-<input type=hidden name=creditlimit value=$form->{creditlimit}>
-<input type=hidden name=creditremaining value=$form->{creditremaining}>
-
-<input type=hidden name=closedto value=$form->{closedto}>
-<input type=hidden name=locked value=$form->{locked}>
-
-
 <table width=100%>
   <tr class=listtop>
     <th class=listtop>$form->{title}</font></th>
@@ -211,27 +231,26 @@ sub form_header {
 	    <table>
 	      <tr>
 		<th align=right nowrap>|.$locale->text('Customer').qq|</th>
-		<td>$customer</td>
-		<input type=hidden name=customer_id value=$form->{customer_id}>
-		<input type=hidden name=oldcustomer value="$form->{oldcustomer}"> 
+		<td colspan=3>$customer</td>
 	      </tr>
 	      <tr>
 	        <td></td>
 		<td colspan=3>
 		  <table>
 		    <tr>
-		      <th nowrap>|.$locale->text('Credit Limit').qq|</th>
+		      <th align=right nowrap>|.$locale->text('Credit Limit').qq|</th>
 		      <td>$form->{creditlimit}</td>
-		      <th nowrap>|.$locale->text('Remaining').qq|</th>
+		      <td width=10></td>
+		      <th align=right nowrap>|.$locale->text('Remaining').qq|</th>
 		      <td class="plus$n">|.$form->format_amount(\%myconfig, $form->{creditremaining}, 0, "0").qq|</font></td>
 		    </tr>
+		    $business
 		  </table>
 		</td>
 	      </tr>
-	      $discount
 	      <tr>
 		<th align=right nowrap>|.$locale->text('Record in').qq|</th>
-		<td><select name=AR>$form->{selectAR}</select></td>
+		<td colspan=3><select name=AR>$form->{selectAR}</select></td>
 		<input type=hidden name=selectAR value="$form->{selectAR}">
 	      </tr>
 	      $department
@@ -239,27 +258,11 @@ sub form_header {
 	  </td>
 	  <td>
 	    <table>
-	      <tr>
-	        <th align=right nowrap>|.$locale->text('Salesperson').qq|</th>
-		<td colspan=3><select name=employee>$form->{selectemployee}</select></td>
-		<input type=hidden name=selectemployee value="|.
-		$form->escape($form->{selectemployee},1).qq|">
-	      </tr>
-	      <tr>
-		<th align=right nowrap>|.$locale->text('Currency').qq|</th>
-		<td><select name=currency>$form->{selectcurrency}</select></td>
-		<input type=hidden name=selectcurrency value="$form->{selectcurrency}">
-		<input type=hidden name=defaultcurrency value=$form->{defaultcurrency}>
-		<input type=hidden name=fxgain_accno value=$form->{fxgain_accno}>
-		<input type=hidden name=fxloss_accno value=$form->{fxloss_accno}>
-		$exchangerate
-	      </tr>
+	      $employee
+	      $exchangerate
 	      $lang
 	    </table>
 	  </td>
-	<input type=hidden name=invnumber value=$form->{invnumber}>
-	<input type=hidden name=transdate value=$form->{transdate}>
-	<input type=hidden name=duedate value=$form->{duedate}>
 	</tr>
       </table>
     </td>
@@ -268,17 +271,12 @@ sub form_header {
     <td>
     </td>
   </tr>
-
-
-<input type=hidden name=taxaccounts value="$form->{taxaccounts}">
 |;
 
-  foreach $item (split / /, $form->{taxaccounts}) {
-    print qq|
-<input type=hidden name="${item}_rate" value="$form->{"${item}_rate"}">
-<input type=hidden name="${item}_description" value="$form->{"${item}_description"}">
-<input type=hidden name="${item}_taxnumber" value="$form->{"${item}_taxnumber"}">
-|;
+  $form->hide_form(qw(taxaccounts duedate invnumber transdate selectcurrency defaultcurrency));
+
+  for (split / /, $form->{taxaccounts}) {
+    $form->hide_form("${_}_rate", "${_}_description", "${_}_taxnumber");
   }
 
 }
@@ -288,48 +286,101 @@ sub form_header {
 sub form_footer {
 
   $form->{invtotal} = $form->{invsubtotal};
+
+  $form->{taxincluded} = ($form->{taxincluded}) ? "checked" : "";
+
+  $taxincluded = "";
+  if ($form->{taxaccounts}) {
+    $taxincluded = qq|
+              <tr height="5"></tr>
+	      <tr>
+	        <td align=right>
+		<input name=taxincluded class=checkbox type=checkbox value=1 $form->{taxincluded}></td><th align=left>|.$locale->text('Tax Included').qq|</th>
+	      </tr>
+|;
+  }
   
   if (!$form->{taxincluded}) {
     
-    foreach $item (split / /, $form->{taxaccounts}) {
-      if ($form->{"${item}_base"}) {
-	$form->{"${item}_total"} = $form->round_amount($form->{"${item}_base"} * $form->{"${item}_rate"}, 2);
-	$form->{invtotal} += $form->{"${item}_total"};
-	$form->{"${item}_total"} = $form->format_amount(\%myconfig, $form->{"${item}_total"}, 2, 0);
+    for (split / /, $form->{taxaccounts}) {
+      if ($form->{"${_}_base"}) {
+	$form->{"${_}_total"} = $form->round_amount($form->{"${_}_base"} * $form->{"${_}_rate"}, 2);
+	$form->{invtotal} += $form->{"${_}_total"};
+	$form->{"${_}_total"} = $form->format_amount(\%myconfig, $form->{"${_}_total"}, 2, 0);
 	
 	$tax .= qq|
 	      <tr>
-		<th align=right>$form->{"${item}_description"}</th>
-		<td align=right>$form->{"${item}_total"}</td>
+		<th align=right>$form->{"${_}_description"}</th>
+		<td align=right>$form->{"${_}_total"}</td>
 	      </tr>
-  |;
+|;
       }
     }
-  }
 
+    $form->{invsubtotal} = $form->format_amount(\%myconfig, $form->{invsubtotal}, 2, 0);
 
-  $form->{invsubtotal} = $form->format_amount(\%myconfig, $form->{invsubtotal}, 2, 0);
-
-  $subtotal = qq|
+    $subtotal = qq|
 	      <tr>
 		<th align=right>|.$locale->text('Subtotal').qq|</th>
 		<td align=right>$form->{invsubtotal}</td>
 	      </tr>
 |;
+  }
 
+  @column_index = qw(paid source memo AR_paid);
+
+  $column_data{paid} = "<th>".$locale->text('Amount')."</th>";
+  $column_data{source} = "<th>".$locale->text('Source')."</th>";
+  $column_data{memo} = "<th>".$locale->text('Memo')."</th>";
+  $column_data{AR_paid} = "<th>&nbsp;</th>";
+  
+  print qq|
+  <tr>
+    <td>
+      <table width=100%>
+	<tr valign=top>
+	  <td>
+	    <table>
+	      <tr>
+|;
+
+  for (@column_index) { print "$column_data{$_}\n"; }
+  
+  print qq|
+	      </tr>
+|;
 
   $totalpaid = 0;
   
-  $form->{paidaccounts} = 1;
-  $i = 1;
+  $form->{paidaccounts}++ if ($form->{"paid_$form->{paidaccounts}"});
+  for $i (1 .. $form->{paidaccounts}) {
   
-  $form->{"selectAR_paid_$i"} = $form->{selectAR_paid};
-  $form->{"selectAR_paid_$i"} =~ s/option>\Q$form->{"AR_paid_$i"}\E/option selected>$form->{"AR_paid_$i"}/;
+    $form->{"selectAR_paid_$i"} = $form->{selectAR_paid};
+    $form->{"selectAR_paid_$i"} =~ s/option>\Q$form->{"AR_paid_$i"}\E/option selected>$form->{"AR_paid_$i"}/;
   
   # format amounts
-  $totalpaid += $form->{"paid_$i"};
-  $form->{"paid_$i"} = $form->format_amount(\%myconfig, $form->{"paid_$i"}, 2);
-  $form->{"exchangerate_$i"} = $form->format_amount(\%myconfig, $form->{"exchangerate_$i"});
+    $totalpaid += $form->{"paid_$i"};
+    $form->{"paid_$i"} = $form->format_amount(\%myconfig, $form->{"paid_$i"}, 2);
+    $form->{"exchangerate_$i"} = $form->format_amount(\%myconfig, $form->{"exchangerate_$i"});
+
+
+    $column_data{paid} = qq|<td><input name="paid_$i" size=11 value=$form->{"paid_$i"}></td>|;
+    $column_data{source} = qq|<td><input name="source_$i" size=10 value="$form->{"source_$i"}"></td>|;
+    $column_data{memo} = qq|<td><input name="memo_$i" size=10 value="$form->{"memo_$i"}"></td>|;
+    $column_data{AR_paid} = qq|<td><select name="AR_paid_$i">$form->{"selectAR_paid_$i"}</select></td>|;
+
+    print qq|
+	      <tr>
+|;
+    for (@column_index) { print "$column_data{$_}\n"; }
+  
+    print qq|
+	      </tr>
+|;
+
+    $form->hide_form("cleared_$i", "exchangerate_$i", "forex_$i");
+
+  }
   
   $form->{change} = 0;
   if ($totalpaid > $form->{invtotal}) {
@@ -338,37 +389,11 @@ sub form_footer {
   $form->{oldchange} = $form->{change};
   $form->{change} = $form->format_amount(\%myconfig, $form->{change}, 2, 0);
   $form->{totalpaid} = $form->format_amount(\%myconfig, $totalpaid, 2);
-
  
   $form->{oldinvtotal} = $form->{invtotal};
   $form->{invtotal} = $form->format_amount(\%myconfig, $form->{invtotal}, 2, 0);
-  
  
   print qq|
-
-<input type=hidden name="exchangerate_$i" value=$form->{"exchangerate"}>
-<input type=hidden name="forex_$i" value=$form->{"forex_$i"}>
-
-  <tr>
-    <td>
-      <table width=100%>
-	<tr valign=bottom>
-	  <td>
-	    <table>
-	      <tr>
-	        <td></td>
-                <th>|.$locale->text('Paid').qq|</th>
-		<th>|.$locale->text('Source').qq|</th>
-		<th>|.$locale->text('Memo').qq|</th>
-		<th>|.$locale->text('Account').qq|</th>
-	      </tr>
-	      <tr>
-	        <td></td>
-		<td><input name="paid_$i" size=11 value=$form->{"paid_$i"}></td>
-		<td><input name="source_$i" size=10 value="$form->{"source_$i"}"></td>
-		<td><input name="memo_$i" size=10 value="$form->{"memo_$i"}"></td>
-	        <td><select name="AR_paid_$i">$form->{"selectAR_paid_$i"}</select></td>
-	      </tr>
 	      <tr>
 		<th align=right>|.$locale->text('Change').qq|</th>
 		<th>$form->{change}</th>
@@ -383,7 +408,7 @@ sub form_footer {
 		<th align=right>|.$locale->text('Total').qq|</th>
 		<td align=right>$form->{invtotal}</td>
 	      </tr>
-<input type=hidden name=taxincluded value=$form->{taxincluded}>
+	      $taxincluded
 	    </table>
 	  </td>
 	</tr>
@@ -391,21 +416,15 @@ sub form_footer {
     </td>
   </tr>
  
-<input type=hidden name=paidaccounts value=$form->{paidaccounts}>
-<input type=hidden name=selectAR_paid value="$form->{selectAR_paid}">
-<input type=hidden name=oldinvtotal value=$form->{oldinvtotal}>
 <input type=hidden name=oldtotalpaid value=$totalpaid>
-
-<input type=hidden name=change value=$form->{change}>
-<input type=hidden name=oldchange value=$form->{oldchange}>
-
 <input type=hidden name=datepaid value=$form->{transdate}>
-<input type=hidden name=invtotal value=$form->{invtotal}>
 
 <tr>
   <td>
 |;
 
+  $form->hide_form(qw(paidaccounts selectAR_paid oldinvtotal change oldchange invtotal));
+  
   &print_options;
 
   print qq|
@@ -417,50 +436,63 @@ sub form_footer {
 </table>
 |;
 
-  $transdate = $form->datetonum($form->{transdate}, \%myconfig);
-  $closedto = $form->datetonum($form->{closedto}, \%myconfig);
- 
+  $transdate = $form->datetonum(\%myconfig, $form->{transdate});
+  $closedto = $form->datetonum(\%myconfig, $form->{closedto});
+
+# type=submit $locale->text('Update')
+# type=submit $locale->text('Print')
+# type=submit $locale->text('Post')
+# type=submit $locale->text('Print and Post')
+# type=submit $locale->text('Delete')
+
   if (! $form->{readonly}) {
+    %button = ('Update' => { ndx => 1, key => 'U', value => $locale->text('Update') },
+	       'Print' => { ndx => 2, key => 'P', value => $locale->text('Print') },
+	       'Post' => { ndx => 3, key => 'O', value => $locale->text('Post') },
+	       'Print and Post' => { ndx => 4, key => 'R', value => $locale->text('Print and Post') },
+	       'Delete' => { ndx => 5, key => 'D', value => $locale->text('Delete') },
+	      );
+   
     if ($transdate > $closedto) {
-      print qq|
-      <input class=submit type=submit name=action value="|.$locale->text('Update').qq|">
-      <input class=submit type=submit name=action value="|.$locale->text('Print').qq|">
-      <input class=submit type=submit name=action value="|.$locale->text('Post').qq|">|;
 
-      if ($form->{id}) {
-	print qq|
-      <input class=submit type=submit name=action value="|.$locale->text('Delete').qq|">|;
+      if (! $form->{id}) {
+	delete $button{'Delete'};
       }
-    
 
-      print "<p>\n";
+      delete $button{'Print and Post'} unless $latex;
+      
+      for (sort { $button{$a}->{ndx} <=> $button{$b}->{ndx} } keys %button) { $form->print_button(\%button, $_) }
+
+      print qq|<p>
+      <input type=text size=1 value="B" accesskey="B" title="[Alt-B]">\n|;
     
       if ($form->{partsgroup}) {
+	$form->{partsgroup} =~ s/\r//g;
+	$form->{partsgroup} = $form->quote($form->{partsgroup});
+
+	$spc = ($form->{path} =~ /lynx/) ? "." : " ";
 	print qq|
 <input type=hidden name=nextsub value=lookup_partsgroup>
 <input type=hidden name=partsgroup value="$form->{partsgroup}">|;
 
 	foreach $item (split /\n/, $form->{partsgroup}) {
-	  $item =~ s/
-//;
 	  ($partsgroup, $translation) = split /--/, $item;
 	  $item = ($translation) ? $translation : $partsgroup;
-	  print qq| <input class=submit type=submit name=action value=".$item">\n|;
+	  print qq| <input class=submit type=submit name=action value="$spc$item">\n| if $item;
 	}
       }
+
     }
   }
+
+  if ($form->{menubar}) {
+    require "$form->{path}/menu.pl";
+    &menubar;
+  }
+  
+  $form->hide_form(qw(rowcount callback path login sessionid));
   
   print qq|
-
-<input type=hidden name=rowcount value=$form->{rowcount}>
-
-<input name=callback type=hidden value="$form->{callback}">
-
-<input type=hidden name=path value=$form->{path}>
-<input type=hidden name=login value=$form->{login}>
-<input type=hidden name=sessionid value=$form->{sessionid}>
-
 </form>
 
 </body>
@@ -487,17 +519,20 @@ sub post {
 
   $form->isblank("exchangerate", $locale->text('Exchange rate missing!')) if ($form->{currency} ne $form->{defaultcurrency});
   
-  $paid = $form->parse_amount(\%myconfig, $form->{"paid_1"});
+  $paid = 0;
+  for (1 .. $form->{paidaccounts}) { $paid += $form->parse_amount(\%myconfig, $form->{"paid_$_"}); }
   $total = $form->parse_amount(\%myconfig, $form->{invtotal});
   
-  $form->{"paid_1"} = $form->{invtotal} if $paid > $total;
+  # deduct change from first payment
+  $form->{"paid_1"} = $form->format_amount(\%myconfig, $form->parse_amount(\%myconfig, $form->{"paid_1"}) - ($paid - $total), 2) if $paid > $total;
   
   ($form->{AR}) = split /--/, $form->{AR};
 
-  $form->{invnumber} = $form->update_defaults(\%myconfig, "sinumber") unless $form->{invnumber};
-  
-  $form->redirect($locale->text('Posted!')) if (IS->post_invoice(\%myconfig, \%$form));
-  $form->error($locale->text('Cannot post transaction!'));
+  if (IS->post_invoice(\%myconfig, \%$form)) {
+    $form->redirect($locale->text('Posted!'));
+  } else {
+    $form->error($locale->text('Cannot post transaction!'));
+  }
   
 }
 
@@ -509,7 +544,7 @@ sub display_row {
     
   $form->{invsubtotal} = 0;
 
-  map { $form->{"${_}_base"} = 0 } (split / /, $form->{taxaccounts});
+  for (split / /, $form->{taxaccounts}) { $form->{"${_}_base"} = 0; }
   
   $column_data{partnumber} = qq|<th class=listheading nowrap>|.$locale->text('Number').qq|</th>|;
   $column_data{description} = qq|<th class=listheading nowrap>|.$locale->text('Description').qq|</th>|;
@@ -525,7 +560,7 @@ sub display_row {
       <table width=100%>
 	<tr class=listheading>|;
 
-  map { print "\n$column_data{$_}" } @column_index;
+  for (@column_index) { print "\n$column_data{$_}"; };
 
   print qq|
         </tr>
@@ -536,18 +571,24 @@ sub display_row {
   
   for $i (1 .. $numrows) {
     # undo formatting
-    map { $form->{"${_}_$i"} = $form->parse_amount(\%myconfig, $form->{"${_}_$i"}) } qw(qty discount sellprice);
+    for (qw(qty discount sellprice)) { $form->{"${_}_$i"} = $form->parse_amount(\%myconfig, $form->{"${_}_$i"}); }
 
     ($dec) = ($form->{"sellprice_$i"} =~ /\.(\d+)/);
     $dec = length $dec;
     $decimalplaces = ($dec > 2) ? $dec : 2;
 
     if (($form->{"qty_$i"} != $form->{"oldqty_$i"}) || ($form->{currency} ne $form->{oldcurrency})) {
-# check for a pricematrix
-      foreach $item (split / /, $form->{"pricematrix_$i"}) {
-	($q, $p) = split /:/, $item;
-	if ($p && $form->{"qty_$i"} > $q) {
-	  $form->{"sellprice_$i"} = $form->round_amount($p / $exchangerate, $decimalplaces);
+      # check for a pricematrix
+      @a = split / /, $form->{"pricematrix_$i"};
+      if (scalar @a) {
+	foreach $item (@a) {
+	  ($q, $p) = split /:/, $item;
+	  if (($p * 1) && ($form->{"qty_$i"} >= ($q * 1))) {
+	    ($dec) = ($p =~ /\.(\d+)/);
+	    $dec = length $dec;
+	    $decimalplaces = ($dec > 2) ? $dec : 2;
+	    $form->{"sellprice_$i"} = $form->round_amount($p / $exchangerate, $decimalplaces);
+	  }
 	}
       }
     }
@@ -562,20 +603,18 @@ sub display_row {
     $linetotal = $form->round_amount($form->{"sellprice_$i"} - $discount, $decimalplaces);
     $linetotal = $form->round_amount($linetotal * $form->{"qty_$i"}, 2);
 
-    map { $form->{"${_}_$i"} = $form->quote($form->{"${_}_$i"}) } qw(partnumber sku description partsgroup unit);
+    for (qw(partnumber sku description partsgroup unit)) { $form->{"${_}_$i"} = $form->quote($form->{"${_}_$i"}); }
     
-    $column_data{partnumber} = qq|<td><input name="partnumber_$i" size=20 value="$form->{"partnumber_$i"}"></td>|;
+    $column_data{partnumber} = qq|<td><input name="partnumber_$i" size=20 value="$form->{"partnumber_$i"}" accesskey="$i" title="[Alt-$i]"></td>|;
 
-    if (($rows = $form->numtextrows($form->{"description_$i"}, 25, 6)) > 1) {
-      $column_data{description} = qq|<td><textarea name="description_$i" rows=$rows cols=25 wrap=soft>$form->{"description_$i"}</textarea></td>|;
+    if (($rows = $form->numtextrows($form->{"description_$i"}, 40, 6)) > 1) {
+      $column_data{description} = qq|<td><textarea name="description_$i" rows=$rows cols=46 wrap=soft>$form->{"description_$i"}</textarea></td>|;
     } else {
-      $column_data{description} = qq|<td><input name="description_$i" size=30 value="$form->{"description_$i"}"></td>|;
+      $column_data{description} = qq|<td><input name="description_$i" size=48 value="$form->{"description_$i"}"></td>|;
     }
 
-    $column_data{partsgroup} = qq|<input type=hidden name="partsgroup_$i" value="$form->{"partsgroup_$i"}">|;
-
     $column_data{qty} = qq|<td align=right><input name="qty_$i" size=5 value=|.$form->format_amount(\%myconfig, $form->{"qty_$i"}).qq|></td>|;
-    $column_data{unit} = qq|<td><input type=hidden name="unit_$i" value="$form->{"unit_$i"}">$form->{"unit_$i"}</td>|;
+    $column_data{unit} = qq|<td>$form->{"unit_$i"}</td>|;
     $column_data{sellprice} = qq|<td align=right><input name="sellprice_$i" size=9 value=|.$form->format_amount(\%myconfig, $form->{"sellprice_$i"}, $decimalplaces).qq|></td>|;
     $column_data{linetotal} = qq|<td align=right>|.$form->format_amount(\%myconfig, $linetotal, 2).qq|</td>|;
     
@@ -587,25 +626,17 @@ sub display_row {
     print qq|
         <tr valign=top>|;
 
-    map { print "\n$column_data{$_}" } @column_index;
+    for (@column_index) { print "\n$column_data{$_}"; }
   
     print qq|
         </tr>
-
-<input type=hidden name="id_$i" value=$form->{"id_$i"}>
-<input type=hidden name="inventory_accno_$i" value=$form->{"inventory_accno_$i"}>
-<input type=hidden name="income_accno_$i" value=$form->{"income_accno_$i"}>
-<input type=hidden name="expense_accno_$i" value=$form->{"expense_accno_$i"}>
-<input type=hidden name="listprice_$i" value="$form->{"listprice_$i"}">
-<input type=hidden name="assembly_$i" value="$form->{"assembly_$i"}">
-<input type=hidden name="taxaccounts_$i" value="$form->{"taxaccounts_$i"}">
-<input type=hidden name="pricematrix_$i" value="$form->{"pricematrix_$i"}">
-<input type=hidden name="oldqty_$i" value="$form->{"qty_$i"}">
-<input type=hidden name="sku_$i" value="$form->{"sku_$i"}">
-
 |;
 
-    map { $form->{"${_}_base"} += $linetotal } (split / /, $form->{"taxaccounts_$i"});
+    $form->{"oldqty_$i"} = $form->{"qty_$i"};
+
+    for (qw(id listprice lastcost taxaccounts pricematrix oldqty sku partsgroup unit inventory_accno_id income_accno_id expense_accno_id)) { $form->hide_form("${_}_$i") }
+      
+    for (split / /, $form->{"taxaccounts_$i"}) { $form->{"${_}_base"} += $linetotal; }
   
     $form->{invsubtotal} += $linetotal;
   }
@@ -624,22 +655,20 @@ sub display_row {
 
 sub print {
   
-  $paid = $form->parse_amount(\%myconfig, $form->{"paid_1"});
-  $total = $form->parse_amount(\%myconfig, $form->{invtotal});
-
-  $form->{change} = 0;
-  if ($paid > $total) {
-    $form->{paid} = $total - $paid;
-    $form->{"paid_1"} = $form->format_amount(\%myconfig, $paid, 2, 0);
-    $form->{change} = $form->format_amount(\%myconfig, $paid - $total, 2, 0);
+  if (!$form->{invnumber}) {
+    $form->{invnumber} = $form->update_defaults(\%myconfig, "sinumber");
+    if ($form->{media} eq 'screen') {
+      &update;
+      exit;
+    }
   }
 
   $old_form = new Form;
-  map { $old_form->{$_} = $form->{$_} } keys %$form;
+  for (keys %$form) { $old_form->{$_} = $form->{$_}; }
   
-  map { $form->{$_} =~ s/--.*//g } qw(employee department);
+  for (qw(employee department)) { $form->{$_} =~ s/--.*//g; }
   $form->{invdate} = $form->{transdate};
-  $form->{invtime} = scalar localtime;
+  $form->{dateprinted} = scalar localtime;
 
   &print_form($old_form);
 
@@ -657,30 +686,42 @@ sub print_form {
     &update;
     exit;
   }
- 
- 
+
   &validate_items;
 
   &{ "$form->{vc}_details" };
 
   @a = ();
-  map { push @a, ("partnumber_$_", "description_$_") } (1 .. $form->{rowcount});
-  map { push @a, "${_}_description" } split / /, $form->{taxaccounts};
+  for (1 .. $form->{rowcount}) { push @a, ("partnumber_$_", "description_$_"); }
+  for (split / /, $form->{taxaccounts}) { push @a, "${_}_description"; }
   $form->format_string(@a);
 
   # format payment dates
-  map { $form->{"datepaid_$_"} = $locale->date(\%myconfig, $form->{"datepaid_$_"}) } (1 .. $form->{paidaccounts});
+  for (1 .. $form->{paidaccounts}) { $form->{"datepaid_$_"} = $locale->date(\%myconfig, $form->{"datepaid_$_"}); }
   
   IS->invoice_details(\%myconfig, \%$form);
 
-  map { $form->{$_} = $myconfig{$_} } (qw(company address tel fax businessnumber));
+  if ($form->parse_amount(\%myconfig, $form->{total}) <= 0) {
+    $form->{total} = 0;
+  } else {
+    $form->{change} = 0;
+  }
+
+  for (qw(company address tel fax businessnumber)) { $form->{$_} = $myconfig{$_}; }
   $form->{username} = $myconfig{name};
-  map { $form->{$_} =~ s/\\n/ /g } qw(company address);
+
+  $form->{address} =~ s/\\n/\n/g;
+  push @a, qw(company address tel fax businessnumber username);
+  $form->format_string(@a);
 
   $form->{templates} = "$myconfig{templates}";
   $form->{IN} = "$form->{type}.$form->{format}";
 
-  if ($form->{media} !~ /screen/) {
+  if ($form->{format} =~ /(postscript|pdf)/) {
+    $form->{IN} =~ s/$&$/tex/;
+  }
+  
+  if ($form->{media} ne 'screen') {
     $form->{OUT} = "| $printer{$form->{media}}";
   }
 
@@ -701,29 +742,38 @@ sub print_form {
   $old_form->{printed} = $form->{printed};
   
   # if we got back here restore the previous form
-  if ($form->{media} !~ /screen/) {
+  if ($form->{media} ne 'screen') {
     # restore and display form
-    map { $form->{$_} = $old_form->{$_} } keys %$old_form;
-    map { $form->{$_} = $form->parse_amount(\%myconfig, $form->{$_}) } qw(exchangerate);
+    for (keys %$old_form) { $form->{$_} = $old_form->{$_}; }
+    $form->{exchangerate} = $form->parse_amount(\%myconfig, $form->{exchangerate});
 
-    $form->{rowcount}--;
     for $i (1 .. $form->{paidaccounts}) {
-      map { $form->{"${_}_$i"} = $form->parse_amount(\%myconfig, $form->{"${_}_$i"}) } qw(paid exchangerate);
+      for (qw(paid exchangerate)) { $form->{"${_}_$i"} = $form->parse_amount(\%myconfig, $form->{"${_}_$i"}); }
     }
 
     delete $form->{pre};
 
-    &display_form;
-    exit;
+    if (! $form->{printandpost}) {
+      $form->{rowcount}--;
+      &display_form;
+    }
   }
 
 }
 
 
+sub print_and_post {
+
+  $form->error($locale->text('Select a printer!')) if ($form->{media} eq 'screen');
+  $form->{printandpost} = 1;
+  &print;
+  &post;
+
+}
+  
 sub lookup_partsgroup {
 
-  $form->{action} =~ s/
-//;
+  $form->{action} =~ s/\r//;
   $form->{action} = substr($form->{action}, 1);
 
   if ($form->{language_code}) {
@@ -763,10 +813,12 @@ sub print_options {
     <td>|.$locale->text('Screen').qq|</td>|;
 
   if (%printer) {
-    map { $media .= qq|
+    for (keys %printer) {
+      $media .= qq|
     <td><input class=radio type=radio name=media value="$_"></td>
     <td nowrap>$_</td>
-| } keys %printer;
+|;
+    }
   }
 
   $media =~ s/(value="\Q$form->{media}\E")/$1 checked/;
@@ -796,27 +848,30 @@ sub receipts {
   $form->{db} = 'ar';
   RP->paymentaccounts(\%myconfig, \%$form);
   
-  map { $paymentaccounts .= "$_->{accno} " } @{ $form->{PR} };
+  $paymentaccounts = "";
+  for (@{ $form->{PR} } ) { $paymentaccounts .= "$_->{accno} "; }
 
-  # accounting years
-  $form->{selectaccountingyear} = "<option>\n";
-  map { $form->{selectaccountingyear} .= qq|<option>$_\n| } @{ $form->{all_years} };
-  $form->{selectaccountingmonth} = "<option>\n";
-  map { $form->{selectaccountingmonth} .= qq|<option value=$_>|.$locale->text($form->{all_month}{$_}).qq|\n| } sort keys %{ $form->{all_month} };
+  if (@{ $form->{all_years} }) {
+    # accounting years
+    $form->{selectaccountingyear} = "<option>\n";
+    for (@{ $form->{all_years} }) { $form->{selectaccountingyear} .= qq|<option>$_\n|; }
+    $form->{selectaccountingmonth} = "<option>\n";
+    for (sort keys %{ $form->{all_month} }) { $form->{selectaccountingmonth} .= qq|<option value=$_>|.$locale->text($form->{all_month}{$_}).qq|\n|; }
 
-  $selectfrom = qq|
+    $selectfrom = qq|
         <tr>
 	<th align=right>|.$locale->text('Period').qq|</th>
 	<td colspan=3>
 	<select name=month>$form->{selectaccountingmonth}</select>
 	<select name=year>$form->{selectaccountingyear}</select>
-	<input name=interval class=radio type=radio value=0 checked>|.$locale->text('Current').qq|
-	<input name=interval class=radio type=radio value=1>|.$locale->text('Month').qq|
-	<input name=interval class=radio type=radio value=3>|.$locale->text('Quarter').qq|
-	<input name=interval class=radio type=radio value=12>|.$locale->text('Year').qq|
+	<input name=interval class=radio type=radio value=0 checked>&nbsp;|.$locale->text('Current').qq|
+	<input name=interval class=radio type=radio value=1>&nbsp;|.$locale->text('Month').qq|
+	<input name=interval class=radio type=radio value=3>&nbsp;|.$locale->text('Quarter').qq|
+	<input name=interval class=radio type=radio value=12>&nbsp;|.$locale->text('Year').qq|
 	</td>
       </tr>
 |;
+  }
 
   $form->header;
   
@@ -865,7 +920,15 @@ sub receipts {
 <input type=hidden name=sessionid value=$form->{sessionid}>
 
 <input type=submit class=submit name=action value="|.$locale->text('Continue').qq|">
+|;
 
+  if ($form->{menubar}) {
+    require "$form->{path}/menu.pl";
+    &menubar;
+  }
+
+  print qq|
+ 
 </form>
 
 </body>
