@@ -2,8 +2,7 @@
 # SQL-Ledger Accounting
 # Copyright (c) 2003
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 # 
 #
@@ -39,27 +38,22 @@ if (-f "$form->{path}/$form->{login}_arapprn.pl") {
 
 
 sub print {
-  
-  if ($form->{AP}) {
-    $form->{vc} = "vendor";
-    $form->{ARAP} = "AP";
-    $invfld = "vinumber";
-  }
-  if ($form->{AR}) {
-    $form->{vc} = "customer";
-    $form->{ARAP} = "AR";
-    $invfld = "sinumber";
-  }
-  
+
   if ($form->{media} !~ /screen/) {
     $form->error($locale->text('Select postscript or PDF!')) if $form->{format} !~ /(postscript|pdf)/;
+    $old_form = new Form;
+    for (keys %$form) { $old_form->{$_} = $form->{$_} }
+  }
+ 
+  if ($form->{formname} =~ /(check|receipt)/) {
+    if ($form->{media} eq 'screen') {
+      $form->error($locale->text('Select postscript or PDF!')) if $form->{format} !~ /(postscript|pdf)/;
+    }
   }
 
-  if ($form->{media} eq 'screen' && $form->{formname} =~ /(check|receipt)/) {
-    $form->error($locale->text('Select postscript or PDF!')) if $form->{format} !~ /(postscript|pdf)/;
-  }
-  
   if (! $form->{invnumber}) {
+    $invfld = 'sinumber';
+    $invfld = 'vinumber' if $form->{ARAP} eq 'AP';
     $form->{invnumber} = $form->update_defaults(\%myconfig, $invfld);
     if ($form->{media} eq 'screen') {
       if ($form->{media} eq 'screen') {
@@ -71,7 +65,7 @@ sub print {
 
   if ($form->{formname} =~ /(check|receipt)/) {
     if ($form->{media} ne 'screen') {
-      map { delete $form->{$_} } qw(action header);
+      for (qw(action header)) { delete $form->{$_} }
       $form->{invtotal} = $form->{oldinvtotal};
       
       foreach $key (keys %$form) {
@@ -97,13 +91,13 @@ sub print {
     
   }
 
-  &{ "print_$form->{formname}" }(1);
+  &{ "print_$form->{formname}" }($old_form, 1);
 
 }
 
 
 sub print_check {
-  my ($i) = @_;
+  my ($old_form, $i) = @_;
   
   $display_form = ($form->{display_form}) ? $form->{display_form} : "display_form";
 
@@ -119,9 +113,14 @@ sub print_check {
   }
 
   $form->{amount} = $form->{"paid_$i"};
-  map { $form->{$_} = $form->{"${_}_$i"} } qw(datepaid source memo);
 
-  
+  if (($form->{formname} eq 'check' && $form->{vc} eq 'customer') ||
+    ($form->{formname} eq 'receipt' && $form->{vc} eq 'vendor')) {
+    $form->{amount} =~ s/-//g;
+  }
+    
+  for (qw(datepaid source memo)) { $form->{$_} = $form->{"${_}_$i"} }
+
   &{ "$form->{vc}_details" };
   @a = qw(name address1 address2 city state zipcode country);
  
@@ -135,24 +134,24 @@ sub print_check {
   push(@{ $form->{paid} }, $form->{"paid_$i"});
 
   use SL::CP;
-  if ($form->{language_code}) {
-    $c = new CP $form->{language_code};
-  } else {
-    $c = new CP $myconfig->{countrycode};
-  } 
+  $c = CP->new(($form->{language_code}) ? $form->{language_code} : $myconfig{countrycode}); 
   $c->init;
   ($whole, $form->{decimal}) = split /\./, $form->parse_amount(\%myconfig, $form->{amount});
+
   $form->{decimal} .= "00";
   $form->{decimal} = substr($form->{decimal}, 0, 2);
-  $form->{text_amount} = $c->num2text($whole); 
+  $form->{text_decimal} = $c->num2text($form->{decimal} * 1);
+  $form->{text_amount} = $c->num2text($whole);
   
   ($form->{employee}) = split /--/, $form->{employee};
 
   $form->{notes} =~ s/^\s+//g;
   push @a, "notes";
 
-  map { $form->{$_} = $myconfig{$_} } (qw(company address tel fax businessnumber));
-  push @a, qw(company address tel fax businessnumber);
+  for (qw(company address tel fax businessnumber)) { $form->{$_} = $myconfig{$_} }
+  $form->{address} =~ s/\\n/\n/g;
+
+  push @a, qw(company address tel fax businessnumber text_amount text_decimal);
   
   $form->format_string(@a);
 
@@ -163,19 +162,9 @@ sub print_check {
     $form->{IN} =~ s/html$/tex/;
   }
 
-  if ($form->{media} !~ /(screen|queue)/) {
+  if ($form->{media} !~ /(screen)/) {
     $form->{OUT} = "| $printer{$form->{media}}";
     
-    $reference = $form->{invnumber};
-    
-    if ($form->{formname} =~ /(check|receipt)/) {
-      $form->{rowcount} = 1;
-      $form->{"id_1"} = $form->{id};
-      $form->{"checked_1"} = 1;
-      $form->{account} = $form->{"$form->{ARAP}_paid_$i"};
-      $reference = $form->{"source_$i"};
-    }
-      
     if ($form->{printed} !~ /$form->{formname}/) {
 
       $form->{printed} .= " $form->{formname}";
@@ -185,62 +174,18 @@ sub print_check {
     }
 
     %audittrail = ( tablename   => lc $form->{ARAP},
-                    reference   => $reference,
+                    reference   => $form->{invnumber},
 		    formname    => $form->{formname},
 		    action      => 'printed',
 		    id          => $form->{id} );
     
     %status = ();
-    map { $status{$_} = $form->{$_} } qw(printed queued audittrail);
+    for (qw(printed audittrail)) { $status{$_} = $form->{$_} }
     
     $status{audittrail} .= $form->audittrail("", \%myconfig, \%audittrail);
 
   }
 
-  if ($form->{media} eq 'queue') {
-    %queued = split / /, $form->{queued};
-    
-    $reference = $form->{invnumber};
- 
-    if ($form->{formname} =~ /(check|receipt)/) {
-      $form->{rowcount} = 1;
-      $form->{"id_1"} = $form->{id};
-      $form->{"checked_1"} = 1;
-      $form->{account} = $form->{"$form->{ARAP}_paid_$i"};
-      $reference = $form->{"source_$i"};
-    }
- 
-    if ($filename = $queued{$form->{formname}}) {
-      $form->{queued} =~ s/$form->{formname} $filename//;
-      unlink "$spool/$filename";
-      $filename =~ s/\..*$//g;
-    } else {
-      $filename = time;
-      $filename .= $$;
-    }
-
-    $filename .= ($form->{format} eq 'postscript') ? '.ps' : '.pdf';
-    $form->{OUT} = ">$spool/$filename";
-
-    $form->{queued} .= " $form->{formname} $filename";
-    $form->{queued} =~ s/^ //;
-
-    # save status
-    $form->update_status(\%myconfig);
-
-    %audittrail = ( tablename   => lc $form->{ARAP},
-                    reference   => $reference,
-		    formname    => $form->{formname},
-		    action      => 'queued',
-		    id          => $form->{id} );
-
-    %status = ();
-    map { $status{$_} = $form->{$_} } qw(printed queued audittrail);
-
-    $status{audittrail} .= $form->audittrail("", \%myconfig, \%audittrail);
-
-  }
-
   $form->{fileid} = $invnumber;
   $form->{fileid} =~ s/(\s|\W)+//g;
 
@@ -250,7 +195,7 @@ sub print_check {
   
     $previousform = $form->unescape($form->{previousform});
 
-    map { delete $form->{$_} } keys %$form;
+    for (keys %$form) { delete $form->{$_} }
 
     foreach $item (split /&/, $previousform) {
       ($key, $value) = split /=/, $item, 2;
@@ -258,16 +203,16 @@ sub print_check {
       $form->{$key} = $value;
     }
 
-    map { $form->{$_} = $form->parse_amount(\%myconfig, $form->{$_}) } qw(exchangerate creditlimit creditremaining);
+    for (qw(exchangerate creditlimit creditremaining)) { $form->{$_} = $form->parse_amount(\%myconfig, $form->{$_}) }
 
-    map { $form->{"amount_$_"} = $form->parse_amount(\%myconfig, $form->{"amount_$_"}) } (1 .. $form->{rowcount});
-    map { $form->{"tax_$_"} = $form->parse_amount(\%myconfig, $form->{"tax_$_"}) } split / /, $form->{taxaccounts};
+    for (1 .. $form->{rowcount}) { $form->{"amount_$_"} = $form->parse_amount(\%myconfig, $form->{"amount_$_"}) }
+    for (split / /, $form->{taxaccounts}) { $form->{"tax_$_"} = $form->parse_amount(\%myconfig, $form->{"tax_$_"}) }
 
     for $i (1 .. $form->{paidaccounts}) {
-      map { $form->{"${_}_$i"} = $form->parse_amount(\%myconfig, $form->{"${_}_$i"}) } qw(paid exchangerate);
+      for (qw(paid exchangerate)) { $form->{"${_}_$i"} = $form->parse_amount(\%myconfig, $form->{"${_}_$i"}) }
     }
 
-    map { $form->{$_} = $status{$_} } qw(printed queued audittrail);
+    for (qw(printed audittrail)) { $form->{$_} = $status{$_} }
 
     &{ "$display_form" };
     
@@ -276,18 +221,20 @@ sub print_check {
 }
 
 
-sub print_receipt { &print_check; }
+sub print_receipt {
+  my ($old_form, $i) = @_;
+  
+  &print_check($old_form, $i);
+
+}
 
 
 sub print_transaction {
+  my ($old_form) = @_;
  
   $display_form = ($form->{display_form}) ? $form->{display_form} : "display_form";
  
-  if ($form->{media} !~ /screen/) {
-    $old_form = new Form;
-    map { $old_form->{$_} = $form->{$_} } keys %$form;
-  }
- 
+
   &{ "$form->{vc}_details" };
   @a = qw(name address1 address2 city state zipcode country);
   
@@ -296,11 +243,13 @@ sub print_transaction {
   foreach $i (1 .. $form->{rowcount} - 1) {
     ($form->{tempaccno}, $form->{tempaccount}) = split /--/, $form->{"$form->{ARAP}_amount_$i"};
     ($form->{tempprojectnumber}) = split /--/, $form->{"projectnumber_$i"};
+    $form->{tempdescription} = $form->{"description_$i"};
     
-    $form->format_string(qw(tempaccno tempaccount tempprojectnumber));
+    $form->format_string(qw(tempaccno tempaccount tempprojectnumber tempdescription));
     
     push(@{ $form->{accno} }, $form->{tempaccno});
     push(@{ $form->{account} }, $form->{tempaccount});
+    push(@{ $form->{description} }, $form->{tempdescription});
     push(@{ $form->{projectnumber} }, $form->{tempprojectnumber});
 
     push(@{ $form->{amount} }, $form->{"amount_$i"});
@@ -311,18 +260,27 @@ sub print_transaction {
 
   foreach $accno (split / /, $form->{taxaccounts}) {
     if ($form->{"tax_$accno"}) {
+      $form->format_string("${accno}_description");
+
       $tax += $form->parse_amount(\%myconfig, $form->{"tax_$accno"});
+
+      $form->{"${accno}_tax"} = $form->{"tax_$accno"};
       push(@{ $form->{tax} }, $form->{"tax_$accno"});
+      
       push(@{ $form->{taxdescription} }, $form->{"${accno}_description"});
-      push(@{ $form->{taxrate} }, $form->{"${accno}_rate"} * 100);
+
+      $form->{"${accno}_taxrate"} = $form->format_amount($myconfig, $form->{"${accno}_rate"} * 100);
+      push(@{ $form->{taxrate} }, $form->{"${accno}_taxrate"});
+      
       push(@{ $form->{taxnumber} }, $form->{"${accno}_taxnumber"});
     }
   }
     
- 
+  $tax = 0 if $form->{taxincluded};
+
   push @a, $form->{ARAP};
   $form->format_string(@a);
-  
+
   $form->{paid} = 0;
   for $i (1 .. $form->{paidaccounts} - 1) {
 
@@ -348,36 +306,35 @@ sub print_transaction {
     
   }
 
-
   $form->{invtotal} = $form->{subtotal} + $tax;
   $form->{total} = $form->{invtotal} - $form->{paid};
   
   use SL::CP;
-  if ($form->{language_code}) {
-    $c = new CP $form->{language_code};
-  } else {
-    $c = new CP $myconfig->{countrycode};
-  } 
+  $c = CP->new(($form->{language_code}) ? $form->{language_code} : $myconfig{countrycode}); 
   $c->init;
   ($whole, $form->{decimal}) = split /\./, $form->{invtotal};
+
   $form->{decimal} .= "00";
   $form->{decimal} = substr($form->{decimal}, 0, 2);
+  $form->{text_decimal} = $c->num2text($form->{decimal} * 1); 
   $form->{text_amount} = $c->num2text($whole); 
   
-  map { $form->{$_} = $form->format_amount(\%myconfig, $form->{$_}, 2) } qw(invtotal subtotal paid total);
+  for (qw(invtotal subtotal paid total)) { $form->{$_} = $form->format_amount(\%myconfig, $form->{$_}, 2) }
   
   ($form->{employee}) = split /--/, $form->{employee};
 
   if (exists $form->{longformat}) {
-    map { $form->{$_} = $locale->date(\%myconfig, $form->{$_}, $form->{longformat}) } ("duedate", "transdate");
+    for (qw(duedate transdate)) { $form->{$_} = $locale->date(\%myconfig, $form->{$_}, $form->{longformat}) }
   }
 
   $form->{notes} =~ s/^\s+//g;
-
+  
   @a = ("invnumber", "transdate", "duedate", "notes");
 
-  map { $form->{$_} = $myconfig{$_} } (qw(company address tel fax businessnumber));
-  push @a, qw(company address tel fax businessnumber);
+  for (qw(company address tel fax businessnumber)) { $form->{$_} = $myconfig{$_} }
+  $form->{address} =~ s/\\n/\n/g;
+
+  push @a, qw(company address tel fax businessnumber text_amount text_decimal);
   
   $form->format_string(@a);
 
@@ -390,7 +347,7 @@ sub print_transaction {
     $form->{IN} =~ s/html$/tex/;
   }
 
-  if ($form->{media} !~ /(screen|queue)/) {
+  if ($form->{media} !~ /(screen)/) {
     $form->{OUT} = "| $printer{$form->{media}}";
     
     if ($form->{printed} !~ /$form->{formname}/) {
@@ -401,7 +358,7 @@ sub print_transaction {
       $form->update_status(\%myconfig);
     }
 
-    $old_form->{printed} = $form->{printed};
+    $old_form->{printed} = $form->{printed} if %$old_form;
     
     %audittrail = ( tablename   => lc $form->{ARAP},
                     reference   => $form->{"invnumber"},
@@ -409,39 +366,7 @@ sub print_transaction {
 		    action      => 'printed',
 		    id          => $form->{id} );
     
-    $old_form->{audittrail} .= $form->audittrail("", \%myconfig, \%audittrail);
-
-  }
-
-  if ($form->{media} eq 'queue') {
-    %queued = split / /, $form->{queued};
-
-    if ($filename = $queued{$form->{formname}}) {
-      $form->{queued} =~ s/$form->{formname} $filename//;
-      unlink "$spool/$filename";
-      $filename =~ s/\..*$//g;
-    } else {
-      $filename = time;
-      $filename .= $$;
-    }
-
-    $filename .= ($form->{format} eq 'postscript') ? '.ps' : '.pdf';
-    $form->{OUT} = ">$spool/$filename";
-
-    $form->{queued} .= " $form->{formname} $filename";
-    $form->{queued} =~ s/^ //;
-
-    # save status
-    $form->update_status(\%myconfig);
-
-    $old_form->{queued} = $form->{queued};
-
-    %audittrail = ( tablename   => lc $form->{ARAP},
-                    reference   => $form->{invnumber},
-		    formname    => $form->{formname},
-		    action      => 'queued',
-		    id          => $form->{id} );
-    $old_form->{audittrail} .= $form->audittrail("", \%myconfig, \%audittrail);
+    $old_form->{audittrail} .= $form->audittrail("", \%myconfig, \%audittrail) if %$old_form;
 
   }
 
@@ -450,27 +375,28 @@ sub print_transaction {
 
   $form->parse_template(\%myconfig, $userspath);
 
-  if ($old_form) {
+  if (%$old_form) {
     $old_form->{invnumber} = $form->{invnumber};
     $old_form->{invtotal} = $form->{invtotal};
 
-    map { delete $form->{$_} } keys %$form;
-    map { $form->{$_} = $old_form->{$_} } keys %$old_form;
+    for (keys %$form) { delete $form->{$_} }
+    for (keys %$old_form) { $form->{$_} = $old_form->{$_} }
 
     if (! $form->{printandpost}) {
-      map { $form->{$_} = $form->parse_amount(\%myconfig, $form->{$_}) } qw(exchangerate creditlimit creditremaining);
+      for (qw(exchangerate creditlimit creditremaining)) { $form->{$_} = $form->parse_amount(\%myconfig, $form->{$_}) }
 
-      map { $form->{"amount_$_"} = $form->parse_amount(\%myconfig, $form->{"amount_$_"}) } (1 .. $form->{rowcount});
-      map { $form->{"tax_$_"} = $form->parse_amount(\%myconfig, $form->{"tax_$_"}) } split / /, $form->{taxaccounts};
+      for (1 .. $form->{rowcount}) { $form->{"amount_$_"} = $form->parse_amount(\%myconfig, $form->{"amount_$_"}) }
+      for (split / /, $form->{taxaccounts}) { $form->{"tax_$_"} = $form->parse_amount(\%myconfig, $form->{"tax_$_"}) }
 
       for $i (1 .. $form->{paidaccounts}) {
-	map { $form->{"${_}_$i"} = $form->parse_amount(\%myconfig, $form->{"${_}_$i"}) } qw(paid exchangerate);
+	for (qw(paid exchangerate)) { $form->{"${_}_$i"} = $form->parse_amount(\%myconfig, $form->{"${_}_$i"}) }
       }
     }
     
     &{ "$display_form" };
 
   }
+
 }
 
 
@@ -509,24 +435,30 @@ sub select_payment {
       <table width=100%>
 	<tr class=listheading>|;
 
-  map { print "\n$column_data{$_}" } @column_index;
+  for (@column_index) { print "\n$column_data{$_}" }
   
   print qq|
 	</tr>
 |;
 
+  $checked = "checked";
   foreach $i (1 .. $form->{paidaccounts} - 1) {
-   $checked = ($i == 1) ? "checked" : "";
 
-   map { $column_data{$_} = qq|<td>$form->{"${_}_$i"}</td>| } @column_index;
-   $column_data{ndx} = qq|<td><input name=ndx class=radio type=radio value=$i $checked></td>|;
-   $column_data{paid} = qq|<td align=right>$form->{"paid_$i"}</td>|;
+    for (@column_index) { $column_data{$_} = qq|<td>$form->{"${_}_$i"}</td>| }
+
+    $paid = $form->{"paid_$i"};
+    $ok = 1;
+
+    $column_data{ndx} = qq|<td><input name=ndx class=radio type=radio value=$i $checked></td>|;
+    $column_data{paid} = qq|<td align=right>$paid</td>|;
+
+    $checked = "";
     
     $j++; $j %= 2;
     print qq|
 	<tr class=listrow$j>|;
 
-    map { print "\n$column_data{$_}" } @column_index;
+    for (@column_index) { print "\n$column_data{$_}" }
 
     print qq|
 	</tr>
@@ -544,14 +476,22 @@ sub select_payment {
 </table>
 |;
 
-  $form->{nextsub} = "payment_selected";
+  for (qw(action nextsub)) { delete $form->{$_} }
 
-  $form->hide_form();
+  $form->hide_form;
   
   print qq|
 
 <br>
-<input class=submit type=submit name=action value="|.$locale->text('Continue').qq|">
+<input type=hidden name=nextsub value=payment_selected>
+|;
+
+  if ($ok) {
+    print qq|
+<input class=submit type=submit name=action value="|.$locale->text('Continue').qq|">|;
+  }
+
+  print qq|
 </form>
 
 </body>
@@ -562,16 +502,13 @@ sub select_payment {
 
 sub payment_selected {
 
-  &{ "print_$form->{formname}" }($form->{ndx});
+  &{ "print_$form->{formname}" }($form->{oldform}, $form->{ndx});
 
 }
 
 
 sub print_options {
 
-  $form->{PD}{$form->{formname}} = "selected";
-  $form->{DF}{$form->{format}} = "selected";
-
   if ($form->{selectlanguage}) {
     $form->{"selectlanguage"} = $form->unescape($form->{"selectlanguage"});
     $form->{"selectlanguage"} =~ s/ selected//;
@@ -581,61 +518,52 @@ sub print_options {
     $form->escape($form->{selectlanguage},1).qq|">|;
   }
   
-  $type = qq|<select name=formname>
-          <option value=transaction $form->{PD}{transaction}>|.$locale->text('Transaction');
+  $form->{selectformname} = $form->unescape($form->{selectformname});
+  $form->{selectformname} =~ s/ selected//;
+  $form->{selectformname} =~ s/(<option value="\Q$form->{formname}\E")/$1 selected/;
+  
+  $type = qq|<select name=formname>$form->{selectformname}</select>
+  <input type=hidden name=selectformname value="|.$form->escape($form->{selectformname},1).qq|">|;
 
-  if ($form->{AR}) {
-    $type .= qq|
-          <option value=receipt $form->{PD}{receipt}>|.$locale->text('Receipt').qq|</select>|;
-  }
-
-  if ($form->{AP}) {
-    $type .= qq|
-          <option value=check $form->{PD}{check}>|.$locale->text('Check').qq|</select>|;
-  }
-	  
   $media = qq|<select name=media>
-          <option value=screen>|.$locale->text('Screen');
+          <option value="screen">|.$locale->text('Screen');
 
+  $form->{selectformat} = qq|<option value="html">html\n|;
+  
   if (%printer && $latex) {
-    map { $media .= qq| 
-          <option value="$_">$_| } sort keys %printer;
+    for (sort keys %printer) { $media .= qq| 
+          <option value="$_">$_| }
   }
 
-  $format = qq|<select name=format>
-            <option value=html $form->{DF}{html}>html|;
-	    
   if ($latex) {
-# disable for now
-#    $media .= qq|
-#          <option value="queue">|.$locale->text('Queue');
-    $format .= qq|
-            <option value=postscript $form->{DF}{postscript}>|.$locale->text('Postscript').qq|
-	    <option value=pdf $form->{DF}{pdf}>|.$locale->text('PDF');
+    $form->{selectformat} .= qq|
+            <option value="postscript">|.$locale->text('Postscript').qq|
+	    <option value="pdf">|.$locale->text('PDF');
   }
 
-  $format .= qq|</select>|;
+  $format = qq|<select name=format>$form->{selectformat}</select>|;
+  $format =~ s/(<option value="\Q$form->{format}\E")/$1 selected/;
+  $format .= qq|
+  <input type=hidden name=selectformat value="|.$form->escape($form->{selectformat},1).qq|">|;
   $media .= qq|</select>|;
   $media =~ s/(<option value="\Q$form->{media}\E")/$1 selected/;
 
   print qq|
   <table width=100%>
     <tr>
-      <td>
-  $type
-  $lang
-  $format
-  $media
-     </td>
-     <td align=right>
+      <td>$type</td>
+      <td>$lang</td>
+      <td>$format</td>
+      <td>$media</td>
+      <td align=right width=90%>
   |;
 
   if ($form->{printed} =~ /$form->{formname}/) {
     print $locale->text('Printed').qq|<br>|;
   }
 
-  if ($form->{queued} =~ /$form->{formname}/) {
-    print $locale->text('Queued');
+  if ($form->{recurring}) {
+    print $locale->text('Scheduled');
   }
 
   print qq|
@@ -650,7 +578,7 @@ sub print_options {
 sub print_and_post {
 
   $form->error($locale->text('Select postscript or PDF!')) if $form->{format} !~ /(postscript|pdf)/;
-  $form->error($locale->text('Select Printer or Queue!')) if $form->{media} eq 'screen';
+  $form->error($locale->text('Select a Printer!')) if $form->{media} eq 'screen';
 
   $form->{printandpost} = 1;
   $form->{display_form} = "post";
@@ -658,3 +586,4 @@ sub print_and_post {
 
 }
 
+
