@@ -2,8 +2,7 @@
 # SQL-Ledger Accounting
 # Copyright (c) 2001
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
 # Contributors:
@@ -71,71 +70,43 @@ sub add {
   
   $form->{callback} = "$form->{script}?action=add&transfer=$form->{transfer}&path=$form->{path}&login=$form->{login}&sessionid=$form->{sessionid}" unless $form->{callback};
 
-  # we use this only to set a default date
-  GL->transaction(\%myconfig, \%$form);
-  
-  map { $form->{selectaccno} .= "<option>$_->{accno}--$_->{description}\n" } @{ $form->{all_accno} };
+  &create_links;
 
-  if (@{ $form->{all_projects} }) {
-    $form->{selectprojectnumber} = "<option>\n";
-    map { $form->{selectprojectnumber} .= qq|<option value="$_->{projectnumber}--$_->{id}">$_->{projectnumber}\n| } @{ $form->{all_projects} };
-  }
-
-  
   $form->{rowcount} = ($form->{transfer}) ? 2 : 9;
+  $form->{oldtransdate} = $form->{transdate};
+  $form->{focus} = "reference";
 
   # departments
-  $form->all_departments(\%myconfig);
-  if (@{ $form->{all_departments} }) {
+  if (@{ $form->{all_department} }) {
     $form->{selectdepartment} = "<option>\n";
 
-    map { $form->{selectdepartment} .= qq|<option value="$_->{description}--$_->{id}">$_->{description}\n| } (@{ $form->{all_departments} });
+    for (@{ $form->{all_department} }) { $form->{selectdepartment} .= qq|<option value="$_->{description}--$_->{id}">$_->{description}\n| }
   }
  
-  &display_form;
+  &display_form(1);
   
 }
 
 
 sub edit {
 
-  GL->transaction(\%myconfig, \%$form);
-
-  map { $form->{selectaccno} .= "<option>$_->{accno}--$_->{description}\n" } @{ $form->{all_accno} };
+  &create_links;
   
-  # projects
-  if (@{ $form->{all_projects} }) {
-    $form->{selectprojectnumber} = "<option>\n";
-    map { $form->{selectprojectnumber} .= qq|<option value="$_->{projectnumber}--$_->{id}">$_->{projectnumber}\n| } @{ $form->{all_projects} };
-  }
-
-  
-  # departments
-  $form->all_departments(\%myconfig);
-  if (@{ $form->{all_departments} }) {
-
-    $form->{department} = "$form->{department}--$form->{department_id}";
-
-    $form->{selectdepartment} = "<option>\n";
-
-    map { $form->{selectdepartment} .= qq|<option value="$_->{description}--$_->{id}">$_->{description}\n| } (@{ $form->{all_departments} });
-  }
- 
-  $form->{locked} = ($form->{revtrans}) ? '1' : ($form->datetonum($form->{transdate}, \%myconfig) <= $form->datetonum($form->{closedto}, \%myconfig));
+  $form->{locked} = ($form->{revtrans}) ? '1' : ($form->datetonum(\%myconfig, $form->{transdate}) <= $form->datetonum(\%myconfig, $form->{closedto}));
 
   # readonly
-  $form->{readonly} = 1 if $myconfig{acs} =~ /General Ledger--Add Transaction/; 
+  if (! $form->{readonly}) {
+    $form->{readonly} = 1 if $myconfig{acs} =~ /General Ledger--Add Transaction/; 
+  }
 
   $form->{title} = "Edit";
   
-  &form_header;
-
   $i = 1;
   foreach $ref (@{ $form->{GL} }) {
     $form->{"accno_$i"} = "$ref->{accno}--$ref->{description}";
 
     $form->{"projectnumber_$i"} = "$ref->{projectnumber}--$ref->{project_id}";
-    $form->{"fx_transaction_$i"} = $ref->{fx_transaction};
+    for (qw(fx_transaction source memo)) { $form->{"${_}_$i"} = $ref->{$_} }
     
     if ($ref->{amount} < 0) {
       $form->{totaldebit} -= $ref->{amount};
@@ -149,56 +120,96 @@ sub edit {
   }
 
   $form->{rowcount} = $i;
+  $form->{focus} = "debit_$i";
   
+  &form_header;
   &display_rows;
-
   &form_footer;
   
 }
 
 
 
+sub create_links {
+  
+  GL->transaction(\%myconfig, \%$form);
+
+  for (@{ $form->{all_accno} }) { $form->{selectaccno} .= "<option>$_->{accno}--$_->{description}\n" }
+  
+  # projects
+  if (@{ $form->{all_project} }) {
+    $form->{selectprojectnumber} = "<option>\n";
+    for (@{ $form->{all_project} }) { $form->{selectprojectnumber} .= qq|<option value="$_->{projectnumber}--$_->{id}">$_->{projectnumber}\n| }
+  }
+
+  # departments
+  if (@{ $form->{all_department} }) {
+    $form->{department} = "$form->{department}--$form->{department_id}";
+    $form->{selectdepartment} = "<option>\n";
+    for (@{ $form->{all_department} }) { $form->{selectdepartment} .= qq|<option value="$_->{description}--$_->{id}">$_->{description}\n| }
+  }
+ 
+}
+
+
 sub search {
 
   $form->{title} = $locale->text('General Ledger')." ".$locale->text('Reports');
   
+  $colspan = 5;
   $form->all_departments(\%myconfig);
   # departments
-  if (@{ $form->{all_departments} }) {
+  if (@{ $form->{all_department} }) {
     $form->{selectdepartment} = "<option>\n";
+    for (@{ $form->{all_department} }) { $form->{selectdepartment} .= qq|<option value="$_->{description}--$_->{id}">$_->{description}\n| }
+
+    $l_department = qq|<input name="l_department" class=checkbox type=checkbox value=Y> |.$locale->text('Department');
 
-    map { $form->{selectdepartment} .= qq|<option value="$_->{description}--$_->{id}">$_->{description}\n| } (@{ $form->{all_departments} });
- 
     $department = qq|
   	<tr>
 	  <th align=right nowrap>|.$locale->text('Department').qq|</th>
-	  <td colspan=3><select name=department>$form->{selectdepartment}</select></td>
+	  <td colspan=$colspan><select name=department>$form->{selectdepartment}</select></td>
 	</tr>
 |;
   }
 
+  if (@{ $form->{all_years} }) {
+    # accounting years
+    $form->{selectaccountingyear} = "<option>\n";
+    for (@{ $form->{all_years} }) { $form->{selectaccountingyear} .= qq|<option>$_\n| }
+    $form->{selectaccountingmonth} = "<option>\n";
+    for (sort keys %{ $form->{all_month} }) { $form->{selectaccountingmonth} .= qq|<option value=$_>|.$locale->text($form->{all_month}{$_}).qq|\n| }
 
-  # accounting years
-  $form->{selectaccountingyear} = "<option>\n";
-  map { $form->{selectaccountingyear} .= qq|<option>$_\n| } @{ $form->{all_years} };
-  $form->{selectaccountingmonth} = "<option>\n";
-  map { $form->{selectaccountingmonth} .= qq|<option value=$_>|.$locale->text($form->{all_month}{$_}).qq|\n| } sort keys %{ $form->{all_month} };
-
-  $selectfrom = qq|
+    $selectfrom = qq|
         <tr>
 	<th align=right>|.$locale->text('Period').qq|</th>
-	<td colspan=3>
+	<td colspan=$colspan>
 	<select name=month>$form->{selectaccountingmonth}</select>
 	<select name=year>$form->{selectaccountingyear}</select>
-	<input name=interval class=radio type=radio value=0 checked>|.$locale->text('Current').qq|
-	<input name=interval class=radio type=radio value=1>|.$locale->text('Month').qq|
-	<input name=interval class=radio type=radio value=3>|.$locale->text('Quarter').qq|
-	<input name=interval class=radio type=radio value=12>|.$locale->text('Year').qq|
+	<input name=interval class=radio type=radio value=0 checked>&nbsp;|.$locale->text('Current').qq|
+	<input name=interval class=radio type=radio value=1>&nbsp;|.$locale->text('Month').qq|
+	<input name=interval class=radio type=radio value=3>&nbsp;|.$locale->text('Quarter').qq|
+	<input name=interval class=radio type=radio value=12>&nbsp;|.$locale->text('Year').qq|
 	</td>
       </tr>
 |;
+  }
+
+  @a = ();
+  push @a, qq|<input name="l_id" class=checkbox type=checkbox value=Y> |.$locale->text('ID');
+  push @a, qq|<input name="l_transdate" class=checkbox type=checkbox value=Y checked> |.$locale->text('Date');
+  push @a, qq|<input name="l_reference" class=checkbox type=checkbox value=Y checked> |.$locale->text('Reference');
+  push @a, qq|<input name="l_description" class=checkbox type=checkbox value=Y checked> |.$locale->text('Description');
+  push @a, qq|<input name="l_notes" class=checkbox type=checkbox value=Y> |.$locale->text('Notes');
+  push @a, $l_department if $l_department;
+  push @a, qq|<input name="l_debit" class=checkbox type=checkbox value=Y checked> |.$locale->text('Debit');
+  push @a, qq|<input name="l_credit" class=checkbox type=checkbox value=Y checked> |.$locale->text('Credit');
+  push @a, qq|<input name="l_source" class=checkbox type=checkbox value=Y checked> |.$locale->text('Source');
+  push @a, qq|<input name="l_memo" class=checkbox type=checkbox value=Y> |.$locale->text('Memo');
+  push @a, qq|<input name="l_accno" class=checkbox type=checkbox value=Y checked> |.$locale->text('Account');
+  push @a, qq|<input name="l_gifi_accno" class=checkbox type=checkbox value=Y> |.$locale->text('GIFI');
+
 
-  
   $form->header;
 
   print qq|
@@ -219,17 +230,22 @@ sub search {
 	<tr>
 	  <th align=right>|.$locale->text('Reference').qq|</th>
 	  <td><input name=reference size=20></td>
+
+	  </tr>
+	  <tr>
 	  <th align=right>|.$locale->text('Source').qq|</th>
 	  <td><input name=source size=20></td>
+	  <th align=right>|.$locale->text('Memo').qq|</th>
+	  <td><input name=memo size=20></td>
 	</tr>
 	$department
 	<tr>
 	  <th align=right>|.$locale->text('Description').qq|</th>
-	  <td colspan=3><input name=description size=40></td>
+	  <td colspan=$colspan><input name=description size=60></td>
 	</tr>
 	<tr>
 	  <th align=right>|.$locale->text('Notes').qq|</th>
-	  <td colspan=3><input name=notes size=40></td>
+	  <td colspan=$colspan><input name=notes size=60></td>
 	</tr>
 	<tr>
 	  <th align=right>|.$locale->text('From').qq|</th>
@@ -246,13 +262,12 @@ sub search {
 	</tr>
 	<tr>
 	  <th align=right>|.$locale->text('Include in Report').qq|</th>
-	  <td colspan=3>
+	  <td colspan=$colspan>
 	    <table>
 	      <tr>
 		<td>
 		  <input name="category" class=radio type=radio value=X checked>&nbsp;|.$locale->text('All').qq|
 		  <input name="category" class=radio type=radio value=A>&nbsp;|.$locale->text('Asset').qq|
-		  <input name="category" class=radio type=radio value=C>&nbsp;|.$locale->text('Contra').qq|
 		  <input name="category" class=radio type=radio value=L>&nbsp;|.$locale->text('Liability').qq|
 		  <input name="category" class=radio type=radio value=Q>&nbsp;|.$locale->text('Equity').qq|
 		  <input name="category" class=radio type=radio value=I>&nbsp;|.$locale->text('Income').qq|
@@ -261,33 +276,20 @@ sub search {
 	      </tr>
 	      <tr>
 		<table>
+|;
+
+  while (@a) {
+    print qq|<tr>\n|;
+    for (1 .. 5) {
+      print qq|<td nowrap>|. shift @a;
+      print qq|</td>\n|;
+    }
+    print qq|</tr>\n|;
+  }
+
+  print qq|
 		  <tr>
-		    <td align=right><input name="l_id" class=checkbox type=checkbox value=Y></td>
-		    <td>|.$locale->text('ID').qq|</td>
-		    <td align=right><input name="l_transdate" class=checkbox type=checkbox value=Y checked></td>
-		    <td>|.$locale->text('Date').qq|</td>
-		    <td align=right><input name="l_reference" class=checkbox type=checkbox value=Y checked></td>
-		    <td>|.$locale->text('Reference').qq|</td>
-		    <td align=right><input name="l_description" class=checkbox type=checkbox value=Y checked></td>
-		    <td>|.$locale->text('Description').qq|</td>
-		    <td align=right><input name="l_notes" class=checkbox type=checkbox value=Y></td>
-		    <td>|.$locale->text('Notes').qq|</td>
-		  </tr>
-		  <tr>
-		    <td align=right><input name="l_debit" class=checkbox type=checkbox value=Y checked></td>
-		    <td>|.$locale->text('Debit').qq|</td>
-		    <td align=right><input name="l_credit" class=checkbox type=checkbox value=Y checked></td>
-		    <td>|.$locale->text('Credit').qq|</td>
-		    <td align=right><input name="l_source" class=checkbox type=checkbox value=Y checked></td>
-		    <td>|.$locale->text('Source').qq|</td>
-		    <td align=right><input name="l_accno" class=checkbox type=checkbox value=Y checked></td>
-		    <td>|.$locale->text('Account').qq|</td>
-		    <td align=right><input name="l_gifi_accno" class=checkbox type=checkbox value=Y></td>
-		    <td>|.$locale->text('GIFI').qq|</td>
-		  </tr>
-		  <tr>
-		    <td align=right><input name="l_subtotal" class=checkbox type=checkbox value=Y></td>
-		    <td>|.$locale->text('Subtotal').qq|</td>
+		    <td nowrap><input name="l_subtotal" class=checkbox type=checkbox value=Y> |.$locale->text('Subtotal').qq|</td>
 		  </tr>
 		</table>
 	      </tr>
@@ -302,14 +304,22 @@ sub search {
 </table>
 
 <input type=hidden name=nextsub value=generate_report>
+|;
 
-<input type=hidden name=path value=$form->{path}>
-<input type=hidden name=login value=$form->{login}>
-<input type=hidden name=sessionid value=$form->{sessionid}>
-
+  $form->hide_form(qw(path login sessionid));
+  
+  print qq|
 <br>
 <input class=submit type=submit name=action value="|.$locale->text('Continue').qq|">
 </form>
+|;
+
+  if ($form->{menubar}) {
+    require "$form->{path}/menu.pl";
+    &menubar;
+  }
+
+  print qq|
 
 </body>
 </html>
@@ -330,7 +340,6 @@ sub generate_report {
   $callback = "$form->{script}?action=generate_report&direction=$form->{direction}&oldsort=$form->{oldsort}&path=$form->{path}&login=$form->{login}&sessionid=$form->{sessionid}";
   
   %acctype = ( 'A' => $locale->text('Asset'),
-               'C' => $locale->text('Contra'),
                'L' => $locale->text('Liability'),
 	       'Q' => $locale->text('Equity'),
 	       'I' => $locale->text('Income'),
@@ -339,7 +348,7 @@ sub generate_report {
   
   $form->{title} = $locale->text('General Ledger');
   
-  $ml = ($form->{ml} =~ /(A|E)/) ? -1 : 1;
+  $ml = ($form->{category} =~ /(A|E)/) ? -1 : 1;
 
   unless ($form->{category} eq 'X') {
     $form->{title} .= " : ".$locale->text($acctype{$form->{category}});
@@ -361,6 +370,12 @@ sub generate_report {
     $option .= "\n<br>" if $option;
     $option .= $locale->text('Source')." : $form->{source}";
   }
+  if ($form->{memo}) {
+    $href .= "&memo=".$form->escape($form->{memo});
+    $callback .= "&memo=".$form->escape($form->{memo},1);
+    $option .= "\n<br>" if $option;
+    $option .= $locale->text('Memo')." : $form->{memo}";
+  }
   if ($form->{reference}) {
     $href .= "&reference=".$form->escape($form->{reference});
     $callback .= "&reference=".$form->escape($form->{reference},1);
@@ -424,10 +439,11 @@ sub generate_report {
   }
 
 
-  @columns = $form->sort_columns(qw(transdate id reference description notes source debit credit accno gifi_accno));
+  @columns = $form->sort_columns(qw(transdate id reference description notes source memo debit credit accno gifi_accno department));
+  pop @columns if $form->{department};
 
   if ($form->{link} =~ /_paid/) {
-    @columns = $form->sort_columns(qw(transdate id reference description notes source cleared debit credit accno gifi_accno));
+    @columns = $form->sort_columns(qw(transdate id reference description notes source memo cleared debit credit accno gifi_accno));
     $form->{l_cleared} = "Y";
   }
 
@@ -460,7 +476,9 @@ sub generate_report {
   $column_header{transdate} = "<th><a class=listheading href=$href&sort=transdate>".$locale->text('Date')."</a></th>";
   $column_header{reference} = "<th><a class=listheading href=$href&sort=reference>".$locale->text('Reference')."</a></th>";
   $column_header{source} = "<th><a class=listheading href=$href&sort=source>".$locale->text('Source')."</a></th>";
+  $column_header{memo} = "<th><a class=listheading href=$href&sort=memo>".$locale->text('Memo')."</a></th>";
   $column_header{description} = "<th><a class=listheading href=$href&sort=description>".$locale->text('Description')."</a></th>";
+  $column_header{department} = "<th><a class=listheading href=$href&sort=department>".$locale->text('Department')."</a></th>";
   $column_header{notes} = "<th class=listheading>".$locale->text('Notes')."</th>";
   $column_header{debit} = "<th class=listheading>".$locale->text('Debit')."</th>";
   $column_header{credit} = "<th class=listheading>".$locale->text('Credit')."</th>";
@@ -488,31 +506,33 @@ sub generate_report {
 	<tr class=listheading>
 |;
 
-map { print "$column_header{$_}\n" } @column_index;
+  for (@column_index) { print "$column_header{$_}\n" }
 
-print "
+  print "
         </tr>
 ";
   
   # add sort to callback
   $form->{callback} = "$callback&sort=$form->{sort}";
   $callback = $form->escape($form->{callback});
-  
+
+  $cml = 1;
   # initial item for subtotals
   if (@{ $form->{GL} }) {
     $sameitem = $form->{GL}->[0]->{$form->{sort}};
+    $cml = -1 if $form->{contra};
   }
   
   if (($form->{accno} || $form->{gifi_accno}) && $form->{balance}) {
 
-    map { $column_data{$_} = "<td>&nbsp;</td>" } @column_index;
-    $column_data{balance} = "<td align=right>".$form->format_amount(\%myconfig, $form->{balance} * $ml, 2, 0)."</td>";
+    for (@column_index) { $column_data{$_} = "<td>&nbsp;</td>" }
+    $column_data{balance} = "<td align=right>".$form->format_amount(\%myconfig, $form->{balance} * $ml * $cml, 2, 0)."</td>";
     
     $i++; $i %= 2;
     print qq|
         <tr class=listrow$i>
 |;
-    map { print "$column_data{$_}\n" } @column_index;
+    for (@column_index) { print "$column_data{$_}\n" }
     
     print qq|
         </tr>
@@ -523,7 +543,8 @@ print "
   $direction = ($form->{direction} eq 'ASC') ? "ASC" : "DESC";
   $form->sort_order();
   $href =~ s/direction=$form->{direction}/direction=$direction/;
-    
+
+  $i = 0;
   foreach $ref (@{ $form->{GL} }) {
 
     # if item ne sort print subtotal
@@ -544,47 +565,52 @@ print "
     $ref->{debit} = $form->format_amount(\%myconfig, $ref->{debit}, 2, "&nbsp;");
     $ref->{credit} = $form->format_amount(\%myconfig, $ref->{credit}, 2, "&nbsp;");
     
-    $column_data{id} = "<td>$ref->{id}</td>";
-    $column_data{transdate} = "<td>$ref->{transdate}</td>";
+    for (qw(id transdate)) { $column_data{$_} = "<td>$ref->{$_}</td>" }
+    
     $column_data{reference} = "<td><a href=$ref->{module}.pl?action=edit&id=$ref->{id}&path=$form->{path}&login=$form->{login}&sessionid=$form->{sessionid}&callback=$callback>$ref->{reference}</td>";
-    $column_data{description} = "<td>$ref->{description}&nbsp;</td>";
-    $column_data{source} = "<td>$ref->{source}&nbsp;</td>";
-    $column_data{notes} = "<td>$ref->{notes}&nbsp;</td>";
+
+    $ref->{notes} =~ s/\r?\n/<br>/g;
+    for (qw(description source memo notes department)) { $column_data{$_} = "<td>$ref->{$_}&nbsp;</td>" }
+    
     $column_data{debit} = "<td align=right>$ref->{debit}</td>";
     $column_data{credit} = "<td align=right>$ref->{credit}</td>";
+    
     $column_data{accno} = "<td><a href=$href&accno=$ref->{accno}&callback=$callback>$ref->{accno}</a></td>";
     $column_data{gifi_accno} = "<td><a href=$href&gifi_accno=$ref->{gifi_accno}&callback=$callback>$ref->{gifi_accno}</a>&nbsp;</td>";
-    $column_data{balance} = "<td align=right>".$form->format_amount(\%myconfig, $form->{balance} * $ml, 2, 0)."</td>";
+    $column_data{balance} = "<td align=right>".$form->format_amount(\%myconfig, $form->{balance} * $ml * $cml, 2, 0)."</td>";
     $column_data{cleared} = ($ref->{cleared}) ? "<td>*</td>" : "<td>&nbsp;</td>";
 
-    $i++; $i %= 2;
+    if ($ref->{id} != $sameid) {
+      $i++; $i %= 2;
+    }
     print "
         <tr class=listrow$i>";
-    map { print "$column_data{$_}\n" } @column_index;
+    for (@column_index) { print "$column_data{$_}\n" }
     print "</tr>";
     
+    $sameid = $ref->{id};
   }
 
 
   &gl_subtotal if ($form->{l_subtotal} eq 'Y');
 
 
-  map { $column_data{$_} = "<td>&nbsp;</td>" } @column_index;
+  for (@column_index) { $column_data{$_} = "<td>&nbsp;</td>" }
   
   $column_data{debit} = "<th align=right class=listtotal>".$form->format_amount(\%myconfig, $totaldebit, 2, "&nbsp;")."</th>";
   $column_data{credit} = "<th align=right class=listtotal>".$form->format_amount(\%myconfig, $totalcredit, 2, "&nbsp;")."</th>";
-  $column_data{balance} = "<th align=right class=listtotal>".$form->format_amount(\%myconfig, $form->{balance} * $ml, 2, 0)."</th>";
+  $column_data{balance} = "<th align=right class=listtotal>".$form->format_amount(\%myconfig, $form->{balance} * $ml * $cml, 2, 0)."</th>";
   
   print qq|
 	<tr class=listtotal>
 |;
 
-  map { print "$column_data{$_}\n" } @column_index;
+  for (@column_index) { print "$column_data{$_}\n" }
 
   $i = 1;
-  if ($myconfig{acs} !~ /GL--GL/) {
-    $button{'GL--Add Transaction'}{code} = qq|<input class=submit type=submit name=action value="|.$locale->text('GL Transaction').qq|"> |;
-    $button{'GL--Add Transaction'}{order} = $i++;
+  if ($myconfig{acs} !~ /General Ledger--General Ledger/) {
+    $button{'General Ledger--Add Transaction'}{code} = qq|<input class=submit type=submit name=action value="|.$locale->text('GL Transaction').qq|"> |;
+    $button{'General Ledger--Add Transaction'}{order} = $i++;
   }
   if ($myconfig{acs} !~ /AR--AR/) {
     $button{'AR--Add Transaction'}{code} = qq|<input class=submit type=submit name=action value="|.$locale->text('AR Transaction').qq|"> |;
@@ -616,14 +642,10 @@ print "
 <br>
 
 <form method=post action=$form->{script}>
-
-<input name=callback type=hidden value="$form->{callback}">
-
-<input type=hidden name=path value=$form->{path}>
-<input type=hidden name=login value=$form->{login}>
-<input type=hidden name=sessionid value=$form->{sessionid}>
 |;
 
+  $form->hide_form(qw(callback path login sessionid));
+  
   foreach $item (sort { $a->{order} <=> $b->{order} } %button) {
     print $item->{code};
   }
@@ -649,14 +671,14 @@ sub gl_subtotal {
   $subtotaldebit = $form->format_amount(\%myconfig, $subtotaldebit, 2, "&nbsp;");
   $subtotalcredit = $form->format_amount(\%myconfig, $subtotalcredit, 2, "&nbsp;");
   
-  map { $column_data{$_} = "<td>&nbsp;</td>" } @column_index;
+  for (@column_index) { $column_data{$_} = "<td>&nbsp;</td>" }
 
   $column_data{debit} = "<th align=right class=listsubtotal>$subtotaldebit</td>";
   $column_data{credit} = "<th align=right class=listsubtotal>$subtotalcredit</td>";
 
   
   print "<tr class=listsubtotal>";
-  map { print "$column_data{$_}\n" } @column_index;
+  for (@column_index) { print "$column_data{$_}\n" }
   print "</tr>";
 
   $subtotaldebit = 0;
@@ -669,20 +691,30 @@ sub gl_subtotal {
 
 sub update {
 
+  if ($form->{transdate} ne $form->{oldtransdate}) {
+    if ($form->{selectprojectnumber}) {
+      $form->all_projects(\%myconfig, undef, $form->{transdate});
+      if (@{ $form->{all_project} }) {
+	$form->{selectprojectnumber} = "<option>\n";
+	for (@{ $form->{all_project} }) { $form->{selectprojectnumber} .= qq|<option value="$_->{projectnumber}--$_->{id}">$_->{projectnumber}\n| }
+	$form->{selectprojectnumber} = $form->escape($form->{selectprojectnumber},1);
+      }
+    }
+    $form->{oldtransdate} = $form->{transdate};
+  }
+  
   @a = ();
   $count = 0;
-  @flds = qw(accno debit credit projectnumber);
+  @flds = qw(accno debit credit projectnumber fx_transaction source memo);
 
   for $i (1 .. $form->{rowcount}) {
     unless (($form->{"debit_$i"} eq "") && ($form->{"credit_$i"} eq "")) {
-      # take accno apart
-      ($form->{"accno_$i"}) = split(/--/, $form->{"accno_$i"});
-      map { $form->{"${_}_$i"} = $form->parse_amount(\%myconfig, $form->{"${_}_$i"}) } qw(debit credit);
+      for (qw(debit credit)) { $form->{"${_}_$i"} = $form->parse_amount(\%myconfig, $form->{"${_}_$i"}) }
       
       push @a, {};
       $j = $#a;
       
-      map { $a[$j]->{$_} = $form->{"${_}_$i"} } @flds;
+      for (@flds) { $a[$j]->{$_} = $form->{"${_}_$i"} }
       $count++;
     }
   }
@@ -690,11 +722,11 @@ sub update {
 
   for $i (1 .. $count) {
     $j = $i - 1;
-    map { $form->{"${_}_$i"} = $a[$j]->{$_} } @flds;
+    for (@flds) { $form->{"${_}_$i"} = $a[$j]->{$_} }
   }
 
   for $i ($count + 1 .. $form->{rowcount}) {
-    map { delete $form->{"${_}_$i"} } @flds;
+    for (@flds) { delete $form->{"${_}_$i"} }
   }
 
   $form->{rowcount} = $count + 1;
@@ -705,67 +737,108 @@ sub update {
 
 
 sub display_form {
+  my ($init) = @_;
 
   &form_header;
-  &display_rows;
+  &display_rows($init);
   &form_footer;
 
 }
 
 
 sub display_rows {
+  my ($init) = @_;
 
-  $form->{selectprojectnumber} = $form->unescape($form->{selectprojectnumber});
+  $form->{selectprojectnumber} = $form->unescape($form->{selectprojectnumber}) if $form->{selectprojectnumber};
   
   $form->{totaldebit} = 0;
   $form->{totalcredit} = 0;
 
   for $i (1 .. $form->{rowcount}) {
- 
-    $form->{totaldebit} += $form->{"debit_$i"};
-    $form->{totalcredit} += $form->{"credit_$i"};
 
-    map { $form->{"${_}_$i"} = ($form->{"${_}_$i"}) ? $form->format_amount(\%myconfig, $form->{"${_}_$i"}, 2) : "" } qw(debit credit);
+    $source = qq|
+    <td><input name="source_$i" size=10 value="$form->{"source_$i"}"></td>|;
+    $memo = qq|
+    <td><input name="memo_$i" value="$form->{"memo_$i"}"></td>|;
 
-    $selectaccno = $form->{selectaccno};
-    $selectaccno =~ s/option>\Q$form->{"accno_$i"}\E/option selected>$form->{"accno_$i"}/;
-
-    if ($form->{selectprojectnumber}) {
-      $selectprojectnumber = $form->{selectprojectnumber};
-      $selectprojectnumber =~ s/(<option value="$form->{"projectnumber_$i"}")/$1 selected/;
+    if ($init) {
+      $accno = qq|
+      <td><select name="accno_$i">$form->{selectaccno}</select></td>|;
       
-      $project = qq|
-  <td><select name="projectnumber_$i">$selectprojectnumber</select></td>|;
-    }
+      if ($form->{selectprojectnumber}) {
+	$project = qq|
+    <td><select name="projectnumber_$i">$form->{selectprojectnumber}</select></td>|;
+      }
     
-  
-    if ($form->{transfer}) {
-      $form->{"fx_transaction_$i"} = ($form->{"fx_transaction_$i"}) ? "checked" : "";
-      $fx_transaction = qq|
-  <td><input name="fx_transaction_$i" class=checkbox type=checkbox value=1 $form->{"fx_transaction_$i"}></td>
-|;
+      if ($form->{transfer}) {
+	$fx_transaction = qq|
+        <td><input name="fx_transaction_$i" class=checkbox type=checkbox value=1></td>
+    |;
+	}
+
     } else {
-      $fx_transaction = qq|
-    <input type=hidden name="fx_transaction_$i" value=$form->{"fx_transaction_$i"}>
-|;
+   
+      $form->{totaldebit} += $form->{"debit_$i"};
+      $form->{totalcredit} += $form->{"credit_$i"};
+
+      for (qw(debit credit)) { $form->{"${_}_$i"} = ($form->{"${_}_$i"}) ? $form->format_amount(\%myconfig, $form->{"${_}_$i"}, 2) : "" }
+
+      if ($i < $form->{rowcount}) {
+	
+	$accno = qq|<td>$form->{"accno_$i"}</td>|;
+
+	if ($form->{selectprojectnumber}) {
+	  $form->{"projectnumber_$i"} = "" if $form->{selectprojectnumber} !~ /$form->{"projectnumber_$i"}/;
+	  
+	  $project = $form->{"projectnumber_$i"};
+	  $project =~ s/--.*//;
+	  $project = qq|<td>$project</td>|;
+	}
+      
+	if ($form->{transfer}) {
+	  $checked = ($form->{"fx_transaction_$i"}) ? "1" : "";
+	  $x = ($checked) ? "x" : "";
+	  $fx_transaction = qq|
+      <td><input type=hidden name="fx_transaction_$i" value="$checked">$x</td>
+    |;
+	}
+	
+	$form->hide_form("accno_$i", "projectnumber_$i");
+	
+      } else {
+	
+	$accno = qq|
+      <td><select name="accno_$i">$form->{selectaccno}</select></td>|;
+
+	if ($form->{selectprojectnumber}) {
+	  $project = qq|
+      <td><select name="projectnumber_$i">$form->{selectprojectnumber}</select></td>|;
+	}
+      
+	if ($form->{transfer}) {
+	  $fx_transaction = qq|
+      <td><input name="fx_transaction_$i" class=checkbox type=checkbox value=1></td>
+    |;
+	}
+      }
     }
     
+    print qq|<tr valign=top>
+    $accno
+    $fx_transaction
+    <td><input name="debit_$i" size=12 value="$form->{"debit_$i"}" accesskey=$i></td>
+    <td><input name="credit_$i" size=12 value=$form->{"credit_$i"}></td>
+    $source
+    $memo
+    $project
+  </tr>
 
-    print qq|<tr>
-  <td><select name="accno_$i">$selectaccno</select></td>
-  $fx_transaction
-  <td><input name="debit_$i" size=12 value=$form->{"debit_$i"}></td>
-  <td><input name="credit_$i" size=12 value=$form->{"credit_$i"}></td>
-  $project
-</tr>
-
-|;
+  |;
   }
 
-
+  $form->hide_form(qw(rowcount selectaccno));
+  
   print qq|
-<input type=hidden name=rowcount value=$form->{rowcount}>
-<input type=hidden name=selectaccno value="$form->{selectaccno}">
 <input type=hidden name=selectprojectnumber value="|.$form->escape($form->{selectprojectnumber},1).qq|">|;
 
 }
@@ -790,7 +863,7 @@ sub form_header {
   $form->{selectdepartment} =~ s/ selected//;
   $form->{selectdepartment} =~ s/(<option value="\Q$form->{department}\E")/$1 selected/;
 
-  map { $form->{$_} = $form->quote($form->{$_}) } qw(reference description notes);
+  for (qw(reference description notes)) { $form->{$_} = $form->quote($form->{$_}) }
 
   if (($rows = $form->numtextrows($form->{description}, 50)) > 1) {
     $description = qq|<textarea name=description rows=$rows cols=50 wrap=soft>$form->{description}</textarea>|;
@@ -821,22 +894,20 @@ sub form_header {
 	  <th class=listheading>|.$locale->text('FX').qq|</th>
 |;
   }
+
+  $focus = ($form->{focus}) ? $form->{focus} : "debit_$form->{rowcount}";
   
   $form->header;
 
   print qq|
-<body>
+<body onload="document.forms[0].${focus}.focus()" />
 
 <form method=post action=$form->{script}>
+|;
 
-<input name=id type=hidden value=$form->{id}>
-
-<input type=hidden name=transfer value=$form->{transfer}>
-
-<input type=hidden name=selectaccno value="$form->{selectaccno}">
-
-<input type=hidden name=closedto value=$form->{closedto}>
-<input type=hidden name=locked value=$form->{locked}>
+  $form->hide_form(qw(id transfer selectaccno closedto locked oldtransdate recurring));
+  
+  print qq|
 <input type=hidden name=title value="$title">
 
 <table width=100%>
@@ -846,7 +917,7 @@ sub form_header {
   <tr height="5"></tr>
   <tr>
     <td>
-      <table width=100%>
+      <table>
 	<tr>
 	  <th align=right>|.$locale->text('Reference').qq|</th>
 	  <td><input name=reference size=20 value="$form->{reference}"></td>
@@ -873,6 +944,8 @@ sub form_header {
 	  $fx_transaction
 	  <th class=listheading>|.$locale->text('Debit').qq|</th>
 	  <th class=listheading>|.$locale->text('Credit').qq|</th>
+	  <th class=listheading>|.$locale->text('Source').qq|</th>
+	  <th class=listheading>|.$locale->text('Memo').qq|</th>
 	  $project
 	</tr>
 |;
@@ -882,11 +955,7 @@ sub form_header {
 
 sub form_footer {
 
-  ($dec) = ($form->{totaldebit} =~ /\.(\d+)/);
-  $dec = length $dec;
-  $decimalplaces = ($dec > 2) ? $dec : 2;
-  
-  map { $form->{$_} = $form->format_amount(\%myconfig, $form->{$_}, $decimalplaces, "&nbsp;") } qw(totaldebit totalcredit);
+  for (qw(totaldebit totalcredit)) { $form->{$_} = $form->format_amount(\%myconfig, $form->{$_}, 2, "&nbsp;") }
 
 
   $project = qq|
@@ -905,6 +974,8 @@ sub form_footer {
 	  $fx_transaction
 	  <th class=listtotal align=right>$form->{totaldebit}</th>
 	  <th class=listtotal align=right>$form->{totalcredit}</th>
+	  <th>&nbsp;</th>
+	  <th>&nbsp;</th>
 	  $project
         </tr>
       </table>
@@ -914,42 +985,50 @@ sub form_footer {
     <td><hr size=3 noshade></td>
   </tr>
 </table>
-
-<input type=hidden name=path value=$form->{path}>
-<input type=hidden name=login value=$form->{login}>
-<input type=hidden name=sessionid value=$form->{sessionid}>
-
-<input name=callback type=hidden value="$form->{callback}">
-
-<br>
 |;
 
-  $transdate = $form->datetonum($form->{transdate}, \%myconfig);
-  $closedto = $form->datetonum($form->{closedto}, \%myconfig);
+  $form->hide_form(qw(path login sessionid callback));
   
+  $transdate = $form->datetonum(\%myconfig, $form->{transdate});
+  $closedto = $form->datetonum(\%myconfig, $form->{closedto});
+
+# type=submit $locale->text('Update')
+# type=submit $locale->text('Post')
+# type=submit $locale->text('Schedule')
+# type=submit $locale->text('Post as new')
+# type=submit $locale->text('Delete')
+
   if (! $form->{readonly}) {
+
+    %button = ('Update' => { ndx => 1, key => 'U', value => $locale->text('Update') },
+               'Post' => { ndx => 3, key => 'O', value => $locale->text('Post') },
+	       'Post as new' => { ndx => 6, key => 'N', value => $locale->text('Post as new') },
+	       'Schedule' => { ndx => 7, key => 'H', value => $locale->text('Schedule') },
+	       'Delete' => { ndx => 8, key => 'D', value => $locale->text('Delete') },
+	      );
+    
+    %a = ();
     
     if ($form->{id}) {
-      print qq|<input class=submit type=submit name=action value="|.$locale->text('Update').qq|">
-|;
-
-      if (!$form->{locked}) {
-	print qq|
-	<input class=submit type=submit name=action value="|.$locale->text('Post').qq|">
-	<input class=submit type=submit name=action value="|.$locale->text('Delete').qq|">
-|;
+      for ('Update', 'Post as new', 'Schedule') { $a{$_} = 1 }
+      
+      if (! $form->{locked}) {
+	for ('Post', 'Delete') { $a{$_} = 1 }
       }
-
-      print qq|
-	<input class=submit type=submit name=action value="|.$locale->text('Post as new').qq|">
-|;
       
     } else {
       if ($transdate > $closedto) {
-	print qq|<input class=submit type=submit name=action value="|.$locale->text('Update').qq|">
-	<input class=submit type=submit name=action value="|.$locale->text('Post').qq|">|;
+	for ("Update", "Post", "Schedule") { $a{$_} = 1 }
       }
     }
+
+    for (keys %button) { delete $button{$_} if ! $a{$_} }
+    for (sort { $button{$a}->{ndx} <=> $button{$b}->{ndx} } keys %button) { $form->print_button(\%button, $_) }
+    
+  }
+  
+  if ($form->{recurring}) {
+    print qq|<div align=right>|.$locale->text('Scheduled').qq|</div>|;
   }
 
   if ($form->{menubar}) {
@@ -957,12 +1036,12 @@ sub form_footer {
     &menubar;
   }
   
-  print "
+  print qq|
   </form>
 
 </body>
 </html>
-";
+|;
   
 }
 
@@ -977,7 +1056,9 @@ sub delete {
 <form method=post action=$form->{script}>
 |;
 
-  $form->hide_form();
+  delete $form->{action};
+
+  $form->hide_form;
   
   print qq|
 <h2 class=confirm>|.$locale->text('Confirm!').qq|</h2>
@@ -993,74 +1074,53 @@ sub delete {
 
 sub yes {
 
-  $form->redirect($locale->text('Transaction deleted!')) if (GL->delete_transaction(\%myconfig, \%$form));
-  $form->error($locale->text('Cannot delete transaction!'));
+  if (GL->delete_transaction(\%myconfig, \%$form)) {
+    $form->redirect($locale->text('Transaction deleted!'));
+  } else {
+    $form->error($locale->text('Cannot delete transaction!'));
+  }
   
 }
 
 
 sub post {
 
-  # check if there is something in reference and date
-  $form->isblank("reference", $locale->text('Reference missing!'));
   $form->isblank("transdate", $locale->text('Transaction Date missing!'));
 
-  $transdate = $form->datetonum($form->{transdate}, \%myconfig);
-  $closedto = $form->datetonum($form->{closedto}, \%myconfig);
+  $transdate = $form->datetonum(\%myconfig, $form->{transdate});
+  $closedto = $form->datetonum(\%myconfig, $form->{closedto});
 
-  
   $form->error($locale->text('Cannot post transaction for a closed period!')) if ($transdate <= $closedto);
 
   # add up debits and credits
-  if (!$form->{adjustment}) {
-    for $i (1 .. $form->{rowcount}) {
-      $debit += $form->parse_amount(\%myconfig, $form->{"debit_$i"});
-      $credit += $form->parse_amount(\%myconfig, $form->{"credit_$i"});
-    }
+  for $i (1 .. $form->{rowcount}) {
+    $dr = $form->parse_amount(\%myconfig, $form->{"debit_$i"});
+    $cr = $form->parse_amount(\%myconfig, $form->{"credit_$i"});
     
-    if ($form->round_amount($debit, 2) != $form->round_amount($credit, 2)) {
-      &post_adjustment;
+    if ($dr && $cr) {
+      $form->error($locale->text('Cannot post transaction with a debit and credit entry for the same account!'));
+    }
+    $debit += $dr;
+    $credit += $cr;
+  }
+  
+  if ($form->round_amount($debit, 2) != $form->round_amount($credit, 2)) {
+    $form->error($locale->text('Out of balance transaction!'));
+  }
+
+  if (! $form->{repost}) {
+    if ($form->{id}) {
+      &repost;
       exit;
     }
   }
 
-  $form->redirect($locale->text('Transaction posted!')) if GL->post_transaction(\%myconfig, \%$form);
-  $form->error($locale->text('Cannot post transaction!'));
+  if (GL->post_transaction(\%myconfig, \%$form)) {
+    $form->redirect($locale->text('Transaction posted!'));
+  } else {
+    $form->error($locale->text('Cannot post transaction!'));
+  }
   
 }
 
 
-sub post_as_new {
-
-  $form->{id} = 0;
-  &post;
-
-}
-
-
-sub post_adjustment {
-
-  $form->header;
-
-  print qq|
-<body>
-
-<form method=post action=$form->{script}>
-
-<input type=hidden name=adjustment value=1>
-|;
-
-  $form->hide_form();
-  
-  print qq|
-<h2 class=confirm>|.$locale->text('Warning!').qq|</h2>
-
-<h4>|.$locale->text('Out of balance transaction!').qq|</h4>
-
-<input name=action class=submit type=submit value="|.$locale->text('Post').qq|">
-</form>
-|;
-
-}
-
-
