@@ -2,8 +2,7 @@
 # SQL-Ledger Accounting
 # Copyright (c) 2003
 #
-#  Author: Dieter Simader
-#   Email: dsimader@sql-ledger.org
+#  Author: DWS Systems Inc.
 #     Web: http://www.sql-ledger.org
 #
 #
@@ -64,7 +63,7 @@ sub reconciliation {
   RC->paymentaccounts(\%myconfig, \%$form);
 
   $selection = "";
-  map { $selection .= "<option>$_->{accno}--$_->{description}\n" } @{ $form->{PR} };
+  for (@{ $form->{PR} }) { $selection .= "<option>$_->{accno}--$_->{description}\n" }
 
   $form->{title} = $locale->text('Reconciliation');
 
@@ -73,35 +72,36 @@ sub reconciliation {
     $cleared = qq|
         <input type=hidden name=report value=1>
         <tr>
-	  <td align=right><input type=checkbox style=checkbox name=outstanding value=1 checked></td>
+	  <td align=right><input type=checkbox class=checkbox name=outstanding value=1 checked></td>
 	  <td>|.$locale->text('Outstanding').qq|</td>
-	  <td align=right><input type=checkbox style=checkbox name=cleared value=1></td>
+	  <td align=right><input type=checkbox class=checkbox name=cleared value=1></td>
 	  <td>|.$locale->text('Cleared').qq|</td>
 	</tr>
 |;
 
   }
 
+  if (@{ $form->{all_years} }) {
+    # accounting years
+    $form->{selectaccountingyear} = "<option>\n";
+    for (@{ $form->{all_years} }) { $form->{selectaccountingyear} .= qq|<option>$_\n| }
+    $form->{selectaccountingmonth} = "<option>\n";
+    for (sort keys %{ $form->{all_month} }) { $form->{selectaccountingmonth} .= qq|<option value=$_>|.$locale->text($form->{all_month}{$_}).qq|\n| }
 
-  # accounting years
-  $form->{selectaccountingyear} = "<option>\n";
-  map { $form->{selectaccountingyear} .= qq|<option>$_\n| } @{ $form->{all_years} };
-  $form->{selectaccountingmonth} = "<option>\n";
-  map { $form->{selectaccountingmonth} .= qq|<option value=$_>|.$locale->text($form->{all_month}{$_}).qq|\n| } sort keys %{ $form->{all_month} };
-
-  $selectfrom = qq|
+    $selectfrom = qq|
         <tr>
 	<th align=right>|.$locale->text('Period').qq|</th>
 	<td colspan=3>
 	<select name=month>$form->{selectaccountingmonth}</select>
 	<select name=year>$form->{selectaccountingyear}</select>
-	<input name=interval class=radio type=radio value=0 checked>|.$locale->text('Current').qq|
-	<input name=interval class=radio type=radio value=1>|.$locale->text('Month').qq|
-	<input name=interval class=radio type=radio value=3>|.$locale->text('Quarter').qq|
-	<input name=interval class=radio type=radio value=12>|.$locale->text('Year').qq|
+	<input name=interval class=radio type=radio value=0 checked>&nbsp;|.$locale->text('Current').qq|
+	<input name=interval class=radio type=radio value=1>&nbsp;|.$locale->text('Month').qq|
+	<input name=interval class=radio type=radio value=3>&nbsp;|.$locale->text('Quarter').qq|
+	<input name=interval class=radio type=radio value=12>&nbsp;|.$locale->text('Year').qq|
 	</td>
       </tr>
 |;
+  }
 
 
   $form->header;
@@ -139,7 +139,7 @@ sub reconciliation {
 	  <td>|.$locale->text('Detail').qq|</td>
 	</tr>
 	<tr>
-	  <td align=right><input type=checkbox style=checkbox name=fx_transaction value=1 checked></td>
+	  <td align=right><input type=checkbox class=checkbox name=fx_transaction value=1 checked></td>
 	  <td colspan=3>|.$locale->text('Include Exchange Rate Difference').qq|</td>
 	</tr>
       </table>
@@ -152,14 +152,22 @@ sub reconciliation {
 
 <br>
 <input type=hidden name=nextsub value=get_payments>
+|;
 
-<input type=hidden name=path value=$form->{path}>
-<input type=hidden name=login value=$form->{login}>
-<input type=hidden name=sessionid value=$form->{sessionid}>
+  $form->hide_form(qw(path login sessionid));
 
+  print qq|
 <input type=submit class=submit name=action value="|.$locale->text('Continue').qq|">
 
 </form>
+|;
+
+  if ($form->{menubar}) {
+    require "$form->{path}/menu.pl";
+    &menubar;
+  }
+
+  print qq|
 
 </body>
 </html>
@@ -193,9 +201,9 @@ sub get_payments {
 sub display_form {
   
   if ($form->{report}) {
-    @column_index = qw(transdate source name cleared credit debit);
+    @column_index = qw(transdate source name cleared debit credit);
   } else {
-    @column_index = qw(transdate source name cleared credit debit balance);
+    @column_index = qw(transdate source name cleared debit credit balance);
   }
   
   $column_header{cleared} = qq|<th>|.$locale->text('R').qq|</th>|;
@@ -203,14 +211,8 @@ sub display_form {
   $column_header{name} = "<th class=listheading>".$locale->text('Description')."</a></th>";
   $column_header{transdate} = "<th class=listheading>".$locale->text('Date')."</a></th>";
 
-  if ($form->{category} eq 'A') {
-    $column_header{debit} = "<th class=listheading>".$locale->text('Deposit')."</a></th>";
-    $column_header{credit} = "<th class=listheading>".$locale->text('Payment')."</a></th>";
-  } else {
-    $column_header{debit} = "<th class=listheading>".$locale->text('Decrease')."</a></th>";
-    $column_header{credit} = "<th class=listheading>".$locale->text('Increase')."</a></th>";
-  }
-
+  $column_header{debit} = "<th class=listheading>".$locale->text('Debit')."</a></th>";
+  $column_header{credit} = "<th class=listheading>".$locale->text('Credit')."</a></th>";
   $column_header{balance} = "<th class=listheading>".$locale->text('Balance')."</a></th>";
 
   if ($form->{fromdate}) {
@@ -245,7 +247,7 @@ sub display_form {
 	<tr class=listheading>
 |;
 
-  map { print "\n$column_header{$_}" } @column_index;
+  for (@column_index) { print "\n$column_header{$_}" }
 
   print qq|
         </tr>
@@ -263,16 +265,16 @@ sub display_form {
   $i = 0;
   $j = 0;
   
-  map { $column_data{$_} = "<td>&nbsp;</td>" } qw(cleared transdate source debit credit);
+  for (qw(cleared transdate source debit credit)) { $column_data{$_} = "<td>&nbsp;</td>" }
 
   if (! $form->{report}) {
-  $column_data{name} = qq|<td>|.$locale->text('Beginning Balance').qq|</td>|;
-  $column_data{balance} = "<td align=right>".$form->format_amount(\%myconfig, $balance, 2, 0)."</td>";
-  print qq|
+    $column_data{name} = qq|<td>|.$locale->text('Beginning Balance').qq|</td>|;
+    $column_data{balance} = "<td align=right>".$form->format_amount(\%myconfig, $balance, 2, 0)."</td>";
+    print qq|
 	<tr class=listrow$j>
 |;
 
-  map { print "\n$column_data{$_}" } @column_index;
+  for (@column_index) { print "\n$column_data{$_}" }
 
   print qq|
 	</tr>
@@ -290,10 +292,15 @@ sub display_form {
 
     $checked = ($ref->{cleared}) ? "checked" : "";
     
+    %temp = ();
+    if (!$ref->{fx_transaction}) {
+      for (qw(name source transdate)) { $temp{$_} = $ref->{$_} }
+    }
+      
     $column_data{name} = "<td>";
-    map { $column_data{name} .= "$_<br>" } @{ $ref->{name} };
+    for (@{ $temp{name} }) { $column_data{name} .= "$_<br>" }
     $column_data{name} .= "</td>";
-    $column_data{source} = qq|<td>$ref->{source}&nbsp;</td>
+    $column_data{source} = qq|<td>$temp{source}&nbsp;</td>
     <input type=hidden name="id_$i" value=$ref->{id}>|;
     
     $column_data{debit} = "<td>&nbsp;</td>";
@@ -353,7 +360,7 @@ sub display_form {
       }
     }
     
-    $column_data{transdate} = qq|<td>$ref->{transdate}&nbsp;</td>
+    $column_data{transdate} = qq|<td>$temp{transdate}&nbsp;</td>
     <input type=hidden name="transdate_$i" value=$ref->{transdate}>|;
 
     $j++; $j %= 2;
@@ -361,7 +368,7 @@ sub display_form {
 	<tr class=listrow$j>
 |;
 
-    map { print "\n$column_data{$_}" } @column_index;
+    for (@column_index) { print "\n$column_data{$_}" }
 
     print qq|
 	</tr>
@@ -372,7 +379,7 @@ sub display_form {
   $form->{rowcount} = $i;
   
   # print totals
-  map { $column_data{$_} = "<td>&nbsp;</td>" } @column_index;
+  for (@column_index) { $column_data{$_} = "<td>&nbsp;</td>" }
 
   $column_data{debit} = "<th class=listtotal align=right>".$form->format_amount(\%myconfig, $totaldebits, 2, "&nbsp;")."</th>";
   $column_data{credit} = "<th class=listtotal align=right>".$form->format_amount(\%myconfig, $totalcredits, 2, "&nbsp;")."</th>";
@@ -381,7 +388,7 @@ sub display_form {
 	<tr class=listtotal>
 |;
 
-  map { print "\n$column_data{$_}" } @column_index;
+  for (@column_index) { print "\n$column_data{$_}" }
  
   $form->{statementbalance} = $form->parse_amount(\%myconfig, $form->{statementbalance});
   $difference = $form->format_amount(\%myconfig, $form->{beginningbalance} + $cleared - $form->{statementbalance}, 2, 0);
@@ -433,21 +440,11 @@ sub display_form {
     <td><hr size=3 noshade></td>
   </tr>
 </table>
+|;
 
-<input type=hidden name=fx_transaction value=$form->{fx_transaction}>
-<input type=hidden name=summary value=$form->{summary}>
-
-<input type=hidden name=rowcount value=$form->{rowcount}>
-<input type=hidden name=accno value="$form->{accno}">
-<input type=hidden name=account value="$form->{account}">
-
-<input type=hidden name=fromdate value=$form->{fromdate}>
-<input type=hidden name=todate value=$form->{todate}>
-
-<input type=hidden name=path value=$form->{path}>
-<input type=hidden name=login value=$form->{login}>
-<input type=hidden name=sessionid value=$form->{sessionid}>
-
+  $form->hide_form(qw(fx_transaction summary rowcount accno account fromdate todate path login sessionid));
+  
+  print qq|
 <br>
 <input type=submit class=submit name=action value="|.$locale->text('Update').qq|">
 <input type=submit class=submit name=action value="|.$locale->text('Select all').qq|">
@@ -488,7 +485,7 @@ sub select_all {
   
   RC->payment_transactions(\%myconfig, \%$form);
 
-  map { $_->{cleared} = 1 } @{ $form->{PR} };
+  for (@{ $form->{PR} }) { $_->{cleared} = 1 }
 
   &display_form;
   
