@@ -22,16 +22,32 @@ $gzip = `gzip -V 2>&1`;            # gz decompression utility
 $tar = `tar --version 2>&1`;       # tar archiver
 $latex = `latex -version`;
 
-@source = ( "http://www.sql-ledger.org/source",
-            "http://www.sql-ledger.com/source",
-            "http://abacus.sql-ledger.org/source",
-	    "http://pluto.sql-ledger.org/source" );
+%checkversion = ( www => 3, abacus => 4, pluto => 5, neptune => 8 );
+
+%source = (
+	    1 => { url => "http://voxel.dl.sourceforge.net/sourceforge/sql-ledger", site => "New York, U.S.A", locale => us },
+            2 => { url => "http://easynews.dl.sourceforge.net/sourceforge/sql-ledger", site => "Arizona, U.S.A", locale => us },
+	    3 => { url => "http://www.sql-ledger.com/source", site => "California, U.S.A", locale => us },
+            4 => { url => "http://abacus.sql-ledger.com/source", site => "Toronto, Canada", locale => ca },
+	    5 => { url => "http://pluto.sql-ledger.com/source", site => "Edmonton, Canada", locale => ca },
+	    6 => { url => "http://ufpr.dl.sourceforge.net/sourceforge/sql-ledger", site =>"Brazil", locale => br },
+	    7 => { url => "http://surfnet.dl.sourceforge.net/sourceforge/sql-ledger", site => "The Netherlands", locale => nl },
+	    8 => { url => "http://neptune.sql-ledger.com/source", site => "Ireland", locale => ie },
+	    9 => { url => "http://kent.dl.sourceforge.net/sourceforge/sql-ledger", site => "U.K", locale => uk },
+	    10 => { url => "http://ovh.dl.sourceforge.net/sourceforge/sql-ledger", site => "France", locale => fr },
+	    11 => { url => "http://mesh.dl.sourceforge.net/sourceforge/sql-ledger", site => "Germany", locale => de },
+	    12 => { url => "http://citkit.dl.sourceforge.net/sourceforge/sql-ledger", site => "Russia", locale => ru },
+	    13 => { url => "http://optusnet.dl.sourceforge.net/sourceforge/sql-ledger", site => "Sidney, Australia", locale => au },
+	    14 => { url => "http://nchc.dl.sourceforge.net/sourceforge/sql-ledger", site => "Taiwan", locale => tw },
+	    15 => { url => "http://jaist.dl.sourceforge.net/sourceforge/sql-ledger", site => "Japan", locale => jp }
+	  );
 
 $userspath = "users";         # default for new installation
 
 eval { require "sql-ledger.conf"; };
 
 $filename = shift;
+chomp $filename;
 
 $newinstall = 1;
 
@@ -62,15 +78,14 @@ if (-f "VERSION") {
   chomp $version;
 
   $newinstall = !$version;
+
+  if (! -f "sql-ledger.conf") {
+    $newinstall = 1;
+  }
 }
 
-
-# is this windows? only used for downloading the right code
-$windows = ($^O =~ /MS/) ? '/windows' : '';
-
-
-$webowner = $<;
-$webgroup = $(;
+$webowner = "nobody";
+$webgroup = "nogroup";
 
 if ($httpd = `find /etc /usr/local/etc -type f -name 'httpd.conf'`) {
   chomp $httpd;
@@ -85,6 +100,10 @@ if ($httpd = `find /etc /usr/local/etc -type f -name 'httpd.conf'`) {
 
 }
 
+if ($confd = `find /etc /usr/local/etc -type d -name 'apache*/conf.d'`) {
+  chomp $confd;
+}
+
 system("tput clear");
 
 if ($filename) {
@@ -93,6 +112,7 @@ if ($filename) {
 
 # check for latest version
 &get_latest_version;
+chomp $latest_version;
 
 if (!$newinstall) {
 
@@ -101,10 +121,8 @@ if (!$newinstall) {
 }
 
 if ($version && $latest_version) {
-  if (!$filename && $version le $latest_version) {
-    if (substr($version, 0, rindex($version, ".")) eq substr($latest_version, 0, rindex($latest_version, "."))) {
-      $install .= "\n(u)pgrade to $latest_version\n";
-    }
+  if ($version lt $latest_version) {
+    $install .= "\n(u)pgrade to $latest_version\n";
   }
 }
 
@@ -145,6 +163,23 @@ if ($a !~ /d/) {
   
 }
 
+if ($a ne 'f') {
+  system("tput clear");
+
+  # choose site
+  foreach $item (sort { $a <=> $b } keys %source) {
+    $i++;
+    print qq|$i. $source{$item}{site}\n|;
+  }
+
+  $site = "1";
+
+  print qq|\nChoose Location [$site] : |;
+  $b = <STDIN>;
+  chomp $b;
+  $site = $b if $b;
+}
+
 if ($a eq 'd') {
   &download;
 }
@@ -180,13 +215,11 @@ sub get_latest_version {
   }
 
   if ($lwp) {
-    foreach $source (@source) {
-      $host = $source;
-      $host =~ s/(\w\/).*/$1/g;
-      chop $host;
-      print "\nTrying $host ... ";
+    foreach $source (qw(pluto www abacus neptune)) {
+      $url = $source{$checkversion{$source}}{url};
+      print "\n$source{$checkversion{$source}}{site} ... ";
 
-      $latest_version = LWP::Simple::get("$source/latest_version");
+      $latest_version = LWP::Simple::get("$url/latest_version");
       
       if ($latest_version) {
 	last;
@@ -200,15 +233,12 @@ sub get_latest_version {
       exit 1;
     }
 
-    foreach $source (@source) {
-      $host = $source;
-      $host =~ s/(\w\/).*/$1/g;
-      chop $host;
-      print "\nTrying $host ... ";
-      $ok = `lynx -dump -head $source/latest_version`;
-      if ($ok = ($ok =~ s/HTTP.*?200 OK//g)) {
-	$latest_version = `lynx -dump $source/latest_version`;
-	chomp $latest_version;
+    foreach $source (qw(pluto www abacus neptune)) {
+      $url = $source{$checkversion{$source}}{url};
+      print "\n$source{$checkversion{$source}}{site} ... ";
+      $ok = `lynx -dump -head $url/latest_version`;
+      if ($ok = ($ok =~ s/HTTP.*?200 //)) {
+	$latest_version = `lynx -dump $url/latest_version`;
 	last;
       } else {
 	print "not found";
@@ -217,7 +247,6 @@ sub get_latest_version {
     die unless $ok;
   }
 
-  chomp $latest_version;
   if ($latest_version) {
     print "ok\n";
     1;
@@ -229,33 +258,42 @@ sub get_latest_version {
 sub get_source_code {
 
   $err = 0;
- 
+
+  @order = ();
+  push @order, $site;
+  
+  for (sort { $a <=> $b } keys %source) {
+    push @order, $_;
+  }
+
   if ($latest_version) {
     # download it
+    chomp $latest_version;
     $latest_version = "sql-ledger-${latest_version}.tar.gz";
-    
+
     print "\nStatus\n";
     print "Downloading $latest_version .... ";
 
-    foreach $source (@source) {
-      $host = $source;
-      $host =~ s/(\w\/).*/$1/g;
-      chop $host;
-      print "\nTrying $host .... ";
-    
+    foreach $key (@order) {
+      print "\n$source{$key}{site} .... ";
+
       if ($lwp) {
-	$err = LWP::Simple::getstore("$source$windows/$latest_version", "$latest_version");
+	$err = LWP::Simple::getstore("$source{$key}{url}/$latest_version", "$latest_version");
 	$err -= 200;
       } else {
-	$ok = `lynx -dump -head $source$windows/$latest_version`;
-	$err = !($ok =~ s/HTTP.*?200 OK//);
+	$ok = `lynx -dump -head $source{$key}{url}/$latest_version`;
+	$err = !($ok =~ s/HTTP.*?200 //);
 
 	if (!$err) {
-	  $err = system("lynx -dump $source$windows/$latest_version > $latest_version");
+	  $err = system("lynx -dump $source{$key}{url}/$latest_version > $latest_version");
 	}
       }
 
-      last unless $err;
+      if ($err) {
+	print "failed!";
+      } else {
+	last;
+      }
 
     }
     
@@ -266,7 +304,7 @@ sub get_source_code {
   if ($err) {
     die "Cannot get $latest_version";
   } else {
-    print "ok\n";
+    print "ok!\n";
   }
 
   $latest_version;
@@ -299,6 +337,9 @@ sub install {
     $alias =~ s/.*\///g;
     
     $httpddir = `dirname $httpd`;
+    if ($confd) {
+      $httpddir = $confd;
+    }
     chomp $httpddir;
     $filename = "sql-ledger-httpd.conf";
 
@@ -309,7 +350,7 @@ sub install {
     }
 
     $directives = qq|
-Alias /$alias/ $absolutealias/
+Alias /$alias $absolutealias/
 <Directory $absolutealias>
   AllowOverride All
   AddHandler cgi-script .pl
@@ -337,17 +378,19 @@ This is a new installation.
       print qq|
 Webserver directives were written to $filename
       
-Copy $filename to $httpddir and add
+Copy $filename to $httpddir
 |;
 
-      print qq|
+      if (!$confd) {
+	print qq| and add
 # SQL-Ledger
 Include $httpddir/$filename
 
 to $httpd
-
-Don't forget to restart your webserver!
 |;
+      }
+
+      print qq| and restart your webserver!\n|;
 
       if (!$permset) {
 	print qq|
@@ -362,32 +405,35 @@ could not be set. Login as root and set permissions
 
     } else {
       
-      if (!(`grep "^# SQL-Ledger" $httpd`)) {
-
-	open(FH, ">>$httpd");
-
-	print FH qq|
-
-# SQL-Ledger
-Include $httpddir/$filename
-|;
-	close(FH);
-        
-        print qq|
+       print qq|
 Webserver directives were written to
 
   $httpddir/$filename
 |;
-      }
-    }
+     
+      if (!$confd) {
+	if (!(`grep "^# SQL-Ledger" $httpd`)) {
 
-    if (!$>) {
-      # send SIGHUP to httpd
-      if ($f = `find /var -type f -name 'httpd.pid'`) {
-	$pid = `cat $f`;
-	chomp $pid;
-	if ($pid) {
-	  system("kill -s HUP $pid");
+	  open(FH, ">>$httpd");
+
+	  print FH qq|
+
+# SQL-Ledger
+Include $httpddir/$filename
+|;
+	  close(FH);
+	  
+	}
+      }
+
+      if (!$>) {
+	# send SIGHUP to httpd
+	if ($f = `find /var -type f -name 'httpd.pid'`) {
+	  $pid = `cat $f`;
+	  chomp $pid;
+	  if ($pid) {
+	    system("kill -s HUP $pid");
+	  }
 	}
       }
     }
@@ -398,14 +444,17 @@ Webserver directives were written to
     if ($permset = ($) =~ getgrnam $webgroup)) {
       `chown -hR :$webgroup users templates css spool`;
       chmod 0771, 'users', 'templates', 'css', 'spool';
+      `chown :$webgroup sql-ledger.conf`;
     }
   } else {
     # root
     `chown -hR 0:0 *`;
     `chown -hR $webowner:$webgroup users templates css spool`;
     chmod 0771, 'users', 'templates', 'css', 'spool';
+    `chown $webowner:$webgroup sql-ledger.conf`;
   }
   
+  chmod 0644, 'sql-ledger.conf';
   unlink "sql-ledger.conf.default";
 
   &cleanup;
@@ -468,10 +517,6 @@ sub decompress {
         print "cleaning up ... ";
         `rm -rf sql-ledger`;
         print "done\n";
-
-        # replace shebang if this is windows
-        &shebang if $windows;
-	
       }
     }
   }
@@ -501,28 +546,3 @@ sub cleanup {
 sub remove_lockfile { unlink "$userspath/nologin" if (-f "$userspath/nologin") };
 
 
-sub shebang {
-
-  opendir DIR, ".";
-  @perlfiles = grep /\.pl/, readdir DIR;
-  closedir DIR;
-
-  foreach $file (@perlfiles) {
-    open FH, "+<$file";
-    
-    @file = <FH>;
-
-    seek(FH, 0, 0);
-    truncate(FH, 0);
-
-    $line = shift @file;
-
-    print FH "#!c:\\perl\\bin\\perl\n";
-    print FH @file;
-
-    close(FH);
-
-  }
-}
-
-
