@@ -31,14 +31,14 @@ INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('6','CLASSE 6 -- CHARGES - KOSTEN','H','E','','');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('612140','Internet','A','I','AP_amount:IC_expense','61');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('655000','Ecarts de conversion de devises','A','E','AR_paid:IC_sale:IC_expense','65');
-INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('411590','TVA ACHATS 6%','A','A','AP_tax:IC_taxpart:IC_taxservice:CT_tax','41');
-INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('411591','TVA ACHATS 21%','A','A','AP_tax:IC_taxpart:IC_taxservice:CT_tax','41');
+INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('411590','TVA ACHATS 6%','A','A','AP_tax:IC_taxpart:IC_taxservice','41');
+INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('411591','TVA ACHATS 21%','A','A','AP_tax:IC_taxpart:IC_taxservice','41');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('411','TVA A RECUPERER - TERUG TE VORDEREN BTW','H','I','','41');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('44','Dettes commerciales - Handelsschulden','H','L','','44');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('416','Crances diverses - Diverse vorderingen','A','A','','41');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('409','Rductions de valeur actes (-) - Geboekte waardevermindering','A','A','','40');
-INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('451541','TVA DUE VENTES 21%','A','L','AR_tax:IC_taxpart:IC_taxservice:CT_tax','451');
-INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('451540','TVA DUE VENTES 6%','A','L','AR_tax:IC_taxpart:IC_taxservice:CT_tax','451');
+INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('451541','TVA DUE VENTES 21%','A','L','AR_tax:IC_taxpart:IC_taxservice','451');
+INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('451540','TVA DUE VENTES 6%','A','L','AR_tax:IC_taxpart:IC_taxservice','451');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('451560','TVA DUE CO-CONTRACTANT','A','L','AR_tax','451');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('640','Charges fiscales de l''exploitation - Bedrijfsbelastingen','A','E','','64');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('713','Des Produits finis - In de voorraad gereed product','A','I','IC_sale','71');
@@ -105,7 +105,7 @@ INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('614100','Catalogues et imprimes','A','I','AP_amount:IC_expense','61');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('53','Dpts  terme - Termijndeposito''s','H','Q','','53');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('609400','de marchandises - van handelsgoederen','A','E','','609');
-INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('451501','TVA DUE ACHATS INCO 21%','A','L','AR_tax:IC_taxpart:IC_taxservice:CT_tax','451');
+INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('451501','TVA DUE ACHATS INCO 21%','A','L','AR_tax:IC_taxpart:IC_taxservice','451');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('612010','Chauffage','A','E','AP_amount:IC_expense','61');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('612410','Fournitures informatiques','A','E','AP_amount:IC_expense','61');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('613250','Honoraires scretariat social','A','E','AP_amount:IC_expense','61');
@@ -348,7 +348,7 @@ INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('451640','TVA  recuprer s/NDC mises','A','L','','45');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('454','O.N.S.S. - R.S.Z.','A','L','','45');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('511','Montants non-appels (-) - Niet opgevraagde bedragen aan. (-)','A','Q','','51');
-INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('640700','TVA non-deductible','A','E','AP_tax:IC_taxpart:IC_taxservice:CT_tax','64');
+INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('640700','TVA non-deductible','A','E','AP_tax:IC_taxpart:IC_taxservice','64');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('221010','Construction acquis. de l''exercice - Geb. aanschaffingsprijs dit boekj.','A','A','AP_amount:IC_sale','22');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('401','Effets  recevoir - Te innen wissels','A','A','','40');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('444','Factures  recevoir - Te ontvangen facturen','A','L','','44');
@@ -367,7 +367,7 @@ INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('611340','Entretiens & rparations locaux','A','E','AP_amount:IC_expense','61');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('612150','Frais postaux','A','E','AP_amount:IC_cogs:IC_expense','61');
 INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('200010','Fr. const. acquis. de l''exercice - Kost. opricht./kap.verh. dit boekjaar','A','A','AP_amount','20');
-INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('451500','TVA DUE ACHATS INCO 6%','A','A','AR_tax:IC_taxpart:IC_taxservice:CT_tax','451');
+INSERT INTO chart (accno,description,charttype,category,link,gifi_accno) VALUES ('451500','TVA DUE ACHATS INCO 6%','A','A','AR_tax:IC_taxpart:IC_taxservice','451');
 
 INSERT INTO tax (chart_id,rate,taxnumber) VALUES ((SELECT id FROM chart WHERE accno = '411590'),0.06,'BE ... ... ...');
 INSERT INTO tax (chart_id,rate,taxnumber) VALUES ((SELECT id FROM chart WHERE accno = '411591'),0.21,'BE ... ... ...');
