@@ -5,9 +5,12 @@ SELECT nextval ('id');
 CREATE SEQUENCE invoiceid;
 SELECT nextval ('invoiceid');
 --
-CREATE SEQUENCE orderitemsid MAXVALUE 100000 CYCLE;
+CREATE SEQUENCE orderitemsid;
 SELECT nextval ('orderitemsid');
 --
+CREATE SEQUENCE jcitemsid;
+SELECT nextval ('jcitemsid');
+--
 CREATE TABLE makemodel (
   parts_id int,
   make text,
@@ -31,7 +34,8 @@ CREATE TABLE chart (
   charttype char(1) DEFAULT 'A',
   category char(1),
   link text,
-  gifi_accno text
+  gifi_accno text,
+  contra bool DEFAULT 'f'
 );
 --
 CREATE TABLE gifi (
@@ -62,9 +66,10 @@ CREATE TABLE defaults (
   employeenumber text,
   partnumber text,
   customernumber text,
-  vendornumber text
+  vendornumber text,
+  glnumber text
 );
-INSERT INTO defaults (version) VALUES ('2.4.4');
+INSERT INTO defaults (version) VALUES ('2.6.4');
 --
 CREATE TABLE acc_trans (
   trans_id int,
@@ -75,7 +80,8 @@ CREATE TABLE acc_trans (
   cleared bool DEFAULT 'f',
   fx_transaction bool DEFAULT 'f',
   project_id int,
-  memo text
+  memo text,
+  invoice_id int
 );
 --
 CREATE TABLE invoice (
@@ -92,7 +98,8 @@ CREATE TABLE invoice (
   unit varchar(5),
   project_id int,
   deliverydate date,
-  serialnumber text
+  serialnumber text,
+  notes text
 );
 --
 CREATE TABLE customer (
@@ -155,7 +162,9 @@ CREATE TABLE parts (
   image text,
   drawing text,
   microfiche text,
-  partsgroup_id int
+  partsgroup_id int,
+  project_id int,
+  avgcost float
 );
 --
 CREATE TABLE assembly (
@@ -164,7 +173,7 @@ CREATE TABLE assembly (
   qty float,
   bom bool,
   adj bool
-);
+) WITH OIDS;
 --
 CREATE TABLE ar (
   id int DEFAULT nextval ( 'id' ),
@@ -189,7 +198,8 @@ CREATE TABLE ar (
   intnotes text,
   department_id int default 0,
   shipvia text,
-  language_code varchar(6)
+  language_code varchar(6),
+  ponumber text
 );
 --
 CREATE TABLE ap (
@@ -211,9 +221,12 @@ CREATE TABLE ap (
   till varchar(20),
   quonumber text,
   intnotes text,
-  department_id int default 0,
+  department_id int DEFAULT 0,
   shipvia text,
-  language_code varchar(6)
+  language_code varchar(6),
+  ponumber text,
+  shippingpoint text,
+  terms int2 DEFAULT 0
 );
 --
 CREATE TABLE partstax (
@@ -257,10 +270,13 @@ CREATE TABLE oe (
   intnotes text,
   department_id int default 0,
   shipvia text,
-  language_code varchar(6)
+  language_code varchar(6),
+  ponumber text,
+  terms int2 DEFAULT 0
 );
 --
 CREATE TABLE orderitems (
+  id int default nextval('orderitemsid'),
   trans_id int,
   parts_id int,
   description text,
@@ -272,8 +288,8 @@ CREATE TABLE orderitems (
   reqdate date,
   ship float4,
   serialnumber text,
-  id int default nextval('orderitemsid')
-);
+  notes text
+) WITH OIDS;
 --
 CREATE TABLE exchangerate (
   curr char(3),
@@ -361,7 +377,13 @@ CREATE TABLE vendor (
 CREATE TABLE project (
   id int default nextval('id'),
   projectnumber text,
-  description text
+  description text,
+  startdate date,
+  enddate date,
+  parts_id int,
+  production float default 0,
+  completed float default 0,
+  customer_id int
 );
 --
 CREATE TABLE partsgroup (
@@ -374,8 +396,7 @@ CREATE TABLE status (
   formname text,
   printed bool default 'f',
   emailed bool default 'f',
-  spoolfile text,
-  chart_id int
+  spoolfile text
 );
 --
 CREATE TABLE department (
@@ -412,12 +433,12 @@ CREATE TABLE warehouse (
 CREATE TABLE inventory (
   warehouse_id int,
   parts_id int,
-  oe_id int,
+  trans_id int,
   orderitems_id int,
   qty float4,
   shippingdate date,
   employee_id int
-);
+) WITH OIDS;
 --
 CREATE TABLE yearend (
   trans_id int,
@@ -470,3 +491,45 @@ CREATE TABLE translation (
   description text
 );
 --
+CREATE TABLE recurring (
+  id int,
+  reference text,
+  startdate date,
+  nextdate date,
+  enddate date,
+  repeat int2,
+  unit varchar(6),
+  howmany int,
+  payment bool default 'f'
+);
+--
+CREATE TABLE recurringemail (
+  id int,
+  formname text,
+  format text,
+  message text
+);
+--
+CREATE TABLE recurringprint (
+  id int,
+  formname text,
+  format text,
+  printer text
+);
+--
+CREATE TABLE jcitems (
+  id int default nextval('jcitemsid'),
+  project_id int,
+  parts_id int,
+  description text,
+  qty float4,
+  allocated float4,
+  sellprice float8,
+  fxsellprice float8,
+  serialnumber text,
+  checkedin timestamp with time zone,
+  checkedout timestamp with time zone,
+  employee_id int,
+  notes text
+);
+
